(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: BuOrdner.mod,v $
# $Author: lurk $
# $Date: 2001/03/13 15:04:12 $
# $Revision: 1.4 $
# Bemerkungen: 

IMPLEMENTATION MODULE BuOrdner  ADT BuchungenOrdner

                                --  Enthalten in Buchungen

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE BuOrdner;

(* ADT BuchungenOrdner *)

(* IMPORTE *)

FROM Bu          IMPORT TBu, TBuArt;
FROM BuenIMPORTE IMPORT TBuID, TKstID, TVtrID, TVtr;
FROM String      IMPORT TString;
FROM Stringliste IMPORT TStringliste;

IMPORT BuOrdnerSchablone;


(* Lokale Datenstruktur als Modulgedaechtnis *)

VAR VAR_BuOrdner     : TBuOrdner;


(*
SEMANTIK
Die Semantik wird mit Hilfe dreier lokaler Integervariablen kap, 
pos und maxkap spezifiziert. Die Variablen kap und pos beschreiben  
den aktuellen Zustand (aktuelle Position und Anzahl der enthaltenen
Elemente) und  maxkap > 0 die Maximalkapazitaet des Ordners, 
d.h. es gilt stets:
            0 <= kap <= maxkap   und    0 <= pos <= kap+1

*)


PROCEDURE       Create ();
(*
Vb:     keine
Erzeugt und initialisiert das Architektur-ADO BuOrdner.
Nb:     Architektur-ADO BuOrdner existiert und ist leer
*)

BEGIN
  VAR_BuOrdner := BuOrdnerSchablone.Create();
END Create;


PROCEDURE       Dispose ();
(*
Vb:     keine
Entfernt alle Realisierungs-ADOs, die in dem Architektur-ADO BuOrdner
enthalten sind.
Entfernt das Architektur-ADO BuOrdner aus dem System
Nb:     Architektur-ADO BuOrdner existiert nicht mehr
*)

BEGIN
  BuOrdnerSchablone.Dispose(VAR_BuOrdner);
END Dispose;




PROCEDURE       Insert(inoutBu : TBu; VAR outMessage : TString) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.Insert(VAR_BuOrdner, inoutBu, outMessage);
END Insert;



PROCEDURE       Search(inBuID : TBuID; VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.Search(VAR_BuOrdner, inBuID, outBu);
END Search;


PROCEDURE       Update(inBu : TBu; VAR outMessage : TString):BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.Update(VAR_BuOrdner, inBu, outMessage);
END Update;



PROCEDURE       BerGebGelderDerKst (inKstID : TKstID) : INTEGER; 
BEGIN
  RETURN BuOrdnerSchablone.BerGebGelderDerKst(VAR_BuOrdner, inKstID);
END BerGebGelderDerKst;




PROCEDURE       StorniereBuZuVtr (inVtrID : TVtrID ; VAR outMessage : TString
                                 ) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.StorniereBuZuVtr(VAR_BuOrdner, inVtrID,
                                            outMessage);
END StorniereBuZuVtr;





(* -------------------------------------------------------------------------- *)
(* Iteratoren ueber nicht stornierte Buchungen                                *)
(* -------------------------------------------------------------------------- *)


PROCEDURE       First (VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.First(VAR_BuOrdner, outBu);
END First;


PROCEDURE       Next (inoutBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.Next(VAR_BuOrdner, inoutBu);
END Next;


PROCEDURE       FirstBuZuKst (inKstID : TKstID; VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.FirstBuZuKst(VAR_BuOrdner, inKstID, outBu);
END FirstBuZuKst;


 
PROCEDURE       NextBuZuKst  (inKstID : TKstID; inoutBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.NextBuZuKst(VAR_BuOrdner, inKstID, inoutBu);
END NextBuZuKst;




PROCEDURE       FirstBuZuVtr (inVtrID : TVtrID; VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.FirstBuZuVtr(VAR_BuOrdner, inVtrID, outBu);
END FirstBuZuVtr;


PROCEDURE        NextBuZuVtr  (inVtrID : TVtrID; inoutBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.NextBuZuVtr(VAR_BuOrdner, inVtrID, inoutBu);
END NextBuZuVtr;




PROCEDURE       FirstBuMitBuArtZuVtr (inVtrID : TVtrID; inBuArt : TBuArt;
                                      VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN  BuOrdnerSchablone.FirstBuMitBuArtZuVtr(VAR_BuOrdner,
                                                 inVtrID, inBuArt, outBu);
END FirstBuMitBuArtZuVtr;


PROCEDURE        NextBuMitBuArtZuVtr  (inVtrID : TVtrID; inBuArt : TBuArt;
                                       inoutBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.NextBuMitBuArtZuVtr(VAR_BuOrdner, inVtrID,
                                               inBuArt, inoutBu);
END NextBuMitBuArtZuVtr;




(* -------------------------------------------------------------------------- *)
(* Iterator ueber alle Buchungen (einschl. der stornierten)                   *)
(* -------------------------------------------------------------------------- *)


PROCEDURE       FirstBuUndStorno (VAR outBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.FirstBuUndStorno(VAR_BuOrdner, outBu);
END FirstBuUndStorno;


PROCEDURE       NextBuUndStorno (inoutBu : TBu) : BOOLEAN;
BEGIN
  RETURN BuOrdnerSchablone.NextBuUndStorno(VAR_BuOrdner, inoutBu);
END NextBuUndStorno;





(*************************** StringOperationen ***************************)
 
                                         
PROCEDURE       GibBuenAlsStringliste(): TStringliste;
BEGIN
  RETURN BuOrdnerSchablone.GibBuenAlsStringliste (VAR_BuOrdner);
END GibBuenAlsStringliste;
 
                                         
PROCEDURE       GibAllesAlsStringliste(): TStringliste;
BEGIN
  RETURN BuOrdnerSchablone.GibAllesAlsStringliste (VAR_BuOrdner);
END GibAllesAlsStringliste;
 




(********************** Backup/Restore-Operationen **********************)



PROCEDURE       BackupInsert (inBu : TBu);
(*
Vb:     Bu.ExistiertBuID(inBu) = TRUE
Fuegt die Buchung in den Buchungen-Ordner ein.
Es wird kein Identifikator gesetzt, da das Objekt bereits mit einem
Identifikator versehen ist
Nb:    inBu existiert im Buchungen-Ordner
*)

BEGIN
  BuOrdnerSchablone.BackupInsert(VAR_BuOrdner, inBu);
END BackupInsert;




BEGIN

(* Architektur-ADO BuchungenOrdner (BuOrdner) erstellen *)

Create();


END BuOrdner.
