(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: BuenPersistent.mod,v $
# $Author: lurk $
# $Date: 2001/03/10 14:09:32 $
# $Revision: 1.2 $
# Bemerkungen: 

IMPLEMENTATION MODULE BuenPersistent   ADT BuenPersistent
                                       --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE BuenPersistent;

(* ADT BuenPersistent *)


FROM BuenIMPORTE IMPORT THandle, WriteLineInVDat, ReadLineInVDat, EofInVDat;

IMPORT String, Stringliste, Bu, BuOrdner;


CONST EndeKonstante = "=============================";
      RFehlerKonstante = " Buen Load, Read-Fehler";
      WFehlerKonstante = " Buen Save, Write-Fehler bei String=";


TYPE TString = String.TString;
     TStringliste = Stringliste.TStringliste;
     TBu = Bu.TBu;



PROCEDURE        Load(inHandle : THandle; VAR outMessage : TString) : BOOLEAN;
(*
Vb:     Datei existiert und ist zum Lesen geoeffnet
        und die Datei wurde mit Save erstellt
Ladet den Inhalt des Buchungen-Ordners mit dem Inhalt der Datei, auf die
inHandle verweist.
Nb:    (Rueckgabewert = TRUE  =>  Buchungen-Ordner ist geladen AND
                                  outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

(* Implementierung:
   Die Backup-Datei wird zeilenweise eingelesen. Eine Zeile repraesentiert
   ein Attribut eines Objekts Bu. Das Ende eines Objekts ist durch die
   EndeKonstante erkenntlich. Aus den Attribut-Zeilen wird eine Stringliste
   erstellt, mit der Prozedur
     CreateObjectFromStringliste
   ein Objekt Bu erzeugt und in den Buchungen-Ordner eingefuegt.
   Das Einfuegen erfolgt ohne Erzeugen einer PersID.
*)

  VAR rc         : BOOLEAN;
      rcOrdner   : BOOLEAN;
      lfdBu      : TBu;
      lfdString  : TString;
      EndeString : TString;
      FehlerString : TString;
      ObjektAlsStringliste : TStringliste;

BEGIN
   (* Zunaechst wird der vorhandene Ordner geloescht und
      ein neuer Ordner angelegt.
   *)
   BuOrdner.Dispose();
   BuOrdner.Create();

   rc := TRUE;
   EndeString := String.Create();
   String.Schreibe(EndeString, EndeKonstante);
   FehlerString := String.Create();
   String.Schreibe(FehlerString, RFehlerKonstante);

   ObjektAlsStringliste := Stringliste.Create();
   WHILE (rc AND NOT EofInVDat(inHandle)) DO
     rc := ReadLineInVDat(inHandle, lfdString, outMessage);
     IF rc THEN
        IF String.IsEqual(lfdString, EndeString) THEN
           (* Alle Attribute eines Objekts sind eingelesen *)
           (* Objekt erzeugen und in den Ordner einfuegen *)
           rc := Bu.CreateObjectFromStringliste(ObjektAlsStringliste, lfdBu,
                                                outMessage);
           IF rc THEN
              BuOrdner.BackupInsert(lfdBu);
              Bu.Dispose(lfdBu);
              (* Fuer das naechste Objekt neue Stringliste erzeugen *)
              Stringliste.Dispose(ObjektAlsStringliste);
              ObjektAlsStringliste := Stringliste.Create();
           END; (* IF CreateObjectFromStringliste OK *)
        ELSE
           (* Attribut zur Stringliste hinzufuegen *)
           Stringliste.Append(ObjektAlsStringliste, lfdString);
        END; (* IF EndeKonstante erreicht*)
        String.Dispose(lfdString);
     ELSE
        String.Anhaengen(FehlerString, outMessage);
     END (* IF Zeile einlesen OK *)
   END; (* WHILE Schleife ueber die Backup-Datei *)

   String.Dispose(EndeString);
   String.Dispose(FehlerString);
   IF ObjektAlsStringliste # NIL THEN
      Stringliste.Dispose(ObjektAlsStringliste);
   END; (* IF *)

   RETURN rc;

END Load;



PROCEDURE        Save(inHandle : THandle; VAR outMessage : TString) : BOOLEAN;
(*
Vb:     Datei existiert und ist zum Schreiben geoeffnet
Speichert den Inhalt des Buchungen-Ordner in der Datei, auf die inHandle
verweist.
Nb:    (Rueckgabewert = TRUE
        =>  Buchungen-Ordner-Inhalt ist auf SekundaerSpeicher geschrieben
            AND
            outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

(* Implementierung:
   Von jedem Element des Buchungen-Ordners wird mit der Prozedur
     CreateStringlisteFromObject
   eine Stringliste erzeugt. Die Stringliste enthaelt jedes Attribut des
   Objekts als String.
   Jeder String der Stringliste wird in eine Zeile der Datei geschrieben.
*)


  VAR rc         : BOOLEAN;
      rcOrdner   : BOOLEAN;
      lfdBu      : TBu;
      lfdString  : TString;
      EndeString : TString;
      FehlerString : TString;
      ObjektAlsStringliste : TStringliste;

BEGIN
   rc := TRUE;
   rcOrdner := BuOrdner.FirstBuUndStorno (lfdBu);

   IF rcOrdner THEN
      (* Abfrage erforderlich, da im FALSE-Fall DisposeBu(lfdBu) entfaellt *)
      EndeString := String.Create();
      String.Schreibe(EndeString, EndeKonstante);
      FehlerString := String.Create();
      String.Schreibe(FehlerString, WFehlerKonstante);

      WHILE (rc AND rcOrdner)  DO

         ObjektAlsStringliste := Bu.CreateStringlisteFromObject(lfdBu);
         IF NOT Stringliste.IsEmpty(ObjektAlsStringliste) THEN
            Stringliste.GoStart(ObjektAlsStringliste);

            WHILE (rc AND NOT Stringliste.OffEnd(ObjektAlsStringliste)) DO
               lfdString := Stringliste.Current(ObjektAlsStringliste);
               rc := WriteLineInVDat(inHandle, lfdString, outMessage);
               IF NOT rc THEN
                  String.Anhaengen(FehlerString, outMessage);
                  String.Anhaengen(lfdString, outMessage);
               END; (* IF WriteLineInVDat NOT OK *)
               String.Dispose(lfdString);
               Stringliste.StepNext(ObjektAlsStringliste);
            END; (* WHILE Schleife ueber die Elemente der Stringliste *)

            (* Zur besseren Uebersicht in der Datei wird jedes Objekt mit
               einer Endezeile abgeschlossen.
            *)
             IF rc THEN
                rc := WriteLineInVDat(inHandle, EndeString, outMessage);
                IF NOT rc THEN
                   String.Anhaengen(FehlerString, outMessage);
                   String.Anhaengen(EndeString, outMessage);
                END; (* IF WriteLineInVDat von EndeString NOT OK *)
             END; (* IF WriteLineInVDat der WHILE-Schleife OK *)

         END; (* IF ObjektAlsStringliste nicht leer *)

         Stringliste.Dispose(ObjektAlsStringliste);
       
         rcOrdner := BuOrdner.NextBuUndStorno(lfdBu);

      END; (* WHILE Schleife ueber die Elemente des Buchungen-Ordners *)

      Bu.Dispose(lfdBu);
      String.Dispose(EndeString);
      String.Dispose(FehlerString);

   END; (* IF First *)

   RETURN rc;

END Save;



END BuenPersistent.

