(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HKListe.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:58 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE HKListe   ADT SortierteHilfskraefteListe
                                Instantiierungs-Modul
                                        --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE HKListe;

(* ADT SortierteHilfskraefteListe *)

(* IMPORTE *)


  IMPORT HK;
  IMPORT SortierteObjektliste;

  TYPE THKListe =  SortierteObjektliste.TSortierteObjektliste;
       TObjektreferenz = SortierteObjektliste.TObjektreferenz;


(* SEMANTIK:

Die Semantik ist dem Modul "SortierteObjektliste.def" entnommen

Die Semantik wird auf Basis zweier lokaler Integervariablen len und pos,
die den Zustand der sortierten Objektliste beschreiben, und einer
lokalen Kapazitaetskonstanten maxlen spezifiziert: 
Sei len die Laenge der sortierten Objektliste, maxlen > 0 die Maximal-
kapazitaet der Liste, d.h. 0 <= len <= maxlen, und pos die Position 
des aktuellen Listenelements mit 0 <= pos <= len + 1.
Die Indizes alt und neu bezeichnen den Wert einer Zustandsvariablen vor
bzw. nach Ausfuehren einer Operation. Werden die neuen Zustaende der
Variablen nicht explizit angegeben, so sind sie invariant.
*)

PROCEDURE  Create () : THKListe;
(*
Vb:	keine
Generiert und initialisert ein ADO THKListe.
Nb:	Rueckgabe-ADO THKListe existiert AND IsEmpty(THKListe)
*)

  VAR outHKListe : THKListe;

BEGIN
  outHKListe := SortierteObjektliste.Create (DisposeHK,
                                             DuplicateHK,
                                             EqualizeHK,
                                             IsEqualHK,
                                             SmallerHK,
                                             SameHK);
  RETURN outHKListe;

END Create;


PROCEDURE Dispose (VAR inoutHKListe : THKListe);
(*
Vb:	IsEmpty (inoutHKListe : THKListe)
Entfernt das ADO inoutHKListe aus dem System.
Nb:	ADO inoutHKListe existiert nicht mehr
*)
BEGIN
  SortierteObjektliste.Dispose (inoutHKListe);
END Dispose;


PROCEDURE Current(inHKListe : THKListe) : THK;
(*
Vb:	NOT (OffStart (inHKListe) OR OffEnd (inHKListe))
Liefert das Objekt an der aktuellen Position.
Nb:	NOT (OffStart(inHKListe) OR OffEnd(inHKListe))
*)
BEGIN
  RETURN HK.HK (SortierteObjektliste.Current (inHKListe));
END Current;


PROCEDURE Update (inoutHKListe : THKListe;
                  inObjekt     : THK);
(*
Vb:	NOT (OffStart (inoutHKListe) OR OffEnd (inoutHKListe))
Ueberschreibt unter Beachtung der Ordnung die Attributwerte des 
Objekts an der aktuellen Position mit denen von inObjekt. 
Anschliessend sind das Objekt an der ggf. neuen aktuellen 
Position und inObjekt zustandsgleich
Nb:	NOT (OffStart (inoutHKListe) OR OffEnd (inoutHKListe)
*)
BEGIN
  SortierteObjektliste.Update (inoutHKListe, HK.OBJEKTREFERENZ (inObjekt));
END Update;


PROCEDURE Insert (inoutHKListe : THKListe;
                  inObjekt     : THK);
(*
Vb:	keine
Einfuegen eines Objekts bzgl. der Sortierreihenfolge
Anschliessend befindet sich inObjekt an der aktuellen Position 
Nb:	(1 <= inoutHKListe.pos_neu <= inoutHKListe.len) AND
              (inoutHKListe.len_neu = inoutHKListe.len_alt + 1) AND NOT 
              (IsEmpty (inoutHKListe) OR OffStart (inoutHKListe)
               OR OffEnd (inoutHKListe) )
*)

BEGIN
  SortierteObjektliste.Insert (inoutHKListe, HK.OBJEKTREFERENZ (inObjekt));
END Insert;


PROCEDURE Delete (inoutHKListe : THKListe);
(*
Vb:	NOT (OffStart (inoutHKListe) OR OffEnd (inoutHKListe))
Loeschen des Objekts an der aktuellen Position; das Listenelement rechts 
neben dem zu loeschenden Element befindet sich anschliessend an der 
(unveraenderten) aktuellen Position. 
Wird das letzte Objekt geloescht, steht die aktuelle Position auf OffEnd
Nb:     (inoutHKListe.len_neu = inoutHKListe.len_alt - 1) OR
         OffStart (inoutHKListe))
*)
BEGIN
  SortierteObjektliste.Delete (inoutHKListe);
END Delete;


PROCEDURE Search (inoutHKListe : THKListe;
                  inObjekt     : THK) : BOOLEAN;
(*
Vb:	keine
Suche nach inObjekt; wird ein zustandsgleiches Objekt gefunden, 
ist dort die neue aktuelle Position; sonst OffEnd
Sei found der boole'sche Rueckgabeparameter, dann gilt:
Nb: (found => NOT (OffStart (inoutHKListe) OR OffEnd (inoutHKListe))) AND
    (NOT found => OffEnd (inoutHKListe))
*)

BEGIN
  RETURN SortierteObjektliste.Search (inoutHKListe,
                                      HK.OBJEKTREFERENZ (inObjekt));
END Search;


PROCEDURE GoSmallest (inoutHKListe : THKListe);
(*
Vb:	keine
Die Position des kleinsten Objekts der Liste wird zur aktuellen Position
Nb:	inoutHKListe.pos_neu = 1
*)
BEGIN
  SortierteObjektliste.GoSmallest (inoutHKListe);
END GoSmallest;


PROCEDURE StopAt (inoutHKListe : THKListe;
                  inObjekt     : THK) : BOOLEAN;
(*
Vb:	keine
Suche nach einem bzgl. SameValue Objekt gleichen Objekt. Wird ein
solches Objekt gefunden, ist dort die neue aktuelle Position; 
Seien found der boole'sche Rueckgabeparameter und aktObjekt bzw.   
MaxObjekt die Objekte an pos_neu bzw. pos = len (falls diese 
existieren), dann gilt:
Nb:    (found => NOT (OffStart (inoutHKListe) OR OffEnd (inoutHKListe)) AND
                 Objekt.SameValue (inObjekt, aktObjekt)) AND
       (NOT found => (IsEmpty (inoutHKListe) AND OffEnd (inoutHKListe)) OR
                     (NOT IsEmpty (inoutHKListe) AND OffEnd (inoutHKListe) AND
                      Objekt.SmallerValue (MaxObjekt, inObjekt)) OR
                      (NOT (OffStart (inoutHKListe) OR
                            OffEnd (inoutHKListe)) AND
                       Objekt.SmallerValue (aktObjekt, inObjekt))
*)

BEGIN
  RETURN SortierteObjektliste.StopAt (inoutHKListe, HK.OBJEKTREFERENZ (inObjekt));
END StopAt;



PROCEDURE GoLargest (inoutHKListe : THKListe);
(*
Vb:	keine
Die Position des groessten Objekts der Liste wird zur aktuellen Position
Nb:	inoutHKListe.pos_neu = inoutHKListe.len
*)

BEGIN
  SortierteObjektliste.GoLargest (inoutHKListe);
END GoLargest;


PROCEDURE StepNext (inoutHKListe : THKListe);
(*
Vb:	NOT OffEnd (inoutHKListe)
Die Position hinter der aktuellen Position wird zur aktuellen Position
Nb:	(inoutHKListe.pos_neu = inoutHKListe.pos_alt + 1
         OR (IsEmpty (inoutHKListe) => OffEnd (inoutHKListe)
*)

BEGIN
  SortierteObjektliste.StepNext (inoutHKListe);
END StepNext;



PROCEDURE StepBack (inoutHKListe : THKListe);
(*
Vb:	NOT OffStart (inoutHKListe)
Die Position vor der aktuellen Position wird zur aktuellen Position
Nb:	inoutHKListe.pos_neu = inoutHKListe.pos_alt - 1) OR
        (IsEmpty (inoutHKListe) => OffStart (inoutHKListe)
*)

BEGIN
  SortierteObjektliste.StepBack (inoutHKListe);
END StepBack;


PROCEDURE IsEmpty (inHKListe : THKListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Liste leer ist:
IsEmpty (inHKListe) <=> (inHKListe.len = 0)
Nb:	IsEmpty (inHKListe) => OffStart (inHKListe) OR OffEnd (inHKListe)
*)

BEGIN
  RETURN SortierteObjektliste.IsEmpty (inHKListe);
END IsEmpty;


PROCEDURE OffStart (inHKListe : THKListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position vor dem ersten Objekt aktuell ist:
OffStart (inHKListe) <=> inHKListe.pos = 0
Nb:	keine
*)

BEGIN
  RETURN SortierteObjektliste.OffStart (inHKListe);
END OffStart;


PROCEDURE OffEnd (inHKListe : THKListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position hinter dem letzten Objekt aktuell ist:
OffEnd (inHKListe) <=> inHKListe.pos = inHKListe.len + 1
Nb:	Rueckgabe-ADO THKListe existiert AND IsEmpty(THKListe)
*)

BEGIN
  RETURN SortierteObjektliste.OffEnd (inHKListe);
END OffEnd;
 

(* ------------------------------------------------------------------
   -- Typkonvertierungen von THK und TObjektreferenz
   ------------------------------------------------------------------
*)

PROCEDURE DisposeHK (VAR inoutReferenz : TObjektreferenz);
 VAR aktHK : THK;
BEGIN
  aktHK := HK.HK (inoutReferenz);
  HK.Dispose (aktHK);
  inoutReferenz := HK.OBJEKTREFERENZ (aktHK);
END DisposeHK;

PROCEDURE DuplicateHK (inReferenz : TObjektreferenz) : TObjektreferenz;
  VAR Source      : THK;
      outDuplikat : THK;
BEGIN
  Source := HK.HK (inReferenz);
  outDuplikat := HK.Duplicate (Source);
  RETURN HK.OBJEKTREFERENZ (outDuplikat)
END DuplicateHK;

PROCEDURE EqualizeHK (inoutZiel, inQuelle : TObjektreferenz);
  VAR ZielHK  : THK;
      QuellHK : THK;
BEGIN
 ZielHK := HK.HK (inoutZiel);
 QuellHK := HK.HK (inQuelle);
 HK.Equalize (ZielHK, QuellHK)
END EqualizeHK;

PROCEDURE IsEqualHK (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR HK1 : THK;
      HK2 : THK;
BEGIN
  HK1 := HK.HK (inReferenz1);
  HK2 := HK.HK (inReferenz2);
  RETURN HK.IsEqual (HK1, HK2)
END IsEqualHK;

PROCEDURE SmallerHK (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR HK1 : THK;
      HK2 : THK;
BEGIN
  HK1 := HK.HK (inReferenz1);
  HK2 := HK.HK (inReferenz2);
  RETURN HK.SmallerHK (HK1, HK2)
END SmallerHK;

PROCEDURE SameHK (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR HK1 : THK;
      HK2 : THK;
BEGIN
  HK1 := HK.HK (inReferenz1);
  HK2 := HK.HK (inReferenz2);
  RETURN HK.SameHK (HK1, HK2)
END SameHK;

END HKListe.
