(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HKOrdnerSchablone.mod,v $
# $Author: lurk $
# $Date: 2001/03/13 10:44:05 $
# $Revision: 1.2 $
# Bemerkungen: 

IMPLEMENTATION MODULE HKOrdnerSchablone  ADT HilfskraefteOrdnerSchablone

                                         --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE HKOrdnerSchablone;

(* ADT HilfskraefteOrdnerSchablone *)

(* IMPORTE *)

FROM Storage     IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM      IMPORT TSIZE;
FROM WholeStr    IMPORT CardToStr;
FROM Datum       IMPORT TDatum, DisposeDatum, DatumAlsString, IstDatumEternity;
FROM HKeIMPORTE  IMPORT TPersID, DisposePersID, IsEqualPersID,
                        HoleNeuePersID, UeberlasseAltePersID,
                        PersIDAlsString;

IMPORT HK, HKListe, String, Stringliste;

CONST LaengePersIDNR = 5;

TYPE THK = HK.THK;
     TPraesTage = HK.TPraesTage;
     TString = String.TString;
     TStringliste = Stringliste.TStringliste;
     THKOrdner = HKListe.THKListe;



(*
SEMANTIK
Die Semantik wird mit Hilfe dreier lokaler Integervariablen kap, 
pos und maxkap spezifiziert. Die Variablen kap und pos beschreiben  
den aktuellen Zustand (aktuelle Position und Anzahl der enthaltenen
Elemente) und  maxkap > 0 die Maximalkapazitaet des Ordners, 
d.h. es gilt stets:
            0 <= kap <= maxkap   und    0 <= pos <= kap+1

*)


PROCEDURE        Create () : THKOrdner;
(*
Vb:       keine
Erzeugt und initialisiert ein ADO HKOrdner.
Nb:       Rueckgabe-ADO HKOrdner existiert und ist leer
*)

  VAR HKOrdner : THKOrdner;

BEGIN
  HKOrdner := HKListe.Create();

  RETURN HKOrdner;

END Create;


PROCEDURE      Dispose (VAR inoutHKOrdner : THKOrdner);
(*
Vb:      keine
Entfernt alle Realisierungs-ADOs, die in inoutHKOrdner enthalten sind.
Entfernt das ADO inoutHKOrdner aus dem System
Nb:      ADO HKOrdner existiert nicht mehr
*)

BEGIN
  HKListe.Dispose(inoutHKOrdner);

END Dispose;



PROCEDURE       IsEmpty (inHKOrdner : THKOrdner) : BOOLEAN;
(*
VB:      keine
Test, ob der Ordner leer ist
Nb:      IsEmpty (inHKOrdner) <=> kap = 0
*)
BEGIN

  RETURN HKListe.IsEmpty (inHKOrdner);

END IsEmpty;




PROCEDURE       Insert(inoutHKOrdner : THKOrdner; inoutHK: THK;
                       VAR outMessage : TString) : BOOLEAN;
(*
Vb:     NOT Search(inoutHKOrdner, HK.LiesPersID(inoutHK), x)
Fuegt die Hilfskraft in den Ordner ein und setzt den Identifikator der Hilfskraft
Nb:    (Rueckgabewert = TRUE
        => (kap_neu = kap_alt+1)  AND
           (Search(inoutHKOrdner, HK.LiesPersID(inoutHK), x) = TRUE)  AND
           outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR aktPersID : TPersID;
      rc        : BOOLEAN;

BEGIN
  IF NOT HK.ExistiertPersID(inoutHK) THEN
     aktPersID := HoleNeuePersID();
     HK.SetzePersID(inoutHK, aktPersID);
     HKListe.Insert(inoutHKOrdner,inoutHK);
     DisposePersID(aktPersID);
     rc := TRUE;
  ELSE
      outMessage := String.Create();
      String.Schreibe(outMessage, "PersID ist bereits gesetzt.");
      rc := FALSE;
  END; (* IF *)

  RETURN rc;

END Insert;


PROCEDURE       Delete(inoutHKOrdner : THKOrdner; inHK:THK;
                       VAR outMessage : TString):BOOLEAN;
(*
Vb:     Search(inoutHKOrdner, HK.LiesPersID(inoutHK), x)
Loescht die Hilfskraft mit PersID = HK.LiesPersID(inoutHKOrdner, inHK)
aus dem Ordner
Nb:    (Rueckgabewert = TRUE
        => (kap_neu = kap_alt+1)                 AND
           (Search(inoutHKOrdner, HK.LiesPersID(inHK), x) = TRUE)  AND
           outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR aktPersID : TPersID;
      rc        : BOOLEAN;

BEGIN
  IF HKListe.StopAt(inoutHKOrdner, inHK) THEN
     HKListe.Delete(inoutHKOrdner);
     aktPersID := HK.LiesPersID(inHK);
     UeberlasseAltePersID(aktPersID);
     DisposePersID(aktPersID);
     rc := TRUE;
  ELSE
     outMessage := String.Create();
     String.Schreibe(outMessage,
                     "Hilfskraft ist nicht im Hilfskraefte-Ordner.");
     rc := FALSE;

  END; (* IF *)

  RETURN rc;

END Delete;


PROCEDURE       Search(inoutHKOrdner : THKOrdner;
                       inPersID : TPersID; VAR outHK : THK) : BOOLEAN;
(*
Vb:      keine
Sucht die Hilfskraft mit der uebergebenen PersID; wird die Hilfskraft gefunden,
so wird sie in outHK bereitgestellt.
Nb:    (Rueckgabewert = TRUE
        <=> HK.LiesPersID(outHK) = inPersID 
*)

  VAR aktPersID : TPersID;
      rc        : BOOLEAN;
      lfdHK     : THK;
      lfdPersID : TPersID;

BEGIN
  rc := FALSE;
  IF NOT IsEmpty (inoutHKOrdner) THEN  
     HKListe.GoSmallest (inoutHKOrdner);
     WHILE NOT (rc OR HKListe.OffEnd (inoutHKOrdner)) DO
       lfdHK := HKListe.Current (inoutHKOrdner);
       lfdPersID := HK.LiesPersID (lfdHK);
       IF IsEqualPersID (lfdPersID, inPersID) THEN
         rc := TRUE;
         outHK := HK.Duplicate (lfdHK)  
       ELSE 
         HKListe.StepNext (inoutHKOrdner)
       END; (* IF *)
       DisposePersID (lfdPersID);
       HK.Dispose (lfdHK)
     END; (* WHILE nicht gefunden*)
  END; (* IF Ordner nicht leer*)

  RETURN rc
   
END Search;


PROCEDURE       Update(inoutHKOrdner : THKOrdner; inHK : THK;
                       VAR outMessage : TString):BOOLEAN;
(*
Vb:     Search(inoutHKOrdner, HK.LiesPersID(inoutHK), x)
Aktualisiert die Daten der Hilfskraft inHK im Ordner
Nb:    (Rueckgabewert = TRUE
        => Search(inoutHKOrdner, HK.LiesPersID(inHK), x)  AND
           outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR  rc : BOOLEAN;

BEGIN
  IF HKListe.StopAt(inoutHKOrdner, inHK) THEN
     HKListe.Update(inoutHKOrdner, inHK);
     rc := TRUE;
  ELSE
     outMessage := String.Create();
     String.Schreibe(outMessage,
                     "Hilfskraft ist nicht im Hilfskraefte-Ordner.");
     rc := FALSE;

  END; (* IF *)

  RETURN rc;

END Update;


PROCEDURE       First (inoutHKOrdner : THKOrdner; VAR outHK : THK) : BOOLEAN;
(*
Vb:     keine
Die erste Hilfskraft (auf pos = 1) wird in outHK bereitgestellt
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = 1) AND (outHK enthaelt die 1. Hilfskraft des Ordners)
*)

  VAR  rc : BOOLEAN;

BEGIN

  IF NOT IsEmpty(inoutHKOrdner) THEN
    HKListe.GoSmallest(inoutHKOrdner);
    outHK := HKListe.Current(inoutHKOrdner);
    rc := TRUE;
  ELSE
    rc := FALSE;
  END; (* IF *)

  RETURN rc;

END First;


PROCEDURE       Next(inoutHKOrdner : THKOrdner; inoutHK : THK) : BOOLEAN;
(*
Vb:     Search(inoutHKOrdner, HK.LiesPersID(inoutHK), x)
Rueckgabe der Hilfskraft an der naechsten Position (pos_alt + 1, falls existent)
Nb:     (Rueckgabewert = TRUE
         <=> (pos_neu = pos_alt + 1) AND (inoutHK enthaelt die naechste
                                          Hilfskraft des Ordners)
*)

  VAR aktPersID : TPersID;
      rc        : BOOLEAN;
      lfdHK     : THK;

BEGIN
    rc := FALSE;

    IF HKListe.StopAt (inoutHKOrdner, inoutHK) THEN
      HKListe.StepNext (inoutHKOrdner);
      IF NOT HKListe.OffEnd (inoutHKOrdner) THEN
        lfdHK := HKListe.Current (inoutHKOrdner);
        HK.Equalize (inoutHK, lfdHK);
        HK.Dispose (lfdHK);
        rc := TRUE
      END (* IF NOT HKListe.OffEnd *)
    END; (* IF HKListe.StopAt *)

    RETURN rc;

END Next;


(*************************** StringOperationen ***************************)
 
                                         
PROCEDURE GibHKeAlsStringliste(inoutHKOrdner : THKOrdner): TStringliste;
(* Gib die Hilfskrfte in sortierter Reihenfolge (nach Familienname) mit
   folgender Formatierung als Stringliste aus;
   Formatierung: PersId (5Zeichen), 1 Leerzeichen, Familienname(25Zeichen),
   1 Leerzeichen, Vorname (15Zeichen), 1 Leerzeichen, Wohnort (10Zeichen)
   insgesamt also 58Zeichen.
*)

  VAR HKeStringliste : TStringliste;
      HKString       : TString;
      lfdHK          : THK;


      (* -------------------- lokale Prozedur ------------------------------ *)

      PROCEDURE GenerateString(inHK : THK; inoutHKString : TString);
                (* Erstellt aus den Attributen PersID, Name, Vorname, Wohnort
                   aus inHK einen String (inoutHKString)
                *)
        VAR lfdPersID    : TPersID;
            lfdString    : TString;
            fixedString  : TString;
            Blank        : TString;
            LaengeString : CARDINAL;
            strArray   : ARRAY [0..255] OF CHAR;
            i : CARDINAL;

        (* ------------- lokale Prozedur in lokaler Prozedur --------------- *)

        PROCEDURE GenerateFixedString(inString    : TString;
                                      inFixed     : CARDINAL;
                                      inoutString : TString);
                  (* Uebertraegt inString linksbuending nach
                     inoutString mit fester Stringlaenge inFixed
                  *)
          VAR strLaenge : CARDINAL;
              j         : CARDINAL;

        BEGIN
          strLaenge := String.Laenge(inString);
          IF strLaenge < inFixed THEN
             String.Auslesen(inString, 0, strLaenge, inoutString);
             FOR j := 1 TO inFixed - strLaenge DO
                 String.Anhaengen(Blank, inoutString);
             END; (* FOR *)
          ELSE
             (* inFixed Stellen von inString uebertragen *)
             String.Auslesen(lfdString, 0, inFixed, inoutString);
          END; (* IF *)
        END GenerateFixedString;

        (* ---------Ende  lokale Prozedur in lokaler Prozedur --------------- *)


      BEGIN                                 (* ProzedurKoerper GenerateString *)
        fixedString := String.Create();
        Blank := String.Create();
        String.Schreibe(Blank," ");

        (* 5 Stellen PersID *)
        lfdPersID := HK.LiesPersID(inHK);
        lfdString := PersIDAlsString(lfdPersID, LaengePersIDNR);
        DisposePersID(lfdPersID);
        LaengeString := String.Laenge(lfdString);
        IF LaengeString < 5 THEN
           FOR i := 1 TO 5 - LaengeString DO
               String.Anhaengen(Blank, inoutHKString);
           END; (* FOR *)
        END; (* IF LaengePersID < 5 *)
        String.Anhaengen(lfdString, inoutHKString);
        String.Anhaengen(Blank, inoutHKString);
        String.Dispose(lfdString);

        (* 25 Stellen Nachname *)
        lfdString := HK.LiesName(inHK);
        GenerateFixedString(lfdString, 25, fixedString);
        String.Anhaengen(fixedString, inoutHKString);
        String.Anhaengen(Blank, inoutHKString);
        String.Dispose(lfdString);

        (* 15 Stellen Vorname *)
        lfdString := HK.LiesVorname(inHK);
        GenerateFixedString(lfdString, 15, fixedString);
        String.Anhaengen(fixedString, inoutHKString);
        String.Anhaengen(Blank, inoutHKString);
        String.Dispose(lfdString);

        (* 10 Stellen Wohnort *)
        lfdString := HK.LiesStadt(inHK);
        GenerateFixedString(lfdString, 10, fixedString);
        String.Anhaengen(fixedString, inoutHKString);
        String.Dispose(lfdString);

        String.Dispose(Blank);
        String.Dispose(fixedString);

      END GenerateString;

      (* -------------------- Ende lokale Prozedur ------------------------- *)


BEGIN                       (* ProzedurKoerper GibHKeAlsStringliste *)
  HKeStringliste := Stringliste.Create();

  IF First (inoutHKOrdner, lfdHK) THEN
     HKString := String.Create();
     GenerateString(lfdHK, HKString);
     Stringliste.Append(HKeStringliste, HKString);
     String.Loeschen(HKString, 0, String.Laenge(HKString));
       
     WHILE Next(inoutHKOrdner, lfdHK) DO
        GenerateString(lfdHK, HKString);
        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));
     END; (* WHILE *)

     HK.Dispose(lfdHK);
     String.Dispose(HKString);

  END; (* IF First *)

  RETURN HKeStringliste;
 

END GibHKeAlsStringliste;



(* -------------------------------------------------------------------------- *)

PROCEDURE GibAllesAlsStringliste(inoutHKOrdner : THKOrdner): TStringliste;

CONST ANZAHL_ATTRIBUTE = 25;
      ZEILENLAENGE = 80;

TYPE TName      = ARRAY [0..79] OF CHAR;
     TAttribut  = RECORD
                    AttributName : TName;
                    WertLaenge   : CARDINAL;
                                   (* einschl. nachfolgender Zwischenraeume, da
                                      alle Werte linksbuendig eingetragen
                                      werden
                                   *)
                  END;

     TAttributTab = ARRAY [1..ANZAHL_ATTRIBUTE] OF TAttribut;

     TZeile = ARRAY [1..ZEILENLAENGE] OF CHAR;

CONST AttributTab = TAttributTab
      {
       {"PersID: ",8}, {"Name: ", 58},
       {"PersonalNr: ", 11}, {"GebDatum: ", 11},
                                      {"Staatsang: ", 11}, {"Geschlecht: ", 1},
       {"Privatadresse:                         | Dienstadresse:", 0},
       {"", 39}, {"| ", 39},
       {"", 39}, {"| ", 39},
       {"", 39}, {"| ", 39},
       {"Tel: ", 15}, {"Fax: ", 14}, {"| Tel: ", 15}, {"Fax: ", 14},
       {"EMail: ", 32}, {"| EMail: ", 39},
       {"Internes:", 0},
       {"Vorgeseh. StdZahl: ", 4}, {"Verfgbar ab: ", 12},
                                                       {"Prsenztage: ", 20},
       {"Abschluss: ", 12}, {"Interessen: ", 35}
      };

      EndeZeile = TZeile{"=" BY ZEILENLAENGE};
      INITVALUECARDINAL = 99999999;

VAR lfdHK : THK;
    HKString : TString;
    HKeStringliste : TStringliste;
    lfdPersID    : TPersID;
    lfdString    : TString;
    lfdString1   : TString;
    zwString     : TString;  (* String zum Zwischenspeichern *)
    lfdDatum     : TDatum;
    lfdPraesTage : TPraesTage;
    LaengeString : CARDINAL;
    strArray     : ARRAY [0..255] OF CHAR;
    charStr      : ARRAY [0..0] OF CHAR;
    Komma        : TString;
    Blank        : TString;
    index        : CARDINAL; (* Index in der Attribut-Tabelle *)
    CardinalZahl : CARDINAL;
    rc : BOOLEAN;



BEGIN                       (* ProzedurKoerper GibAllesAlsStringliste *)

  HKeStringliste := Stringliste.Create();
  rc := TRUE;

  IF First (inoutHKOrdner, lfdHK) THEN
     HKString       := String.Create();
     Komma := String.Create();
     String.Schreibe(Komma, ", ");
     Blank := String.Create();
     String.Schreibe(Blank, " ");
     zwString := String.Create();
       
     WHILE rc DO

        index := 1;  (* Index in der Attribut-Tabelle *)
        (* Zu Beginn eine Leerzeile *)
        String.Loeschen(HKString, 0, String.Laenge(HKString));
        Stringliste.Append(HKeStringliste, HKString);

        (* PersID *)
        lfdPersID := HK.LiesPersID(lfdHK);
        lfdString := PersIDAlsString(lfdPersID, LaengePersIDNR);
        DisposePersID(lfdPersID);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        (* Nachname, Vorname *)
        lfdString := HK.LiesName(lfdHK);
        lfdString1 := HK.LiesVorname(lfdHK);
        IF ((String.Laenge(lfdString) > 0)  AND
            (String.Laenge(lfdString1) > 0)) THEN
            String.Anhaengen(Komma, lfdString);
        END;
        String.Anhaengen(lfdString1, lfdString);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        String.Dispose(lfdString1);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* PersonalNr *)
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        CardinalZahl := HK.LiesPersonalnr(lfdHK);
        IF CardinalZahl # INITVALUECARDINAL THEN
           CardToStr(CardinalZahl, strArray);
           String.Schreibe(zwString, strArray);
        ELSE
           String.Loeschen(zwString, 0, String.Laenge(zwString));
        END;
        ChangeStringToFixedString(zwString, AttributTab[index].WertLaenge);
        String.Anhaengen(zwString, HKString);
        index := index + 1;

        (* Geburtsdatum *)
        lfdDatum := HK.LiesGeburtsdatum(lfdHK);
        lfdString := DatumAlsString(lfdDatum);
        IF IstDatumEternity(lfdDatum)  THEN
           String.Loeschen(lfdString, 0, String.Laenge(lfdString));
        END; (* IF Datum undefiniert *)
        ChangeStringToFixedString(lfdString,AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        DisposeDatum(lfdDatum);
        index := index + 1;

        (* Staatsangehoerigkeit *)
        lfdString := HK.LiesStaatsang(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        (* Geschlecht *)
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        charStr[0] := HK.LiesGeschlecht(lfdHK);
        String.Schreibe(zwString, charStr);
        ChangeStringToFixedString(zwString, AttributTab[index].WertLaenge);
        String.Anhaengen(zwString, HKString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Leerzeile *)
        Stringliste.Append(HKeStringliste, HKString);


        (* PrivatAdresse, DienstAdresse *)
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Firmenname von DienstAdresse, der Teil bei PrivatAdresse ist leer *)
        String.Loeschen(zwString, 0, String.Laenge(zwString));
        ChangeStringToFixedString(zwString, AttributTab[index].WertLaenge);
        String.Anhaengen(zwString, HKString);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        index := index + 1;
        lfdString := HK.LiesDienstName(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));


        (* Strasse von PrivatAdresse, DienstAdresse *)
        lfdString := HK.LiesStrasse(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;
        lfdString := HK.LiesDienstStr(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Plz, Ort von PrivatAdresse, DienstAdresse *)
        lfdString  := HK.LiesPostleitzahl(lfdHK);
        lfdString1 := HK.LiesStadt(lfdHK);
        String.Anhaengen(Blank, lfdString);
        String.Anhaengen(lfdString1, lfdString);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        String.Dispose(lfdString1);
        index := index + 1;
        lfdString := HK.LiesDienstPlz(lfdHK);
        lfdString1 := HK.LiesDienstStadt(lfdHK);
        String.Anhaengen(Blank, lfdString);
        String.Anhaengen(lfdString1, lfdString);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        String.Dispose(lfdString1);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));


        (* Tel, Fax von PrivatAdresse, DienstAdresse *)
        lfdString  := HK.LiesTelefon(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        lfdString  := HK.LiesFax(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        lfdString  := HK.LiesDienstTel(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        lfdString  := HK.LiesDienstFax(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));


        lfdString  := HK.LieseMail(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        lfdString  := HK.LiesDiensteMail(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Internes *)
        (* Leerzeile *)
        Stringliste.Append(HKeStringliste, HKString);

        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Vorgesehene Stundenzahl *)
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        CardinalZahl := HK.LiesVorStdzahl(lfdHK);
        IF CardinalZahl # INITVALUECARDINAL THEN
           CardToStr(CardinalZahl, strArray);
           String.Schreibe(zwString, strArray);
        ELSE
           String.Loeschen(zwString, 0, String.Laenge(zwString));
        END;
        ChangeStringToFixedString(zwString, AttributTab[index].WertLaenge);
        String.Anhaengen(zwString, HKString);
        index := index + 1;

        (* VerfuegbarAb *)
        lfdDatum  := HK.LiesVerfuegAb(lfdHK);
        lfdString := DatumAlsString(lfdDatum);
        IF IstDatumEternity(lfdDatum)  THEN
           String.Loeschen(lfdString, 0, String.Laenge(lfdString));
        END; (* IF Datum undefiniert *)
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        DisposeDatum(lfdDatum);
        index := index + 1;

        (* Praesenztage *)
        lfdPraesTage  := HK.LiesPraestage(lfdHK);
        lfdString := HK.PraesTageAlsString(lfdPraesTage);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        HK.DisposePraesTage(lfdPraesTage);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));

        (* Abschluss *)
        lfdString  := HK.LiesAbschluss(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        (* Interessen *)
        lfdString  := HK.LiesInteressen(lfdHK);
        ChangeStringToFixedString(lfdString, AttributTab[index].WertLaenge);
        String.Schreibe(zwString,AttributTab[index].AttributName);
        String.Anhaengen(zwString, HKString);
        String.Anhaengen(lfdString, HKString);
        String.Dispose(lfdString);
        index := index + 1;

        Stringliste.Append(HKeStringliste, HKString);
        String.Loeschen(HKString, 0, String.Laenge(HKString));


        (* Zur bessseren Uebersicht Endestring hinzufuegen *)
        String.Schreibe(zwString, EndeZeile);
        Stringliste.Append(HKeStringliste, zwString);

        rc := Next(inoutHKOrdner, lfdHK);

     END; (* WHILE SCHLEIFE ueber den Hilfskraefte-Ordner*)

     HK.Dispose(lfdHK);
     String.Dispose(HKString);
     String.Dispose(Komma);
     String.Dispose(Blank);
     String.Dispose(zwString);

  END; (* IF First *)

  RETURN HKeStringliste;
 

END GibAllesAlsStringliste;


(* -------------------------------------------------------------------------- *)

PROCEDURE ChangeStringToFixedString(inoutString : TString;
                                    inFixed     : CARDINAL);
          (* Setzt inoutString auf die feste Stringlaenge inFixed *)

  VAR strLaenge : CARDINAL;
      j         : CARDINAL;
      Blank     : TString;

BEGIN
  Blank := String.Create();
  String.Schreibe(Blank, " ");
  strLaenge := String.Laenge(inoutString);
  IF strLaenge < inFixed THEN
     FOR j := 1 TO inFixed - strLaenge DO
         String.Anhaengen(Blank, inoutString);
     END; (* FOR *)
  ELSE
     (* String auf die Laenge inFixed verkuerzen *)
      String.Loeschen(inoutString, inFixed, strLaenge - inFixed);
  END; (* IF *)
  String.Dispose(Blank);

END ChangeStringToFixedString;

(* -------------------------------------------------------------------------- *)

 

(********************** Backup/Restore-Operationen **********************)


PROCEDURE       BackupInsert(inoutHKOrdner : THKOrdner; inHK: THK);
(*
Vb:     HK.ExistiertPersID(inHK) = TRUE
Fuegt die Hilfskraft in den BackupOrdner ein.
Es wird kein Identifikator gesetzt, da das Objekt bereits mit einem
Identifikator versehen ist
Nb:    inHK existiert im BackupOrdner
*)

BEGIN
  HKListe.Insert(inoutHKOrdner, inHK);
END BackupInsert;


END HKOrdnerSchablone.
