(*345678901234567890123456789012345678901234567890123456789012345678901234567890
        10        20        30        40        50         60       70        80
*)
(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Wochentage.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:47:58 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

IMPLEMENTATION MODULE Wochentage   ADT Wochentage
                                       --  Enthalten in Hilfskraefte

Sigrid Preuss

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE Wochentage;

(* ADT Wochentage *)

FROM Storage	IMPORT ALLOCATE,DEALLOCATE;
FROM SYSTEM	IMPORT TSIZE;

IMPORT String;

TYPE TWochentag   = (Mo, Di, Mi, Do, Fr, Sa ,So);
     TMWochentage = SET OF TWochentag;
     TWochentage  = POINTER TO TMWochentage;

     TString = String.TString;


(*
SEMANTIK
Die Basis-Operationen Dispose, Duplicate, IsEqual, Equalize
besitzen die uebliche Semantik.
*)

(* Basis-Operationen *)

PROCEDURE       Create () : TWochentage;
(*
Vb:     keine
Generiert ein ADO Wochentage und initialisiert es als leere Menge

Nb:     Rueckgabe-ADO Wochentage existiert
*)


  VAR Wochentage : TWochentage;

BEGIN

  ALLOCATE (Wochentage, TSIZE(TMWochentage));

  Wochentage^ := TMWochentage{};  (* Initialisierung mit leerer Menge *)

  RETURN Wochentage;

END Create;


PROCEDURE       Dispose (VAR inoutWochentage : TWochentage);

BEGIN

  DEALLOCATE (inoutWochentage, TSIZE(TMWochentage));
  inoutWochentage := NIL;

END Dispose;


PROCEDURE       Duplicate (inWochentage : TWochentage) : TWochentage;

  VAR Duplikat : TWochentage;

BEGIN
  Duplikat := Create();
  Equalize(Duplikat,inWochentage);
  RETURN Duplikat;
END Duplicate;


PROCEDURE       IsEqual (inWochentage1, inWochentage2 : TWochentage) : BOOLEAN;


BEGIN

  RETURN (inWochentage1^ = inWochentage2^);

END IsEqual;


PROCEDURE       Equalize (inoutZielWochentage : TWochentage;
                          inQuelleWochentage : TWochentage);
BEGIN
 inoutZielWochentage^ := inQuelleWochentage^;
END Equalize;




PROCEDURE       PruefeWochentage (inStringTage   : TString;
                                  VAR outMessage : TString) : BOOLEAN;
(*
Vb:    keine
Pruefen, ob inStringTage korrekt ist
Jeder Wochentag wird durch 2 Zeichen repraesentiert und die Wochentage
werden durch Komma getrennt,
z.B. "Mo,Mi,Sa" oder "Mo,Di,Mi,Do,Fr,Sa,So"
Ein Leerstring repraesentiert die leere Menge der Wochentage.
Nb:    (Rueckgabewert = TRUE
        => IsEqual(inStringTage,
                   WochentageAlsString(StringInWochentage(inStringTage)))
           AND
              outMessage undefiniert)
        AND
        (Rueckgabewert = FALSE
         => outMessage enthaelt eine Fehlermeldung)
*)

  VAR strArray   : ARRAY [0..255] OF CHAR;
      i  : CARDINAL;
      rc : BOOLEAN;

BEGIN
  String.Lies(inStringTage, strArray);
  i := 0;
  rc := TRUE;
  WHILE rc AND (i < LENGTH(strArray)) DO
    CASE strArray[i] OF

   'M','D' : IF (strArray[i+1] # 'o')  AND (strArray[i+1] # 'i') THEN
                (* NOT Mo, Di, Mi, Do*) 
                rc := FALSE;
             END; (* IF ueber das 2.Zeichen des Tages *)

      |'S' : IF (strArray[i+1] # 'a')  AND (strArray[i+1] # 'o') THEN
                (* NOT Sa, So *) 
                rc := FALSE;
             END; (* IF ueber das 2.Zeichen des Tages *)

      |'F' : IF strArray[i+1] # 'r' THEN
                (* NOT Fr *) 
                rc := FALSE;
             END; (* IF ueber das 2.Zeichen des Tages *)
    ELSE
      rc := FALSE;

    END; (* CASE ueber das 1.Zeichen des Tages *)

    i := i + 3;

  END; (* WHILE Schleife ueber den Eingabe-String inStringTage *)

  IF NOT rc THEN
     outMessage := String.Create();
     String.Schreibe(outMessage,
                    "Wochentags-Angaben falsch, korrektes Beispiel: Mo,Mi,Fr");
  END; (* IF *)

  RETURN rc;

END PruefeWochentage;


PROCEDURE       WochentageAlsString (inWochentage : TWochentage) : TString;
(*
Vb:     keine
Konvertiert inWochentage in einen String
Jeder Wochentag wird durch 2 Zeichen repraesentiert und die Wochentage
werden durch Komma getrennt,
z.B. "Mo,Mi,Sa" oder "Mo,Di,Mi,Do,Fr,Sa,So"
Ein Leerstring repraesentiert die leere Menge der Wochentage.
Nb:    IsEqual(inWochentage, StringInWochentage(Rueckgabe-String))
*)

  VAR wochentag     : TWochentag;
      strWochentage : TString;
      tabWochentage : TString;
      zwString      : TString;
      stringLaenge  : CARDINAL;
      startIndex    : CARDINAL;

BEGIN
  strWochentage := String.Create();
  zwString := String.Create();
  tabWochentage := String.Create();
  String.Schreibe(tabWochentage, "Mo,Di,Mi,Do,Fr,Sa,So,");
  FOR wochentag := Mo TO So DO
    IF wochentag IN inWochentage^ THEN
       (* Wochentag nach strWochentage uebertragen *)
       startIndex := ORD(wochentag) * 3;
       String.Auslesen(tabWochentage, startIndex, 3, zwString);
       String.Anhaengen(zwString, strWochentage);
    END; (* IF *)
  END; (* FOR Schleife ueber alle Wochentage *)
  stringLaenge := String.Laenge(strWochentage);
  IF stringLaenge > 1 THEN
     (* Letztes Komma entfernen *)
     String.Loeschen(strWochentage, stringLaenge - 1, 1);
  END; (* IF *)

  String.Dispose(zwString);
  String.Dispose(tabWochentage);

  RETURN strWochentage;

END WochentageAlsString;



PROCEDURE       StringInWochentage (inStringTage : TString) : TWochentage;
(*
Vb:    PruefeWochentage(inStringTage) = TRUE
Konvertiert inStringTage in ein ADO Wochentage
Jeder Wochentag wird durch 2 Zeichen repraesentiert und die Wochentage
werden durch Komma getrennt,
z.B. "Mo,Mi,Sa" oder "Mo,Di,Mi,Do,Fr,Sa,So"
Ein Leerstring repraesentiert die leere Menge der Wochentage.
Nb:    IsEqual(inStringTage, WochentageAlsString(Rueckgabe-ADO Wochentage))
*)

  VAR Wochentage : TWochentage;
      strArray   : ARRAY [0..255] OF CHAR;
      i  : CARDINAL;
      rc : BOOLEAN;

BEGIN
  Wochentage := Create();
  String.Lies(inStringTage, strArray);
  i := 0;
  rc := TRUE;
  WHILE rc AND (i < LENGTH(strArray)) DO
    CASE strArray[i] OF
       'M' : CASE strArray[i+1] OF
                'o' : (* Mo *) INCL(Wochentage^,Mo);
               |'i' : (* Mi *) INCL(Wochentage^,Mi);
             ELSE
               rc := FALSE;
             END; (* CASE ueber das 2.Zeichen des Tages *)

      |'D' : CASE strArray[i+1] OF
                'i' : (* Di *) INCL(Wochentage^,Di);
               |'o' : (* Do *) INCL(Wochentage^,Do);
             ELSE
               rc := FALSE;
             END; (* CASE ueber das 2.Zeichen des Tages *)

      |'S' : CASE strArray[i+1] OF
                'a' : (* Sa *) INCL(Wochentage^,Sa);
               |'o' : (* So *) INCL(Wochentage^,So);
             ELSE
               rc := FALSE;
             END; (* CASE ueber das 2.Zeichen des Tages *)

      |'F' : IF strArray[i+1] = 'r' THEN
                      (* Fr *) INCL(Wochentage^,Fr);
             ELSE
               rc := FALSE;
             END; (* IF ueber das 2.Zeichen des Tages *)
    ELSE
      rc := FALSE;

    END; (* CASE ueber das 1.Zeichen des Tages *)

    i := i + 3;

  END; (* WHILE Schleife ueber den Eingabe-String inStringTage *)

  RETURN Wochentage;

END StringInWochentage;



END Wochentage.

