(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HID.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:29 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE HID;    (* Wird von allen Datenteilsystemen verwandt *)

  FROM Storage  IMPORT ALLOCATE, DEALLOCATE;
  FROM SYSTEM   IMPORT TSIZE;
  FROM GENERIC  IMPORT TObjektreferenz;
  FROM WholeStr IMPORT CardToStr,StrToCard,ConvResults;
  IMPORT        String;

  TYPE TIDDS = RECORD
                    SeqNr , 
                    Schluessel : CARDINAL
                 END;
       TID   = POINTER TO TIDDS;
  
  TYPE TString = String.TString;

  PROCEDURE Create (inSeqNumber : CARDINAL) : TID;
  (* -- Berechnet aus SeqNr die entsprechende ID der internen ID-Folge *)

      PROCEDURE BerechneFolgeglied (inSeqNumber : CARDINAL) : CARDINAL;
      (* -- Berechnet  das "inSeqNumber"-Folgeglied der ID-Folge *)
      VAR Schluessel : CARDINAL;
      BEGIN
        (* -- Vorbed.: inSeq > 0 *)
        Schluessel := 2* inSeqNumber; (* -- "Berechnungsvorschrift" der ID-Folge *)
        RETURN Schluessel
        (* -- Nachbed.: Schluessel ist eindeutig *)
      END BerechneFolgeglied;

  VAR aktID : TID;
  BEGIN (* -- Create *)
    (* -- Vorbed.: keine *)
     ALLOCATE (aktID, TSIZE (TIDDS)) ;
     WITH aktID^ DO
       SeqNr := inSeqNumber;
       Schluessel := BerechneFolgeglied (inSeqNumber)
     END; (* WITH *)
     RETURN aktID
    (* -- Nachbed.: RueckgabeADO aktID existiert *)
  END Create;


  PROCEDURE CreateFolgeNr (inID : TID) : CARDINAL;
  (* -- Gibt  aus ID die FolgNr zurck *)
  BEGIN
        IF inID^.SeqNr > 0 THEN
          RETURN inID^.Schluessel / 2;
	ELSE
	  RETURN 1;
	END (* IF *);
  END CreateFolgeNr;

  
  PROCEDURE Dispose (VAR inoutID : TID);
  (* -- Entfernt das ADO inoutID aus dem System *)
   BEGIN
    (* -- Vorbed.: keine *)
     DEALLOCATE (inoutID, TSIZE (TIDDS));
     inoutID := NIL; 
    (* -- Nachbed.: ADO inoutID existiert nicht mehr *)
   END Dispose; 

  PROCEDURE OBJEKTREFERENZ (inID : TID) : TObjektreferenz;
  (* -- Typtransfer von TID nach TObjektreferenz *)
  VAR IDRef : TObjektreferenz;
  BEGIN
    (* -- Vorbed.: keine *)
    IDRef := inID;
    RETURN IDRef
     (* -- Nachbed.: inID = DAUERAUFTRAG (IDRef)  *)
  END OBJEKTREFERENZ;

  PROCEDURE ID (inObjektreferenz : TObjektreferenz) : TID;
  (* -- Typtransfer von TObjektreferenz nach TID *)
  VAR aktID : TID;
  BEGIN
    (* -- Vorbed.: inObjektreferenz referenziert ein ID-Objekt *)
    aktID := inObjektreferenz;
    RETURN aktID
    (* -- Nachbed.: Referenz von aktDA = inObjektreferenz *)
  END ID;

  PROCEDURE Duplicate (inQuellID : TID) : TID;
  (* -- Erzeugt ein neues ID-Objekt und gleicht dieses an den Zustand bzw. 
     -- die Attribute von inQuellID an. *)
  VAR Duplikat : TID;
  BEGIN
    (* -- Vorbed.: keine *)
    Duplikat := Create (LiesSeqNr (inQuellID));
    Equalize (Duplikat, inQuellID);
    RETURN Duplikat
    (* -- Nachbed.: IsEqual (Duplikat, inQuellID) 
       --           AND NOT (Duplikat = inQuellID) *)
  END Duplicate;

 PROCEDURE IsEqual (inID1, inID2 : TID) : BOOLEAN;
  (* -- Vergleicht inID1 und inID2 auf Zustandsgleichheit *)
   VAR gleich : BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)
    gleich := (inID1^.Schluessel = inID2^.Schluessel)
                 AND (inID1^.SeqNr = inID2^.SeqNr);
    RETURN gleich
    (* -- Nachbed.: keine *)
  END IsEqual;

  PROCEDURE Equalize (inoutZielID, inQuellID : TID);
  (* -- Gleicht den Zustand /die Attribute der ZielID an die QuellID an *)
  BEGIN
    (* -- Vorbed.: keine *)
    inoutZielID^ := inQuellID^
    (* -- Nachbed.: IsEqual (inQuellID, inoutZielID) *)
  END Equalize;

  PROCEDURE LiesSeqNr (inID : TID) : CARDINAL;
  (* -- Liefert die Nummer des Folgeglieds *)
  BEGIN
    (* -- Vorbed.: keine *)  
    RETURN inID^.SeqNr
    (* -- Nachbed.: keine *)
  END LiesSeqNr;

  PROCEDURE SameSeqNr (inID1, inID2 : TID) : BOOLEAN;
  (* -- Prueft, ob die IDs aus dem gleichen Folgeglied generiert  worden sind *)
  VAR same : BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)
    same := (inID1^.SeqNr = inID2^.SeqNr);
    RETURN same
    (* -- Nachbed.: inID1.SeqNr = inID2.SeqNr *)
  END SameSeqNr;

  PROCEDURE SmallerSeqNr (inID1, inID2 : TID) : BOOLEAN;
  (* -- Prueft, ob inID1 aus einem kleineren Folgeglied generiert wurde 
     -- als inID2*)
  VAR smaller : BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)
    smaller := (inID1^.SeqNr < inID2^.SeqNr);
    RETURN smaller
    (* -- Nachbed.: inID1.SeqNr < inID2.SeqNr *)
  END SmallerSeqNr;

(* Hier noch zwei Operationen fr den Typ HID: Konversion in und von TString *)
(* dabei wird natuerlich nur die Schluessel-Nr konvertiert.                  *)

  PROCEDURE HID2Str( inID : TID ; IDLaenge: CARDINAL; filler:CARDINAL): TString ;
  (* -- Vorbed.: inID existiert
     -- gibt die SchluesselNr des ID als String zurueck. IDLaenge steuert die 
	Breite des Strings. Es werden fuehrende Leerzeichen ausgegeben, bis
	die Gesamtlaenge=Laenge ist. Falls IDLaenge=0 angegeben wird, wird der 
	String 'getrimmt' returniert. Falls filler=0 wird zum Auffuellen ein
	Blank benutzt ansonsten '0'.
     -- Nachbed.: keine *)
  VAR aktIDStr	: TString;
      fill	: TString;
      help	: TString;
      CharArray	: ARRAY[1..255] OF CHAR;
  BEGIN
    aktIDStr:=String.Create();
    CardToStr(inID^.Schluessel,CharArray);
    String.Schreibe(aktIDStr,CharArray);
    IF (IDLaenge > 0) AND (IDLaenge > String.Laenge(aktIDStr)) THEN
	fill:=String.Create();
        IF filler=1 THEN
		String.Schreibe(fill," ");
	ELSE
		String.Schreibe(fill,"0");
	END (* IF *);	
	help:=String.Duplicate(aktIDStr);
	String.Schreibe(aktIDStr,"");
	WHILE String.Laenge(aktIDStr)+String.Laenge(help) < IDLaenge DO
	  String.Anhaengen(fill,aktIDStr);
	END (* WHILE *);
	String.Anhaengen(help,aktIDStr);
	String.Dispose(help);
	String.Dispose(fill);
    END (* IF *);
    RETURN aktIDStr;
  END HID2Str;
    

  PROCEDURE Str2HID( inStr: TString ): TID ;
  (* -- Vorbed.: inString existiert
     -- gibt eine ID zurueck deren SequenzNr aus der in inStr enthaltenen 
	SchluesselNr berechnet worden ist.
     -- Nachbed.: Falls inStr keine gueltige ID enthalten hat, werden 
	SequenzNr und  SchluesselNr im Rckgabe ADO auf 0 gesetzt *)     
  VAR aktID: TID;
      aktKey: CARDINAL;
      CharArray: ARRAY[1..255] OF CHAR;
      Result: ConvResults;
      
  BEGIN
    String.Lies(inStr,CharArray);
    StrToCard(CharArray,aktKey,Result);
    IF (aktKey > 0) AND NOT ODD(aktKey) AND (Result=strAllRight) THEN
      aktID:=Create(aktKey DIV 2);
    ELSE
      aktID:=Create(0); (* falls Schrott bergeben wurde,*)
			(* wird eine ID mit SequenzNr und *)
                        (* Schluesselnummer = 0 returniert *)
    END (* IF *);  
    RETURN aktID;
  END Str2HID;


(* Pruefroutine von Hebert Schoenhammer , ohne Gewaehr *)
  PROCEDURE PruefeHID (inString : TString;
                        VAR outMessage : TString) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der inString in eine gueltige ID umgewandelt werden kann
     -- Nachbed.: keine *)

  VAR CharArray : ARRAY [1..255] OF CHAR;
      result : ConvResults;
      wert, rest : CARDINAL;
      OK : BOOLEAN;
  BEGIN
    outMessage := String.Create ();
    String.Lies (inString, CharArray);
    StrToCard(CharArray,wert,result);

    CASE result OF
         strAllRight:    rest := wert MOD 2;
                         IF rest = 0 THEN
                            OK := TRUE;
                            String.Schreibe (outMessage, "");
                         ELSE
                            OK := FALSE;
                            String.Schreibe (
                              outMessage, "Eingabe muss durch 2 teilbar sein");
                         END; (* IF *)

        |strOutOfRange:  OK := FALSE;
                         String.Schreibe (
                                outMessage, "Eingabe ausser Wertebereich");
        |strWrongFormat: OK := FALSE;
                         String.Schreibe (
                                outMessage, "Eingabe in falschem Format");
        |strEmpty      : OK := FALSE;
                         String.Schreibe (outMessage, " Leere Eingabe");
    END; (* CASE *)
    IF OK THEN 
	String.Dispose(outMessage);
    END (* IF *);	
    RETURN OK;
  END PruefeHID; 
  
END HID.
