(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: HIDSpeichern.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:30 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 
*)
(******************************************************************************)


(* Modul zum Speichern des TS HIDVerwaltung. hgw, 28.1.2001                   *)
(* Kurze Beschreibung der Vorgehensweise:                                     *)
(* Das Abspeichern der Liste der freien IDs aus der ID Verwaltung erfolgt ana-*)
(* log zur Speicherung des Inhalts der HiKS Datenteilsysteme. Dabei wird      *)
(* nicht der gesamte Datentyp HID, sondern nur die in ihm enthaltene SequenzNr*)
(* gespeichert. Beim Laden werden aus den SequenzNr wieder HIDs generiert und *)
(* diese per 'UeberlasseID' den freien IDs zugeschlagen. Der erste Eintrag in *)
(* der Datei ist der Eintrag der FolgeNr aus der ID Verwaltung mit dessen     *)
(* Hilfe die naechste neue ID vergeben wird.                                  *)

IMPLEMENTATION MODULE (* -- Modul *) HIDSpeichern;
   
  FROM HIDVerwIMPORTE IMPORT ReadLine, WriteLine, Eof;
  FROM WholeStr IMPORT CardToStr,StrToCard,ConvResults;  
  IMPORT HIDVerwADO;
  IMPORT HID;
  IMPORT String;

  TYPE TID = HID.TID;



  PROCEDURE Load(	inH : THandle;
			VAR Message : TString) : BOOLEAN; 
  VAR FolgeNr 	: CARDINAL;
      str	: TString;
      aktID   	: TID;
      success	: BOOLEAN;
      CharArray	: ARRAY [1..255] OF CHAR;
      Result	: ConvResults;  

  BEGIN
        success:=TRUE;
        IF ReadLine(inH, str, Message) THEN
          (* alte IDVerwaltung wegwerfen *)
	  HIDVerwADO.DisposeIDVerwaltung(); 
	  (* neuen Ordner anlegen *)
	  HIDVerwADO.CreateIDVerwaltung(); 
          (* im ersten Datensatz steht die hchste ID => FolgeNr *)
	  (* konvertiere str in CARDINAL *)
          String.Lies(str,CharArray);
	  String.Dispose(str);
          StrToCard(CharArray, FolgeNr,Result);
 	  aktID:=HID.Create(FolgeNr);
          (* Setze die FolgeNr der ID Verwaltung auf den gerade gelesenen Wert*)
	  HIDVerwADO.SetHighestID(aktID);
          (* Falls noch Daten gelesen werden knnen, werden diese in die  *)
	  (* Liste der freien IDs eingefuegt *)
	  WHILE (NOT Eof(inH)) AND success DO
            IF ReadLine(inH, str, Message) THEN
              (* konvertiere FolgeNr in ID *)
              String.Lies(str,CharArray);
	      String.Dispose(str);
              StrToCard(CharArray, FolgeNr,Result);
	      aktID:=HID.Create(FolgeNr);
              (* Fge die ID der Liste der freien IDs hinzu *)
	      HIDVerwADO.LeaveOldID(aktID);
	    ELSE
	      success:=FALSE;
	    END (* IF *);
	  END (* WHILE *);
	ELSE
          success:=FALSE;
	END (* IF *);
	IF NOT success  THEN
          IF Message=NIL THEN
                Message:=String.Create();
          END (* IF *);
          String.Schreibe(Message,
		"Beim Lesen der ID Verwaltung ist ein Fehler aufgetreten!");
        END (* IF *);
	RETURN success;
  END Load;

	 

  PROCEDURE Save(inH : THandle; VAR Message : TString):BOOLEAN; 
  (* -- Vorbed.: inH verweist auf eine zum Schreiben geoeffnete Datei
     -- restauriert das TS HIDVerwaltung. Speichert den HIDOrdner und die SequenzNr
	auf den Sekundaerspeicher. Falls das Speichern fehlschlaegt, wird false
	zurueckgegeben und in outM eine Fehlernachricht ausgegeben.
     -- Nachbed.: IDVerwaltung ist gespeichert  *)


  VAR aktID	: TID;
      str	: TString;
      FolgeNr	: CARDINAL;
      success	: BOOLEAN;
      CharArray	: ARRAY [1..255] OF CHAR;
  BEGIN
	str:=String.Create();
        success:=TRUE;
        (* im ersten Datensatz steht die hoechste ID => FolgeNr *)
        aktID:=HIDVerwADO.GetHighestID() ;
	FolgeNr:= HID.CreateFolgeNr(aktID);
        CardToStr(FolgeNr, CharArray);
        String.Schreibe(str,CharArray);
        IF WriteLine(inH, str, Message) THEN
          (* Gibt es freie IDs in der IDVerwaltung ? *)
          IF HIDVerwADO.FreeIDsAvailable() THEN
            (* Ja! Hole alle freien IDs und schreibe die SequenzNr weg *)
            aktID:=HIDVerwADO.FirstFreeID();
	    REPEAT
	      FolgeNr:= HID.CreateFolgeNr(aktID);
              CardToStr(FolgeNr, CharArray);
	      String.Schreibe(str,CharArray); 
	      IF NOT WriteLine(inH,str,Message) THEN
                success:=FALSE;
	      END (* IF *);
	    UNTIL NOT HIDVerwADO.NextFreeID(aktID) OR NOT success;
	  END (* IF *);
	ELSE
	  success:=FALSE;
	END (* IF *);
	IF NOT success THEN
	  IF Message=NIL THEN
		Message:=String.Create();
	  END (* IF *);
	  String.Schreibe(Message,
		"Beim Schreiben der ID Verwaltung ist ein Fehler aufgetreten!");
        END (* IF *);

        String.Dispose(str);
	RETURN success;
  END Save;

END HIDSpeichern.
