(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: KondenIMPORTE.mod,v $
# $Author: Sopra $
# $Date: 2001/03/13 08:46:54 $
# $Revision: 1.2 $
Bemerkungen:
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KondenIMPORTE;

(* Modul zur Bndelung der Importe des TS Konditionen *)

  IMPORT VDat, HIDVerwaltung;
  FROM String IMPORT TString;

(* Operationen aus dem TS Virtuelles Dateisystem *)

  PROCEDURE ReadLine ( inH: THandle;
                       VAR inoutString: TString;
                       VAR inoutM: TString): BOOLEAN;
        (* Liest aus der Datei inH von der aktuellen Position bis zum *)
        (* nchsten EOL oder EOF . Falls dabei ein Fehler auftritt wird*)
        (* false und ein Fehler in inoutM zurckgegeben, sonst true *)
  BEGIN
        RETURN VDat.ReadLineInVDat(inH,inoutString,inoutM);
  END ReadLine;

  PROCEDURE WriteLine( inH : THandle;
                       inS : TString;
                       VAR inoutM : TString) : BOOLEAN;
        (* Schreibt in die Datei inH an die aktuelle Position den String inS *)
        (* gefolgt von einem Zeilen-Ende-Zeichen *)
        (* Falls dabei ein Fehler auftritt, wird false und ein Fehler in*)
        (* inoutM zurckgegeben, sonst true *)
  BEGIN
        RETURN VDat.WriteLineInVDat(inH,inS,inoutM);
  END WriteLine;

  PROCEDURE Eof ( inH : THandle): BOOLEAN;
           (* Prft ob Pos = EOF ist *)
  BEGIN
        RETURN VDat.EofInVDat( inH ) ;
  END Eof;          

(* Operationen aus dem TS HID Verwaltung *)

  PROCEDURE HoleNeueHID () : TKID;
  (* -- Vorbed.: keine
     -- Liefert eine z.Z. unbenutzte ID. Sei ouTKID der
     -- Rueckgabeparameter, dann gilt:
     -- Nachbed.: (ouTKID existiert) AND (ouTKID NOT IN IDVerwaltung) *)
  BEGIN
	RETURN HIDVerwaltung.HoleNeueID();
  END HoleNeueHID;

  PROCEDURE UeberlasseAlteHID (inID  : TKID);
  (* -- Vorbed.: inID NOT IN IDVerwaltung)
     -- Uebergibt eine z.Z. unbenutzte ID an die ID-Verwaltung
     -- Nachbed.: inID IN IDVerwaltung *)
  BEGIN
        HIDVerwaltung.UeberlasseAlteID(inID);
  END UeberlasseAlteHID; 


  PROCEDURE DisposeHID (VAR inoutID : TKID);
  (* -- Vorbed.: keine
     -- Entfernt das ADO inoutID aus dem System
     -- Nachbed.: ADO inoutID existiert nicht mehr *)
  BEGIN
    HIDVerwaltung.DisposeHID(inoutID);
  END DisposeHID;

  PROCEDURE DuplicateHID (inQuellKID : TKID) : TKID;
  (* -- Vorbed.: keine
     -- Erzeugt ein neues ID-Objekt und gleicht dieses an den Zustand bzw.
     -- die Attribute von inQuellID an. Bezeichnet Duplikat das Rueckgabe-ADO
     -- der Operation, so gilt:
     -- Nachbed.: IsEqual (Duplikat, inQuellID) AND NOT (Duplikat = inQuellID) *)
  BEGIN
    RETURN HIDVerwaltung.DuplicateHID(inQuellKID);
  END DuplicateHID;    
  
  PROCEDURE IsEqualHID (inKID1, inKID2 : TKID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Ueberprueft die beiden IDs auf Zustandsgleichheit
     -- Nachbed.: keine; *)
  BEGIN
    RETURN HIDVerwaltung.IsEqualHID(inKID1,inKID2);
  END IsEqualHID;

  PROCEDURE EqualizeHID (inoutZielKID, inQuellKID : TKID);
  (* -- Vorbed.: keine
     -- Gleicht den Zustand /die Attribute der ZielID an die QuellID an
     -- Nachbed.: IsEqual (inQuellID, inoutZielID) *)
  BEGIN
    HIDVerwaltung.EqualizeHID(inoutZielKID,inQuellKID);
  END EqualizeHID;

  PROCEDURE LiesSeqNrHID (inKID : TKID) : CARDINAL;
  (* -- Vorbed.: keine
     -- Liefert die Nummer des Folgeglieds
     -- Nachbed.: keine *)
  BEGIN
    RETURN HIDVerwaltung.LiesSeqNrHID(inKID );
  END LiesSeqNrHID;

  PROCEDURE SameSeqNrHID (inKID1, inKID2 : TKID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob inID1 und inID2 aus dem gleichen Folgeglied generiert wurden.
     -- Bezeichne same den boole'schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: same <=> (inID1.SeqNr = inID2.SeqNr) *)
  BEGIN
    RETURN HIDVerwaltung.SameSeqNrHID(inKID1, inKID2);
  END SameSeqNrHID;

  PROCEDURE SmallerSeqNrHID (inKID1, inKID2 : TKID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob inID1 aus einem kleineren Folgeglied generiert wurde als inID2.
     -- Bezeichne smaller den boole'schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: smaller <=> (inID1.SeqNr < inID2.SeqNr) *)
  BEGIN 
    RETURN HIDVerwaltung.SmallerSeqNrHID (inKID1, inKID2);
  END SmallerSeqNrHID;


  PROCEDURE HID2Str( inID : TKID ; IDLaenge: CARDINAL): TString ;
  (* -- Vorbed.: inID existiert
     -- gibt die SchluesselNr des ID als String zurueck. IDLaenge steuert die
        Breite des Strings. Es werden fuehrende Leerzeichen ausgegeben, bis
        die Gesamtlaenge=Laenge ist. Falls IDLaenge=0 angegeben wird, wird der
        String 'getrimmt' returniert.
     -- Nachbed.: keine *)
  BEGIN
        RETURN HIDVerwaltung.HID2Str(inID,IDLaenge);
  END HID2Str; 
   
  PROCEDURE HID2Str4SL( inID : TKID ; IDLaenge: CARDINAL): TString ;
  (* -- Vorbed.: inID existiert
     -- gibt die SchluesselNr des ID als String zurueck. IDLaenge steuert die
        Breite des Strings. Es werden fuehrende NULLEN ausgegeben, bis
        die Gesamtlaenge=Laenge ist. Falls IDLaenge=0 angegeben wird, wird der
        String 'getrimmt' returniert.
     -- Nachbed.: keine *)
  BEGIN
        RETURN HIDVerwaltung.HID2Str4SL(inID,IDLaenge);
  END HID2Str4SL; 
   
  PROCEDURE Str2HID( inStr: TString ): TKID ;
  (* -- Vorbed.: inStr existiert und VAL(inStr) ist eine gerade Zahl
     -- gibt einen ID zurueck dessen SequenzNr aus der in inStr 
	enthaltenen SchluesselNr berechnet worden ist.
     -- Nachbed.: Falls inStr keine gueltige ID enthalten hat werden
	SequenzNr und SchluesselNr im Rckgabe ADO auf 0 gesetzt *)     
  BEGIN
        RETURN HIDVerwaltung.Str2HID(inStr);
  END Str2HID; 

END KondenIMPORTE.






