(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: KondenListe.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:00 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KondenListe;

(* ADT KonditionsListe , abgeleitet vom GADT SortierteObjektliste*)


  IMPORT Kondition;
  IMPORT SortierteObjektliste;

  TYPE TKondListe =  SortierteObjektliste.TSortierteObjektliste;
       TObjektreferenz = SortierteObjektliste.TObjektreferenz;


(* SEMANTIK:

Die Semantik ist dem Modul "SortierteObjektliste.def" entnommen

Die Semantik wird auf Basis zweier lokaler Integervariablen len und pos,
die den Zustand der sortierten Objektliste beschreiben, und einer
lokalen Kapazitaetskonstanten maxlen spezifiziert: 
Sei len die Laenge der sortierten Objektliste, maxlen > 0 die Maximal-
kapazitaet der Liste, d.h. 0 <= len <= maxlen, und pos die Position 
des aktuellen Listenelements mit 0 <= pos <= len + 1.
Die Indizes alt und neu bezeichnen den Wert einer Zustandsvariablen vor
bzw. nach Ausfuehren einer Operation. Werden die neuen Zustaende der
Variablen nicht explizit angegeben, so sind sie invariant.
*)

PROCEDURE  Create () : TKondListe;
(*
Vb:	keine
Generiert und initialisert ein ADO TKondListe.
Nb:	Rueckgabe-ADO TKondListe existiert AND IsEmpty(TKondListe)
*)

  VAR outKondListe : TKondListe;

BEGIN
  outKondListe := SortierteObjektliste.Create (DisposeKond,
                                               DuplicateKond,
                                               EqualizeKond,
                                               IsEqualKond,
                                               FruehereGueltigkeit,
                                               GleicheGueltigkeit);
  RETURN outKondListe;

END Create;


PROCEDURE Dispose (VAR inoutKondListe : TKondListe);
(*
Vb:	IsEmpty (inoutKondListe : TKondListe)
Entfernt das ADO inoutKondListe aus dem System.
Nb:	ADO inoutKondListe existiert nicht mehr
*)
BEGIN
  SortierteObjektliste.Dispose (inoutKondListe);
END Dispose;


PROCEDURE Current(inKondListe : TKondListe) : TKond;
(*
Vb:	NOT (OffStart (inKondListe) OR OffEnd (inKondListe))
Liefert das Objekt an der aktuellen Position.
Nb:	NOT (OffStart(inKondListe) OR OffEnd(inKondListe))
*)
BEGIN
  RETURN Kondition.KONDITION (SortierteObjektliste.Current (inKondListe));
END Current;


PROCEDURE Update (inoutKondListe : TKondListe;
                  inObjekt     : TKond);
(*
Vb:	NOT (OffStart (inoutKondListe) OR OffEnd (inoutKondListe))
Ueberschreibt unter Beachtung der Ordnung die Attributwerte des 
Objekts an der aktuellen Position mit denen von inObjekt. 
Anschliessend sind das Objekt an der ggf. neuen aktuellen 
Position und inObjekt zustandsgleich
Nb:	NOT (OffStart (inoutKondListe) OR OffEnd (inoutKondListe)
*)
BEGIN
  SortierteObjektliste.Update (inoutKondListe, Kondition.OBJEKTREFERENZ (inObjekt));
END Update;


PROCEDURE Insert (inoutKondListe : TKondListe;
                  inObjekt     : TKond);
(*
Vb:	keine
Einfuegen eines Objekts bzgl. der Sortierreihenfolge
Anschliessend befindet sich inObjekt an der aktuellen Position 
Nb:	(1 <= inoutKondListe.pos_neu <= inoutKondListe.len) AND
              (inoutKondListe.len_neu = inoutKondListe.len_alt + 1) AND NOT 
              (IsEmpty (inoutKondListe) OR OffStart (inoutKondListe)
               OR OffEnd (inoutKondListe) )
*)

BEGIN
  SortierteObjektliste.Insert (inoutKondListe, 
                               Kondition.OBJEKTREFERENZ (inObjekt));
END Insert;


PROCEDURE Delete (inoutKondListe : TKondListe);
(*
Vb:	NOT (OffStart (inoutKondListe) OR OffEnd (inoutKondListe))
Loeschen des Objekts an der aktuellen Position; das Listenelement rechts 
neben dem zu loeschenden Element befindet sich anschliessend an der 
(unveraenderten) aktuellen Position. 
Wird das letzte Objekt geloescht, steht die aktuelle Position auf OffEnd
Nb:     (inoutKondListe.len_neu = inoutKondListe.len_alt - 1) OR
         OffStart (inoutKondListe))
*)
BEGIN
  SortierteObjektliste.Delete (inoutKondListe);
END Delete;


PROCEDURE Search (inoutKondListe : TKondListe;
                  inObjekt     : TKond) : BOOLEAN;
(*
Vb:	keine
Suche nach inObjekt; wird ein zustandsgleiches Objekt gefunden, 
ist dort die neue aktuelle Position; sonst OffEnd
Sei found der boole'sche Rueckgabeparameter, dann gilt:
Nb: (found => NOT (OffStart (inoutKondListe) OR OffEnd (inoutKondListe))) AND
    (NOT found => OffEnd (inoutKondListe))
*)

BEGIN
  RETURN SortierteObjektliste.Search (inoutKondListe,
                                      Kondition.OBJEKTREFERENZ (inObjekt));
END Search;


PROCEDURE GoSmallest (inoutKondListe : TKondListe);
(*
Vb:	keine
Die Position des kleinsten Objekts der Liste wird zur aktuellen Position
Nb:	inoutKondListe.pos_neu = 1
*)
BEGIN
  SortierteObjektliste.GoSmallest (inoutKondListe);
END GoSmallest;


PROCEDURE StopAt (inoutKondListe : TKondListe;
                  inObjekt     : TKond) : BOOLEAN;
(*
Vb:	keine
Suche nach einem bzgl. SameValue Objekt gleichen Objekt. Wird ein
solches Objekt gefunden, ist dort die neue aktuelle Position; 
Seien found der boole'sche Rueckgabeparameter und aktObjekt bzw.   
MaxObjekt die Objekte an pos_neu bzw. pos = len (falls diese 
existieren), dann gilt:
Nb:    (found => NOT (OffStart (inoutKondListe) OR OffEnd (inoutKondListe)) AND
                 Objekt.SameValue (inObjekt, aktObjekt)) AND
       (NOT found => (IsEmpty (inoutKondListe) AND OffEnd (inoutKondListe)) OR
                     (NOT IsEmpty (inoutKondListe) AND OffEnd (inoutKondListe) AND
                      Objekt.SmallerValue (MaxObjekt, inObjekt)) OR
                      (NOT (OffStart (inoutKondListe) OR
                            OffEnd (inoutKondListe)) AND
                       Objekt.SmallerValue (aktObjekt, inObjekt))
*)

BEGIN
  RETURN SortierteObjektliste.StopAt (inoutKondListe, Kondition.OBJEKTREFERENZ (inObjekt));
END StopAt;



PROCEDURE GoLargest (inoutKondListe : TKondListe);
(*
Vb:	keine
Die Position des groessten Objekts der Liste wird zur aktuellen Position
Nb:	inoutKondListe.pos_neu = inoutKondListe.len
*)

BEGIN
  SortierteObjektliste.GoLargest (inoutKondListe);
END GoLargest;


PROCEDURE StepNext (inoutKondListe : TKondListe);
(*
Vb:	NOT OffEnd (inoutKondListe)
Die Position hinter der aktuellen Position wird zur aktuellen Position
Nb:	(inoutKondListe.pos_neu = inoutKondListe.pos_alt + 1
         OR (IsEmpty (inoutKondListe) => OffEnd (inoutKondListe)
*)

BEGIN
  SortierteObjektliste.StepNext (inoutKondListe);
END StepNext;



PROCEDURE StepBack (inoutKondListe : TKondListe);
(*
Vb:	NOT OffStart (inoutKondListe)
Die Position vor der aktuellen Position wird zur aktuellen Position
Nb:	inoutKondListe.pos_neu = inoutKondListe.pos_alt - 1) OR
        (IsEmpty (inoutKondListe) => OffStart (inoutKondListe)
*)

BEGIN
  SortierteObjektliste.StepBack (inoutKondListe);
END StepBack;


PROCEDURE IsEmpty (inKondListe : TKondListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Liste leer ist:
IsEmpty (inKondListe) <=> (inKondListe.len = 0)
Nb:	IsEmpty (inKondListe) => OffStart (inKondListe) OR OffEnd (inKondListe)
*)

BEGIN
  RETURN SortierteObjektliste.IsEmpty (inKondListe);
END IsEmpty;


PROCEDURE OffStart (inKondListe : TKondListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position vor dem ersten Objekt aktuell ist:
OffStart (inKondListe) <=> inKondListe.pos = 0
Nb:	keine
*)

BEGIN
  RETURN SortierteObjektliste.OffStart (inKondListe);
END OffStart;


PROCEDURE OffEnd (inKondListe : TKondListe) : BOOLEAN;
(*
Vb:	keine
Test, ob die Position hinter dem letzten Objekt aktuell ist:
OffEnd (inKondListe) <=> inKondListe.pos = inKondListe.len + 1
Nb:	Rueckgabe-ADO TKondListe existiert AND IsEmpty(TKondListe)
*)

BEGIN
  RETURN SortierteObjektliste.OffEnd (inKondListe);
END OffEnd;
 

(* ------------------------------------------------------------------
   -- Typkonvertierungen von TKID und TObjektreferenz
   ------------------------------------------------------------------
*)

PROCEDURE DisposeKond (VAR inoutReferenz : TObjektreferenz);
 VAR aktKond : TKond;
BEGIN
  aktKond := Kondition.KONDITION (inoutReferenz);
  Kondition.Dispose (aktKond);
  inoutReferenz := Kondition.OBJEKTREFERENZ (aktKond);
END DisposeKond;

PROCEDURE DuplicateKond (inReferenz : TObjektreferenz) : TObjektreferenz;
  VAR Source      : TKond;
      outDuplikat : TKond;
BEGIN
  Source := Kondition.KONDITION (inReferenz);
  outDuplikat := Kondition.Duplicate (Source);
  RETURN Kondition.OBJEKTREFERENZ (outDuplikat)
END DuplicateKond;

PROCEDURE EqualizeKond (inoutZiel, inQuelle : TObjektreferenz);
  VAR ZielKond  : TKond;
      QuellKond : TKond;
BEGIN
 ZielKond := Kondition.KONDITION (inoutZiel);
 QuellKond := Kondition.KONDITION (inQuelle);
 Kondition.Equalize (ZielKond, QuellKond)
END EqualizeKond;

PROCEDURE IsEqualKond (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Kond1 : TKond;
      Kond2 : TKond;
BEGIN
  Kond1 := Kondition.KONDITION (inReferenz1);
  Kond2 := Kondition.KONDITION (inReferenz2);
  RETURN Kondition.IsEqual (Kond1, Kond2)
END IsEqualKond;

PROCEDURE SmallerKond (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Kond1 : TKond;
      Kond2 : TKond;
BEGIN
  Kond1 := Kondition.KONDITION (inReferenz1);
  Kond2 := Kondition.KONDITION (inReferenz2);
  RETURN Kondition.SmallerKondition (Kond1, Kond2)
END SmallerKond;

PROCEDURE SameKond (inReferenz1, inReferenz2 : TObjektreferenz) : BOOLEAN;
  VAR Kond1 : TKond;
      Kond2 : TKond;
BEGIN
  Kond1 := Kondition.KONDITION (inReferenz1);
  Kond2 := Kondition.KONDITION (inReferenz2);
  RETURN Kondition.SameKondition (Kond1, Kond2)
END SameKond;

PROCEDURE FruehereGueltigkeit ( inReferenz1, inReferenz2: TObjektreferenz):BOOLEAN;
  VAR Kond1 : TKond;
      Kond2 : TKond;
BEGIN
  Kond1 := Kondition.KONDITION (inReferenz1);
  Kond2 := Kondition.KONDITION (inReferenz2);
  RETURN Kondition.FruehereGueltigkeit (Kond1, Kond2)
END FruehereGueltigkeit;
                         
PROCEDURE GleicheGueltigkeit ( inReferenz1, inReferenz2: TObjektreferenz):BOOLEAN;
  VAR Kond1 : TKond;
      Kond2 : TKond;
BEGIN
  Kond1 := Kondition.KONDITION (inReferenz1);
  Kond2 := Kondition.KONDITION (inReferenz2);
  RETURN Kondition.GleicheGueltigkeit (Kond1, Kond2)
END GleicheGueltigkeit;
                         
END KondenListe.
