(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Konditioninformations-System 'HiKS'                     *)
(*
# $RCSfile: KondenOrdSchab.mod,v $
# $Author: Sopra $
# $Date: 2001/03/14 18:26:03 $
# $Revision: 1.4 $
# Bemerkungen:  Einfuegen von Konditionen nach der Semantik von 3.3.3 
		funktioniert jetzt. hgw, 22.2.2001
		First/Next Kondition im Zeitraum ist getestet. Fehler in 
		Insert behoben. HoleGueltigeKondition ist getestet.
		hgw, 23.2.2001
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KondenOrdSchab;

(* ADT KonditionenOrdnerSchablone, hier ist die eigentliche Funktionalitt *)
(* des TS Konditionen untergebracht. Die Subentittstypen Weihnacht, Tarif *)
(* und Vertragskondition werden auf dieser Ebene verborgen *)

(* IMPORTE *)

FROM Storage		IMPORT ALLOCATE, DEALLOCATE;
FROM SYSTEM		IMPORT TSIZE;
IMPORT KondenIMPORTE, Datum;
IMPORT Kondition;
IMPORT KondenListe;
IMPORT String, Stringliste; 
IMPORT WholeStr;

TYPE TString 		= String.TString;
     TStringliste 	= Stringliste.TStringliste;
     TKondOrdner 	= KondenListe.TKondListe;
     THandle		= KondenIMPORTE.THandle;
     TDatum		= Datum.TDatum;
     
 
(*
SEMANTIK
Die Semantik wird mit Hilfe dreier lokaler Integervariablen kap, 
pos und maxkap spezifiziert. Die Variablen kap und pos beschreiben  
den aktuellen Zustand (aktuelle Position und Anzahl der enthaltenen
Elemente) und  maxkap > 0 die Maximalkapazitaet des Ordners, 
d.h. es gilt stets:
            0 <= kap <= maxkap   und    0 <= pos <= kap+1

*)


PROCEDURE Create () : TKondOrdner;
(*
Vb:	keine
Erzeugt und initialisiert ein ADO KondOrdner.
Nb:	Rueckgabe-ADO KondOrdner existiert und ist leer
*)

  VAR KondOrdner : TKondOrdner;

BEGIN
  ALLOCATE(KondOrdner,TSIZE(TKondOrdner));
  KondOrdner := KondenListe.Create();
  RETURN KondOrdner;
END Create;


PROCEDURE Dispose (VAR inoutKondOrdner : TKondOrdner);
(*
Vb:	keine
Entfernt alle Realisierungs-ADOs, die in inoutKondOrdner enthalten sind.
Entfernt das ADO inoutKondOrdner aus dem System
Nb:	ADO KondOrdner existiert nicht mehr
*)

BEGIN
  KondenListe.Dispose(inoutKondOrdner);
  DEALLOCATE(inoutKondOrdner,TSIZE(TKondOrdner));
  inoutKondOrdner := NIL;
END Dispose;



PROCEDURE IsEmpty (inKondOrdner : TKondOrdner) : BOOLEAN;
(*
VB:	keine
Test, ob der Ordner leer ist
Nb:	 IsEmpty (inKondOrdner) <=> kap = 0
*)
  VAR leer : BOOLEAN;
BEGIN

  RETURN KondenListe.IsEmpty (inKondOrdner);

END IsEmpty;




PROCEDURE Insert(inoutKondOrdner : TKondOrdner; inKondition: TKond;
                       VAR outMessage : TString) : BOOLEAN;
        (*
        vb: sei K die neu einzufgende Kondition. Dann muss gelten: ADO
        Konditionsordner existiert and not isfull and K.GueltigVon > heute
        Fuegt eine Kondition gemaess der Prozessspezifikationen D3.3.1-D3.3.3
        in den Konditionsordner ein. Die restlichen Anweisungen gemaess D3.3
        nach dem Einfuegen bezueglich Vertraege/Buchungen sind entitaets-
        spezifisch und werden im TS PSB modelliert
         nb: sei KL die letzte Kondition im Konditionsordner mit
        KL.Entitaetstyp=K.Entitaetstyp. Dann muss gelten: KO vereinigt mit
        Konditionsordner so dass es keine KE in Konditionsordner gibt mit
        (KE.Entitaetstyp=K.Entitaetstyp and K.GueltigVon<=KE.GueltigVon) and
        KL.GueltigBis=K.GueltigVon - 1 Tag
        *)   
	(* Konditionen umfassen volle Wochen *)


  VAR aktKID 	: TKID;
      aktKond	: TKond;
      inKond	: TKond;
      success	: BOOLEAN;
      D		: TDatum;
      TagVon	: CARDINAL;
      TagBis	: CARDINAL;

BEGIN
  TagVon:= Datum.Wochentag(Kondition.LiesGueltigVon(inKondition));  
  TagBis:= Datum.Wochentag(Kondition.LiesGueltigBis(inKondition));  
  IF (TagVon # 1) THEN
     outMessage := String.Create();
     success:= FALSE;                                
     CASE TagVon OF 
	 2 : String.Schreibe(outMessage, 
		"Das GueltigVon Datum fllt auf einen Dienstag!");
	|3 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum fllt auf einen Mittwoch!"); 
	|4 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum fllt auf einen Donnerstag!"); 
	|5 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum fllt auf einen Freitag!"); 
	|6 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum fllt auf einen Samstag!"); 
	|7 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum fllt auf einen Sonntag!"); 
	|0 :  String.Schreibe(outMessage,
                       "Das GueltigVon Datum ist undefiniert!"); 
     END (* CASE *);
  ELSIF  (TagBis # 7) AND (TagBis # 0) THEN
     outMessage := String.Create();
     success:= FALSE;                                
     CASE TagBis OF 
	 1 :  String.Schreibe(outMessage,
                       "Das GueltigBis Datum fllt auf einen Montag!"); 
	|2 : String.Schreibe(outMessage, 
		"Das GueltigBis Datum fllt auf einen Dienstag!");
	|3 :  String.Schreibe(outMessage,
                       "Das GueltigBis Datum fllt auf einen Mittwoch!"); 
	|4 :  String.Schreibe(outMessage,
                       "Das GueltigBis Datum fllt auf einen Donnerstag!"); 
	|5 :  String.Schreibe(outMessage,
                       "Das GueltigBis Datum fllt auf einen Freitag!"); 
	|6 :  String.Schreibe(outMessage,
                       "Das GueltigBis Datum fllt auf einen Samstag!"); 
     END (* CASE *);
  ELSIF NOT Kondition.ExistiertKID(inKondition) THEN
     inKond:=Kondition.Duplicate(inKondition);
     aktKID := KondenIMPORTE.HoleNeueHID();
     Kondition.SetzeKID(inKond, aktKID);
     KondenListe.Insert(inoutKondOrdner,inKond);
     (* Listenzeiger steht nach Abb. 12.3. im Kurs 1793 auf dem neuen Eintrag *)
     KondenListe.StepNext(inoutKondOrdner);
     (* wir stehen jetzt auf dem nchsten Eintrag *)
     (* jetzt ist nach D3.3. zu prfen, ob es schon Konditionen mit spterer *)
     (* Gueltigkeit gibt. Falls ja, werden diese geloescht *)
     WHILE NOT KondenListe.OffEnd(inoutKondOrdner) DO
       aktKond:=KondenListe.Current(inoutKondOrdner);
       IF (Kondition.LiesEntitaetsTyp(aktKond)=
          Kondition.LiesEntitaetsTyp(inKond)) AND
	  (Kondition.FruehereGueltigkeit(inKond,aktKond) OR 
	   Kondition.GleicheGueltigkeit(inKond,aktKond)) THEN
          (* Loesche aktuelles Listenelement im KonditionenOrdner *)
	  Delete(inoutKondOrdner,aktKond);
       ELSE
	  (* Falls geloescht wurde, stehen wir schon auf dem naechsten Eintrag*)
          KondenListe.StepNext(inoutKondOrdner);
       END (* IF *);
       Kondition.Dispose(aktKond);
     END (* WHILE *);
     (* zurck zur aktuell eingefuegten Kondition *)
     (* Suche der letzten Kondition vom Typ GetEntitaetsTyp(inoutKond) *)
     (* und Eintragen von *)
     (* GueltigBis:=Kondition.GetGueltigVon(inKond)-1 *)
     Kondition.Dispose(inKond);
     IF Search(inoutKondOrdner,aktKID,inKond) THEN
       KondenListe.StepBack(inoutKondOrdner);
       WHILE NOT KondenListe.OffStart(inoutKondOrdner) DO 
         aktKond:=KondenListe.Current(inoutKondOrdner);
         (* Kondition vom gleichen Typ und GueltigBis=UNDEF ? oder *)
	 (* GueltigBis > inKond.GueltigBis ? *)
         IF (Kondition.LiesEntitaetsTyp(aktKond)=
           Kondition.LiesEntitaetsTyp(inKond)) AND
           Datum.DatumInZeitraum(Kondition.LiesGueltigVon(inKond),
				Kondition.LiesGueltigVon(aktKond),
                                Kondition.LiesGueltigBis(aktKond))  THEN
           D:=Kondition.LiesGueltigVon(inKond);
	   Datum.DecDatum(D);
           Kondition.SetzeGueltigBis(aktKond,D);
	   Datum.DisposeDatum(D);
           KondenListe.Update(inoutKondOrdner,aktKond);
         END (* IF *);
	 KondenListe.StepBack(inoutKondOrdner);
	 Kondition.Dispose(aktKond);
       END (* WHILE *);
     END (* IF *);
     KondenIMPORTE.DisposeHID(aktKID);
     success := TRUE;
  ELSE
     outMessage := String.Create();
     String.Schreibe(outMessage, "KID ist bereits gesetzt.");
     success:= FALSE;
  END; (* IF *)

  RETURN success;

END Insert;

PROCEDURE BackupInsert( inoutKondOrdner : TKondOrdner; inoutKond: TKond);
(* Das ist die Kurzfassung des Inserts der bei einem Reload des Systems *)
(* aufgerufen wird. Dabei ist nichts zu prfen und auch keine ID zu ver-*)
(* geben. *)
BEGIN
        KondenListe.Insert(inoutKondOrdner,inoutKond);
END BackupInsert;  

PROCEDURE Delete( inoutKondOrdner : TKondOrdner; inKond:TKond );
(* Dies Funktion wird nur innerhalb des Moduls zum Entfernen von Konditionen *)
(* beim Einfuegen einer neuen Kondition benoetigt. Das HiKS unterstuetzt nach*)
(* Produktskizze nicht das Loeschen von Konditionen *)
(*
Vb:	Kondition.ExistiertKID(inKond)
Loescht die Kondition mit KID = Kondition.LiesKID(inKond) aus dem Ordner
Nb:	NOT Search(inoutKondOrdner,Kondition.LiesKID(inKond))
*)

  VAR aktKID : TKID;

BEGIN
  IF KondenListe.Search(inoutKondOrdner, inKond) THEN

     (* Loesche Element an der aktuellen Position des Listenzeigers *)

     KondenListe.Delete(inoutKondOrdner);

     (* Die KID wird jetzt an die ID Verwaltung zurueckgegeben *)

     aktKID := Kondition.LiesKID(inKond);
     KondenIMPORTE.UeberlasseAlteHID(aktKID);

     (* jetzt kann der Rest der Kondition weg *)

     KondenIMPORTE.DisposeHID(aktKID);
  END; (* IF *)

END Delete;


PROCEDURE Search(inKondOrdner : TKondOrdner;
                       inKID : TKID; VAR outKond : TKond) : BOOLEAN;
(*
Vb:	keine
Sucht die Kondition mit der uebergebenen KID; wird die Kondition gefunden,
so wird sie im OUT-Parameter outKond ausgeben;
Nb:	(rc=>(Kond existiert AND Kondition.KID=inKID AND Rueckgabe = TRUE)
         AND
        (NOT success => (Kond  existiert nicht AND Rueckgabe = FALSE))
*)
  VAR aktKID    : TKID;
      success   : BOOLEAN;
      lfdKond   : TKond;
      lfdKID    : TKID;

BEGIN
  success := FALSE;
  IF NOT IsEmpty (inKondOrdner) THEN  
     KondenListe.GoSmallest (inKondOrdner);
     WHILE NOT (success OR KondenListe.OffEnd (inKondOrdner)) DO
       lfdKond := KondenListe.Current (inKondOrdner);
       lfdKID := Kondition.LiesKID (lfdKond);
       IF KondenIMPORTE.IsEqualHID (lfdKID, inKID) THEN
         success := TRUE;
         outKond := Kondition.Duplicate (lfdKond)  
       ELSE 
         KondenListe.StepNext (inKondOrdner)
       END; (* IF *)
       KondenIMPORTE.DisposeHID (lfdKID);
       Kondition.Dispose (lfdKond)
     END; (* WHILE nicht gefunden*)
  END; (* IF inOrdner nicht leer*)
  RETURN success
   
END Search;


PROCEDURE Update(inoutKondOrdner : TKondOrdner; inKond : TKond;
                       VAR outMessage : TString):BOOLEAN;
(*
Vb:		Search(inoutKondOrdner,inKondition.KID) = TRUE
Aktualisiert die Daten der Kondition inKond im Ordner inoutKondOrdner
Nb:	Search(inoutKondOrdner,Kondition.KID,Kond_neu)=TRUE and inKond=Kond_neu
*)

  VAR  success : BOOLEAN;

BEGIN
  IF KondenListe.Search(inoutKondOrdner, inKond) THEN
     KondenListe.Update(inoutKondOrdner, inKond);
     success := TRUE;
  ELSE
     outMessage := String.Create();
     String.Schreibe(outMessage,
                     "Kondition ist nicht im Konditionen-Ordner.");
     success := FALSE;

  END; (* IF *)

  RETURN success;

END Update;

PROCEDURE KonditionVorhanden( 	inKondOrdner : TKondOrdner; 
				inET: Kondition.TKondTyp ):BOOLEAN;
(*
Vb:	keine
Liefert TRUE falls es eine Kondition vom Typ inET gibt
Nb:	keine
*)
VAR 	aktKond : TKond;
	found 	: BOOLEAN;
BEGIN
	found:=FALSE;
	IF First( inKondOrdner, aktKond) THEN
	  REPEAT
	    IF Kondition.LiesEntitaetsTyp(aktKond)=inET THEN
	      found:=TRUE
	    END (* IF *);
	  UNTIL NOT Next(inKondOrdner, aktKond) OR found;		
	  Kondition.Dispose(aktKond);
	END (* IF *);
	RETURN found;
END  KonditionVorhanden; 


PROCEDURE First (inoutKondOrdner : TKondOrdner; VAR outKond : TKond) : BOOLEAN;
(*
Vb:	NOT IsEmpty(inoutKondOrdner)
Die erste Kondition inoutKondOrdner.pos_neu = 1) wird in outKond zurueckgegeben
und inoutKondOrdner.pos auf diese	positioniert
Nb:	(inoutKondOrdner.pos_neu = 1) AND (outKond enthaelt die Kondition auf
         Position 1)
*)

  VAR  success : BOOLEAN;

BEGIN

  IF NOT IsEmpty(inoutKondOrdner) THEN
    KondenListe.GoSmallest(inoutKondOrdner);
    outKond := KondenListe.Current(inoutKondOrdner);
    success := TRUE;
  ELSE
    success := FALSE;
  END; (* IF *)

  RETURN success ;

END First;


PROCEDURE Next(inoutKondOrdner : TKondOrdner; VAR inoutKond : TKond) : BOOLEAN;
(*
Vb:	 Kondition.ExistiertKID(inoutKond) AND
         Search(inoutKondOrdner,inoutKond.KID)
Rueckgabe der Kondition an der naechsten Position; (pos_alt+1, falls existent)
Nb:	 (inoutKondOrdner.pos_neu = inoutKondOrdner.pos_alt + 1) AND
         (inoutKond enthaelt die Kondition auf inoutKondOrdner.pos_alt AND
         (Rueckgabewert = TRUE) <=>(inoutKondOrdner.pos_neu<=kap))
*)

  VAR aktKID    : TKID;
      success   : BOOLEAN;
      lfdKond   : TKond;
      CharArray	: ARRAY [1..255] OF CHAR;

BEGIN
    success := FALSE;

    IF Search (inoutKondOrdner, Kondition.LiesKID(inoutKond),lfdKond) THEN
      
      KondenListe.StepNext (inoutKondOrdner);
      
      IF NOT KondenListe.OffEnd (inoutKondOrdner) THEN
        lfdKond := KondenListe.Current (inoutKondOrdner);
	Kondition.Dispose(inoutKond);
        inoutKond:=Kondition.Duplicate (lfdKond);
        Kondition.Dispose (lfdKond);
        success := TRUE;
      END (* IF NOT KondenListe.OffEnd *);
    END (* IF KondenListe.Search *);

    RETURN success;

END Next;

(**************************** HiKS-Operationen ***************************)

PROCEDURE HoleGueltigeKond(     inKondenOrdner:TKondOrdner;
				inD: TDatum;
                                VAR outKond:TKond;
                                inET: TKondTyp ):BOOLEAN;
(*
vb: Konditionsordner existiert
Hole die zum Datum inD gltige Kondition vom Typ inET. Falls es keine
solche Kondition gibt, wird FALSE returniert.
nb: --
*)
VAR gefunden : BOOLEAN;
    aktKond  : TKond;
    Dvon,Dbis: TDatum;
BEGIN
  KondenListe.GoLargest(inKondenOrdner);
  gefunden:=FALSE;
  IF NOT KondenListe.OffStart(inKondenOrdner) THEN
    REPEAT
      aktKond:=KondenListe.Current(inKondenOrdner);
      Dvon:=Kondition.LiesGueltigVon(aktKond);
      Dbis:=Kondition.LiesGueltigBis(aktKond);
      IF (Kondition.LiesEntitaetsTyp(aktKond)=inET) AND 
	(Datum.DatumInZeitraum(inD,Dvon,Dbis) OR 
        (Datum.SmallerDatum(Dvon,inD) AND Datum.IstDatumEternity(Dbis))) THEN
         gefunden:=TRUE;
         outKond:=Kondition.Duplicate(aktKond);
      ELSE
         KondenListe.StepBack(inKondenOrdner);
      END (* IF *);
      Datum.DisposeDatum(Dvon);
      Datum.DisposeDatum(Dbis);
;      Kondition.Dispose(aktKond);
    UNTIL gefunden OR KondenListe.OffStart(inKondenOrdner) ;
  END (* IF *);
  RETURN gefunden ;
  
END  HoleGueltigeKond;


PROCEDURE FirstKondImZeitraum(  inKondenOrdner:TKondOrdner;
				inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR outKond: TKond;
                                inET: TKondTyp) : BOOLEAN;
(*
vb: Konditionsordner existiert
Hole die erste zwischen Datum inGueltigVon und inGueltigBis gltige
Kondition.
nb: --
*)
VAR success : BOOLEAN;
BEGIN
  KondenListe.GoSmallest(inKondenOrdner);
  success:=HoleKonditionImZeitraum
         (inKondenOrdner,inGueltigVon, inGueltigBis, inET,outKond);
  RETURN success;
END FirstKondImZeitraum;

PROCEDURE NextKondImZeitraum(   inKondenOrdner:TKondOrdner;
				inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR inoutKond:TKond;
                                inET: TKondTyp):BOOLEAN;
(*
vb: Konditionsordner existiert
Falls mit ErsteKondition... eine Kondition die im Zeitraum GueltigVon-
GueltigBis gilt gefunden werden konnte, dann holt diese Prozedur die
nchste Kondition K vom Typ inET fr diesen Zeitraum. Falls es keine
solche Kondition gibt, wird false zurck gegeben und das ADO inoutKond
existiert nicht.
nb: --
*)
VAR success	: BOOLEAN;
    saveKond 	: TKond;

BEGIN
  (* Neupositionierung auf die letzte gefundene Kondition inoutKond *)
  success:=KondenListe.Search(inKondenOrdner,inoutKond);
  (* gesucht wird ab der nchsten Position *)
  IF success THEN
    KondenListe.StepNext(inKondenOrdner);
    saveKond:=Kondition.Duplicate(inoutKond);
    success:=HoleKonditionImZeitraum
           (inKondenOrdner,inGueltigVon, inGueltigBis, inET,inoutKond); 
    IF NOT success THEN
	inoutKond:=saveKond;
    ELSE
	Kondition.Dispose(saveKond);
    END (* IF *);
  END (* IF *);    
  RETURN success;
END NextKondImZeitraum;

PROCEDURE HoleKonditionImZeitraum( 	inKondenOrdner:TKondOrdner;
					inGueltigVon, inGueltigBis: TDatum;
                                   	inET:TKondTyp; 
					VAR outKond:TKond):BOOLEAN;
(* Wird intern von Erste/Nchste Kondition im Zeitraum benutzt *)
VAR gefunden  : BOOLEAN;
    aktKond   : TKond;
    Dvon,Dbis : TDatum;

BEGIN
  gefunden:=FALSE;
  IF NOT KondenListe.OffEnd(inKondenOrdner) THEN
    REPEAT
      aktKond:=KondenListe.Current(inKondenOrdner);
      Dvon:=Kondition.LiesGueltigVon(aktKond);
      Dbis:=Kondition.LiesGueltigBis(aktKond);
      IF (Kondition.LiesEntitaetsTyp(aktKond)=inET) AND
        Datum.SmallerDatum(Dvon,inGueltigBis) AND
        (Datum.SmallerDatum(inGueltigVon,Dbis) OR 
	Datum.IstDatumEternity(Dbis)) THEN
        gefunden:=TRUE;
        outKond:=Kondition.Duplicate(aktKond);
      ELSE
        KondenListe.StepNext(inKondenOrdner);
      END (* IF *);
      Datum.DisposeDatum(Dvon);
      Datum.DisposeDatum(Dbis);
      Kondition.Dispose(aktKond);
    UNTIL gefunden OR KondenListe.OffEnd(inKondenOrdner) ;
  END (* IF *);
  RETURN gefunden ;
END HoleKonditionImZeitraum;      


PROCEDURE GibKondAlsStringliste(inKondenOrdner:TKondOrdner;
				inET: TKondTyp): TStringliste;
       (* Gib die Weihnachtsgeld-Konditionen mit folgender Formatierung als
          Stringliste aus; Formatierung: KID(5Zeichen); 1 Leerzeichen;
	  GueltigVon (10Zeichen); 1 Leerzeichen;
          GueltigBis(10Zeichen); 1 Leerzeichen; Bemessungsmonat(2Zeichen);
          1 Leerzeichen; Stichtag(6Zeichen); insgesamt also 37 Zeichen *)
       (* Gib die Vertrags-Konditionen mit folgender Formatierung als 
	  Stringliste aus; Formatierung:  Formatierung: KID(5Zeichen); 
	  1 Leerzeichen;GueltigVon (10Zeichen); 
	  1 Leerzeichen; GueltigBis(10Zeichen); 1 Leerzeichen; 
	  Beschftigungshchstdauer(5Zeichen); 1 Leerzeichen;
          Stundenobergrenze(5Zeichen); insgesamt also 39 Zeichen *)  	
       (* Gib die Tarif-Konditionen mit folgender Formatierung als Stringliste
	  aus; Formatierung:  Formatierung: KID(5Zeichen); 1 Leerzeichen;
	  GueltigVon (10Zeichen); 1 Leerzeichen; 
	  GueltigBis(10Zeichen); 1 Leerzeichen; Stundensatz(5Zeichen);
          insgesamt also 33 Zeichen *) 

VAR 	strListe	: Stringliste.TStringliste;
	str		: TString;
	komma		: TString;
	space		: TString;
	null		: TString;
	aktKondStr	: TString;
	aktKond		: TKond;
	CharArray	: ARRAY [1..255] OF CHAR;
	SS		: CARDINAL;
	frak		: CARDINAL;


	(* Hilfsroutine zum rechtsbndigen Ausgeben *)

	PROCEDURE Format(str : TString; len : CARDINAL):TString;
	VAR help	: TString;
	BEGIN
          IF len > 0 THEN
            help:=String.Duplicate(str);
            String.Schreibe(str,"");
            String.Schreibe(space," ");
            WHILE String.Laenge(str)+String.Laenge(help) < len DO
              String.Anhaengen(space,str);
            END (* WHILE *);
            String.Anhaengen(help,str);
            String.Dispose(help);
          END (* IF *); 
	  RETURN str;
	END Format;
BEGIN
	strListe:=Stringliste.Create();
	IF First(inKondenOrdner,aktKond) THEN
	  space:=String.Create();
	  komma:=String.Create();
	  null:=String.Create();
	  String.Schreibe(space," ");
	  String.Schreibe(komma,",");
	  String.Schreibe(null,"0");
	  REPEAT
	    IF Kondition.LiesEntitaetsTyp(aktKond)=inET THEN
	      aktKondStr:=String.Create();
	      str:=KondenIMPORTE.HID2Str4SL(Kondition.LiesKID(aktKond),5);
              String.Anhaengen(str,aktKondStr);
              String.Anhaengen(space,aktKondStr);
	      String.Dispose(str);
	      str:=Datum.DatumAlsString(Kondition.LiesGueltigVon(aktKond));
	      String.Anhaengen(str,aktKondStr);
              String.Anhaengen(space,aktKondStr);
	      String.Dispose(str); 
	      str:=Datum.DatumAlsString(Kondition.LiesGueltigBis(aktKond));
	      String.Anhaengen(str,aktKondStr);
              String.Anhaengen(space,aktKondStr);
	      CASE Kondition.LiesEntitaetsTyp(aktKond) OF
		 Tarif: SS:=Kondition.LiesStundensatz(aktKond);
			WholeStr.CardToStr(TRUNC(FLOAT(SS)/100.0),CharArray);
		        String.Schreibe(str,CharArray);
			String.Anhaengen(komma,str);
			String.Anhaengen(Format(str,3),aktKondStr);
			frak:=TRUNC((FLOAT(SS)/100.0-FLOAT(SS/100))*100.0+0.5);
			String.Schreibe(str,"");
			WholeStr.CardToStr(frak,CharArray);
			IF frak=0 THEN 
				String.Anhaengen(null,str);
				String.Anhaengen(null,str);
			ELSE
				String.Schreibe(str,CharArray);
			END (* IF *);
			String.Anhaengen(str,aktKondStr); 
              		String.Anhaengen(space,aktKondStr);
			String.Dispose(str);
		|Vertrag:
			WholeStr.CardToStr( 
			  Kondition.LiesBHD(aktKond),CharArray );
			String.Schreibe(str,CharArray);
                        String.Anhaengen(Format(str,5),aktKondStr);
              		String.Anhaengen(space,aktKondStr);
			WholeStr.CardToStr(
			  Kondition.LiesStundenobergrenze(aktKond),CharArray );
                        String.Schreibe(str,CharArray);
                        String.Anhaengen(Format(str,5),aktKondStr);
              		String.Anhaengen(space,aktKondStr);
                        String.Dispose(str);
		|Weihnacht:
			WholeStr.CardToStr( 
			  Datum.MonatAlsCard(
                          Kondition.LiesBemessungsmonat(aktKond)),CharArray );
                        String.Schreibe(str,CharArray);
                        String.Anhaengen(Format(str,5),aktKondStr);
              		String.Anhaengen(space,aktKondStr);
                        String.Dispose(str);  
			str:=Datum.StagAlsString(
				Kondition.LiesStichtag(aktKond));
                        String.Anhaengen(str,aktKondStr);
                        String.Dispose(str);
	      END (* CASE *);
	      Stringliste.Append(strListe,aktKondStr);
	      String.Dispose(aktKondStr);
	    END (* IF *);
	  UNTIL NOT Next(inKondenOrdner,aktKond);
	  String.Dispose(space);
	  String.Dispose(komma);
	  String.Dispose(null);
	  Kondition.Dispose(aktKond);
	END (* IF *);	
	RETURN strListe;
		
		
END GibKondAlsStringliste;


END KondenOrdSchab.
