(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Konditioninformations-System 'HiKS'                     *)
(*
# $RCSfile: KondenOrdner.mod,v $
# $Author: Sopra $
# $Date: 2001/03/15 08:44:55 $
# $Revision: 1.7 $
# Bemerkungen: 
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KondenOrdner;

(* ADO KonditionenOrdner *)

(* IMPORTE *)

IMPORT KondenOrdSchab, String, Stringliste, Datum, Kondition,InOut;
IMPORT WholeStr;
FROM SYSTEM IMPORT CAST;

TYPE TString = String.TString;
     TStringliste = Stringliste.TStringliste;
     TDatum=Datum.TDatum;
 
(*
SEMANTIK
Die Semantik wird mit Hilfe dreier lokaler Integervariablen kap, 
pos und maxkap spezifiziert. Die Variablen kap und pos beschreiben  
den aktuellen Zustand (aktuelle Position und Anzahl der enthaltenen
Elemente) und  maxkap > 0 die Maximalkapazitaet des Ordners, 
d.h. es gilt stets:
            0 <= kap <= maxkap   und    0 <= pos <= kap+1

*)

VAR KondenOrdner: TKondOrdner;
    BackupKondenOrdner: TKondOrdner;


PROCEDURE CreateKondenOrdner () ;
(*
Vb:	keine
Erzeugt und initialisiert das ADO KondenOrdner. 
Reloads des Systems erzeugt.
Nb:	Das ADO KondenOrdner existiert ist leer
*)

BEGIN
	KondenOrdner:=KondenOrdSchab.Create();
END CreateKondenOrdner;

PROCEDURE CreateBackupKondenOrdner () ;
(*
Vb:	keine
Erzeugt und initialisiert das ADO BackupKondenOrdner. 
Wir fr Reloads des Systems bentigt.
Nb:	Das ADO BackupKondenOrdner existiert und ist leer
*)

BEGIN
	BackupKondenOrdner:=KondenOrdSchab.Create();
END CreateBackupKondenOrdner;


PROCEDURE Dispose ();
(*
Vb:	keine
Entfernt alle Realisierungs-ADOs, die in KondOrdner enthalten sind.
Entfernt das ADO KondOrdner aus dem System
Nb:	ADO KondenOrdner existiert nicht mehr
*)

BEGIN
	KondenOrdSchab.Dispose(KondenOrdner);
END Dispose;

PROCEDURE BackupDispose ();
(*
Vb:	keine
Entfernt alle Realisierungs-ADOs, die in BackupKondenOrdner enthalten sind.
Entfernt das ADO BackupKondOrdner aus dem System
Nb:	ADO BackupKondenOrdner existiert nicht mehr
*)

BEGIN
	KondenOrdSchab.Dispose(BackupKondenOrdner);
END BackupDispose;

PROCEDURE AktiviereRestore();
(*
Vb:	KondenOrdner und BackupOrdner existieren
nach einem erfolgreichen Restore wird der BackupOrdner zum KondenOrdner
Nb:	KondenOrdner=NIL: KondenOrdner:=BackupOrdner
*)
BEGIN
	Dispose();
	KondenOrdner:=BackupKondenOrdner;
	CreateBackupKondenOrdner;
END AktiviereRestore;
		

PROCEDURE IsEmpty () : BOOLEAN;
(*
VB:	keine
Test, ob der Ordner leer ist
Nb:	 IsEmpty (inKondOrdner) <=> kap = 0
*)
BEGIN
  RETURN KondenOrdSchab.IsEmpty (KondenOrdner);
END IsEmpty;


PROCEDURE Insert(inoutKond: TKond;
                 VAR outMessage : TString) : BOOLEAN;
        (* Semantik hat sich gegenueber der Schablone nicht geaendert *)

BEGIN
	RETURN KondenOrdSchab.Insert(KondenOrdner,inoutKond,outMessage);
END Insert;

PROCEDURE BackupInsert( inoutKond: TKond);
(* Das ist die Kurzfassung des Inserts der bei einem Reload des Systems *)
(* aufgerufen wird. Dabei ist nichts zu prfen und auch keine ID zu ver-*)
(* geben. *)
BEGIN
	KondenOrdSchab.BackupInsert(BackupKondenOrdner,inoutKond);	
END BackupInsert;



PROCEDURE Search( inKID : TKID; VAR outKond : TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *)
BEGIN 
	RETURN KondenOrdSchab.Search(KondenOrdner,inKID,outKond);
END Search;


PROCEDURE Update(inKond : TKond; VAR outMessage : TString):BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *)  
BEGIN
	RETURN KondenOrdSchab.Update(KondenOrdner,inKond ,outMessage );
END Update;


PROCEDURE First ( VAR outKond : TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *)
BEGIN
	RETURN KondenOrdSchab.First(KondenOrdner,outKond);
END First;


PROCEDURE Next(VAR inoutKond : TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *)
BEGIN
	RETURN KondenOrdSchab.Next(KondenOrdner,inoutKond);	
END Next;

(**************************** HiKS-Operationen ***************************)

PROCEDURE HoleGueltigeVKond(    inD: TDatum;
                                VAR outKond:TKond):BOOLEAN;
(*
vb: Konditionsordner existiert
Hole die zum Datum inD gltige Kondition vom Typ Vertrag. Falls es keine
solche Kondition gibt, wird FALSE returniert.
nb: --
*)
BEGIN
	RETURN KondenOrdSchab.HoleGueltigeKond( 
		KondenOrdner,inD,outKond,Vertrag); 
END  HoleGueltigeVKond;

PROCEDURE HoleGueltigeWKond(    inD: TDatum;
                                VAR outKond:TKond):BOOLEAN;
(*
vb: Konditionsordner existiert
Hole die zum Datum inD gltige Kondition vom Typ Weihnacht. Falls es keine
solche Kondition gibt, wird FALSE returniert.
nb: --
*)
BEGIN
	RETURN KondenOrdSchab.HoleGueltigeKond( 
		KondenOrdner,inD,outKond,Weihnacht); 
END  HoleGueltigeWKond;

PROCEDURE HoleGueltigeTKond(    inD: TDatum;
                                VAR outKond:TKond):BOOLEAN;
(*
vb: Konditionsordner existiert
Hole die zum Datum inD gltige Kondition vom Typ Tarif. Falls es keine
solche Kondition gibt, wird FALSE returniert.
nb: --
*)
BEGIN
	RETURN KondenOrdSchab.HoleGueltigeKond( 
		KondenOrdner,inD,outKond,Tarif); 
END  HoleGueltigeTKond;

PROCEDURE VKonditionVorhanden() : BOOLEAN;
(*
Vb:     keine
Liefert TRUE falls es eine Kondition vom Typ Vertrag gibt
Nb:     keine
*) 
BEGIN
	 RETURN KondenOrdSchab.KonditionVorhanden( KondenOrdner,Vertrag);
END VKonditionVorhanden;

PROCEDURE TKonditionVorhanden() : BOOLEAN;
(*
Vb:     keine
Liefert TRUE falls es eine Kondition vom Typ Tarif gibt
Nb:     keine
*) 
BEGIN
	 RETURN KondenOrdSchab.KonditionVorhanden( KondenOrdner,Tarif);
END TKonditionVorhanden;

PROCEDURE WKonditionVorhanden() : BOOLEAN;
(*
Vb:     keine
Liefert TRUE falls es eine Kondition vom Typ Weihnacht gibt
Nb:     keine
*) 
BEGIN
	 RETURN  KondenOrdSchab.KonditionVorhanden( KondenOrdner,Weihnacht);
END WKonditionVorhanden;

PROCEDURE BerechneBrutto(inGueltigVon: TDatum; inGueltigBis: TDatum;
                         inStundenzahl: INTEGER): INTEGER;

(* 
   Gueltige Kondition ermitteln
   FOR alle TK in Tarifkonditionen mit                                                 TK.GueltigVon <= V.GueltigBis AND
       (V.GueltigVon <= TK.GueltigBis OR
        TK.GueltigBis = UNDEF) DO
     Anfang := MAX (TK.GueltigVon, V.GueltigVon);
     Ende := MIN(TK.GueltigBis, V.GueltigBis);
     Wochen := (Ende - nfang).Wochen;
     Betrag += TK.Stundensatz * V.Stundenzahl * Wochen;
   END (* FOR *)
   S. 46 Prozess 3.6.1.2, ab (* Gueltige Kond. ermittlen ..*)
   bis Betrag+= TK.Stundensatz * V.STundenzahl * Wochen;
*) 
VAR 	Anfang,Ende	: TDatum;
	Wochen		: CARDINAL;  
        Betrag		: INTEGER;
	aktKond		: TKond;	
BEGIN
  Betrag:=0; 
  IF FirstTKondImZeitraum(inGueltigVon,inGueltigBis,aktKond) THEN
    REPEAT
      Anfang:=Datum.MaxDatum(Kondition.LiesGueltigVon(aktKond),inGueltigVon);
      Ende:=Datum.MinDatum(Kondition.LiesGueltigBis(aktKond),inGueltigBis);
      Wochen:=Datum.AnzahlWochen(Anfang,Ende);
      Betrag:=Betrag + CAST(INTEGER,Kondition.LiesStundensatz(aktKond)) *
		       inStundenzahl * CAST(INTEGER,Wochen);
    UNTIL NOT NextTKondImZeitraum(inGueltigVon,inGueltigBis,aktKond);
    Kondition.Dispose(aktKond);
  END (* IF *);
  RETURN Betrag;

END BerechneBrutto;


PROCEDURE PruefeVKondInZeitr(   inGueltigVon    : TDatum;
                                inGueltigBis    : TDatum;
                                Stunden         : CARDINAL;
                                Monate          : CARDINAL;  
				VAR outMessage	: TString):BOOLEAN;
(*
  FOR alle VK in Vertragskonditionen mit
       VK.GueltigVon <= V.GueltigBis AND
       (V.GueltigVon <= VK.GueltigBis OR VK.GueltigBis = UNDEF)
     OK := OK AND (Stunden <= VK:Stundenobergrenze AND
     MONATE <= VK.Beschaeftigungshoechstdauer)
   END
   S. 45:ab  OK:= TRUE, FOR alle VK ... bis OK:= OK AND....
*)

VAR     ok              : BOOLEAN;
        aktKond         : TKond;
	str		: TString;
	CharArray	: ARRAY[1..10] OF CHAR;
BEGIN
  ok:=FALSE;
  IF FirstVKondImZeitraum(inGueltigVon,inGueltigBis,aktKond) THEN
    ok:=TRUE;
    LOOP 
      IF Stunden > Kondition.LiesStundenobergrenze(aktKond) THEN
	str:=String.Create();
        outMessage:=String.Create();
        String.Schreibe(outMessage,"Stundenobergrenze in Kondition vom ");
	String.Anhaengen(Datum.DatumAlsString(
		Kondition.LiesGueltigVon(aktKond)), outMessage);
	String.Schreibe(str," ist : ");
        String.Anhaengen(str, outMessage);
	WholeStr.CardToStr(Kondition.LiesStundenobergrenze(aktKond),CharArray);
	String.Schreibe(str,CharArray);
	String.Anhaengen(str,outMessage);
	String.Dispose(str);
	ok:=FALSE;
      END (* IF *);
      IF (Monate > Kondition.LiesBHD(aktKond)) AND ok THEN
	str:=String.Create();
	outMessage:=String.Create(); 
	String.Schreibe(outMessage,"BHD in Kondition vom ");
	String.Anhaengen(Datum.DatumAlsString(
		Kondition.LiesGueltigVon(aktKond)), outMessage);   
	String.Schreibe(str, " ist : ");
	String.Anhaengen(str, outMessage);  
        WholeStr.CardToStr(Kondition.LiesBHD(aktKond),CharArray);
        String.Schreibe(str,CharArray);
	String.Anhaengen(str,outMessage); 
	String.Dispose(str); 
	ok:=FALSE;
      END (* IF *); 
      IF NOT NextVKondImZeitraum(inGueltigVon,inGueltigBis,aktKond) OR
	NOT ok THEN
		EXIT;
      END (* IF *);
    END (* LOOP *);
    Kondition.Dispose(aktKond);
  ELSE
    outMessage:=String.Create();
    String.Schreibe(outMessage,"Keine Vertragskonditionen vorhanden!");
  END (* IF *);
  RETURN ok;
END PruefeVKondInZeitr; 



PROCEDURE FirstVKondImZeitraum( inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR outKond: TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
	RETURN KondenOrdSchab.FirstKondImZeitraum(
	KondenOrdner,inGueltigVon,inGueltigBis,outKond,Vertrag);
END FirstVKondImZeitraum;

PROCEDURE FirstWKondImZeitraum( inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR outKond: TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
	RETURN KondenOrdSchab.FirstKondImZeitraum(
	KondenOrdner,inGueltigVon,inGueltigBis,outKond,Weihnacht);
END FirstWKondImZeitraum;

PROCEDURE FirstTKondImZeitraum( inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR outKond: TKond) : BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
	RETURN KondenOrdSchab.FirstKondImZeitraum(
	KondenOrdner,inGueltigVon,inGueltigBis,outKond,Tarif);
END FirstTKondImZeitraum;

PROCEDURE NextVKondImZeitraum(  inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR inoutKond:TKond):BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
        RETURN KondenOrdSchab.NextKondImZeitraum(
        KondenOrdner,inGueltigVon,inGueltigBis,inoutKond,Vertrag);  
END NextVKondImZeitraum;

PROCEDURE NextWKondImZeitraum(  inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR inoutKond:TKond):BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
        RETURN KondenOrdSchab.NextKondImZeitraum(
        KondenOrdner,inGueltigVon,inGueltigBis,inoutKond,Weihnacht);  
END NextWKondImZeitraum;

PROCEDURE NextTKondImZeitraum(  inGueltigVon,
                                inGueltigBis:TDatum;
                                VAR inoutKond:TKond):BOOLEAN;
(* Semantik hat sich gegenueber der Schablone nicht geaendert *) 
BEGIN
        RETURN KondenOrdSchab.NextKondImZeitraum(
        KondenOrdner,inGueltigVon,inGueltigBis,inoutKond,Tarif);  
END NextTKondImZeitraum;


PROCEDURE GibVKondAlsStringliste(): TStringliste;
       (* Gib die Weihnachtsgeld-Konditionen mit folgender Formatierung als
          Stringliste aus; Formatierung: GueltigVon (10Zeichen); 1 Leerzeichen;
          GueltigBis(10Zeichen); 1 Leerzeichen; Bemessungsmonat(2Zeichen);
          1 Leerzeichen; Stichtag(2Zeichen); insgesamt also 27 Zeichen *)
BEGIN
	RETURN KondenOrdSchab.GibKondAlsStringliste(KondenOrdner,Vertrag);
END GibVKondAlsStringliste;	

PROCEDURE GibWKondAlsStringliste(): TStringliste;
       (* Gib die Vertrags-Konditionen mit folgender Formatierung als 
	  Stringliste aus; Formatierung: GueltigVon (10Zeichen); 
	  1 Leerzeichen; GueltigBis(10Zeichen); 1 Leerzeichen; 
	  Beschftigungshchstdauer(5Zeichen); 1 Leerzeichen;
          Stundenobergrenze(5Zeichen); insgesamt also 33 Zeichen *)  	
BEGIN
        RETURN KondenOrdSchab.GibKondAlsStringliste(KondenOrdner,Weihnacht);
END GibWKondAlsStringliste;
 
PROCEDURE GibTKondAlsStringliste(): TStringliste;
       (* Gib die Tarif-Konditionen mit folgender Formatierung als Stringliste
	  aus; Formatierung: GueltigVon (10Zeichen); 1 Leerzeichen; 
	  GueltigBis(10Zeichen); 1 Leerzeichen; Stundensatz(5Zeichen);
          insgesamt also 27 Zeichen *) 
BEGIN
        RETURN KondenOrdSchab.GibKondAlsStringliste(KondenOrdner,Tarif);
END GibTKondAlsStringliste;

 
BEGIN
	CreateKondenOrdner();
	CreateBackupKondenOrdner();
END KondenOrdner.
