(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Kondition.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:01 $
# $Revision: 1.1.1.1 $
Bemerkungen:

*)
(******************************************************************************)

IMPLEMENTATION MODULE (* -- Muster-ADT *) Kondition;
                                               (* ENTHALTEN IN Konditionen *)


  FROM Storage IMPORT ALLOCATE, DEALLOCATE;
  FROM SYSTEM IMPORT TSIZE;
  FROM GENERIC IMPORT TObjektreferenz;

  FROM KondenIMPORTE IMPORT 	EqualizeHID, UeberlasseAlteHID, 
				SmallerSeqNrHID, HID2Str,Str2HID,
			   	SameSeqNrHID, DisposeHID, DuplicateHID; 

  IMPORT String; 
  IMPORT Datum;

  TYPE TDatum=Datum.TDatum;
       TMonat=Datum.TMonat;
       TStag=Datum.TStag;

  TYPE TKonditionDS = 	RECORD
                          Identifikator    : RECORD
                                                  KID        : TKID;
                                                  existiert : BOOLEAN;
					     END;   
			  GueltigVon:TDatum;
			  GueltigBis:TDatum;
			  CASE EntitaetsTyp:TKondTyp OF
			    Vertrag:	BHD:	CARDINAL;
					SOG:	CARDINAL;
			    |
			    Tarif:	Stundensatz:	CARDINAL;
			    |
			    Weihnacht:	Bemessungsmonat:TMonat;
					Stichtag:	TStag;	
               		  END (* CASE *);           
			END (* RECORD *);
      TKondition    = POINTER TO TKonditionDS;

  PROCEDURE CreateKondition (ETyp: TKondTyp) : TKondition;
  (* -- Generiert und initialisiert ein neues leeres ADO K. ETyp legt den
	Subtyp fest *)
  VAR aktK : TKondition;
  BEGIN
    (* -- Vorbed.: keine *)
    ALLOCATE (aktK, TSIZE (TKonditionDS)) ;
    WITH aktK^ DO
       Identifikator.existiert := FALSE;
       GueltigVon := Datum.CreateDatum();
       GueltigBis := Datum.CreateDatum();  
       EntitaetsTyp := ETyp;
    END; (* WITH *)
    RETURN aktK
    (* -- Nachbed.: (Rueckgabe-ADO aktK existiert) AND NOT ExistiertID (aktK) *)
  END CreateKondition;

  PROCEDURE CreateVertragsKondition () : TKondition;
  (* -- Generiert und initialisiert ein neues leeres ADO K vom Typ Vertrag *)
  (*    Subtyp fest *)
  VAR aktK : TKondition;
  BEGIN
    (* -- Vorbed.: keine *)
    aktK:=CreateKondition(Vertrag);
    WITH aktK^ DO
      BHD:=0;
      SOG:=0;
    END; (* WITH *)
    RETURN aktK
    (* -- Nachbed.: (Rueckgabe-ADO aktK existiert) AND NOT ExistiertID (aktK) *)
  END CreateVertragsKondition;
              
  PROCEDURE CreateTarifKondition () : TKondition;
  (* -- Generiert und initialisiert ein neues leeres ADO K vom Typ Tarif *)
  (*    Subtyp fest *)
  VAR aktK : TKondition;
  BEGIN
    (* -- Vorbed.: keine *)
    aktK:=CreateKondition(Tarif);
    WITH aktK^ DO
      Stundensatz:=0;
    END; (* WITH *)
    RETURN aktK
    (* -- Nachbed.: (Rueckgabe-ADO aktK existiert) AND NOT ExistiertID (aktK) *)
  END CreateTarifKondition;   

  PROCEDURE CreateWeihnachtsKondition () : TKondition;
  (* -- Generiert und initialisiert ein neues leeres ADO K vom Typ Weihnacht *)
  (*    Subtyp fest *)
  VAR aktK : TKondition;
  BEGIN
    (* -- Vorbed.: keine *)
    aktK:=CreateKondition(Weihnacht);
    WITH aktK^ DO
      Bemessungsmonat:= Datum.CreateMonat();
      Stichtag:= Datum.CreateStag();
    END; (* WITH *)
    RETURN aktK
    (* -- Nachbed.: (Rueckgabe-ADO aktK existiert) AND NOT ExistiertID (aktK) *)
  END CreateWeihnachtsKondition;  

  PROCEDURE OBJEKTREFERENZ (inK : TKondition) : TObjektreferenz;
  (* -- Typtransfer von TKondition nach TObjektreferenz *)
  VAR KRef : TObjektreferenz;
  BEGIN
    (* -- Vorbed.: keine *)
    KRef := inK;
    RETURN KRef
     (* -- Nachbed.: inK = KUERAUFTRAG (KRef)  *)
  END OBJEKTREFERENZ;

  PROCEDURE KONDITION (inRef : TObjektreferenz) : TKondition;
  (* -- Typtransfer von TObjektreferenz nach TKondition *)
  VAR aktK : TKondition;
  BEGIN
    (* -- Vorbed.: inRef referenziert ein Kondition-Objekt *)
    aktK := inRef;
    RETURN aktK
    (* -- Nachbed.: Referenz von aktK = inRef *)
  END KONDITION;

  PROCEDURE ExistiertKID (inK : TKondition) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob inK bereits durch eine ID gekennzeichnet ist
     -- Bezeichne exists den boole'schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: exists <=> inK.ID existiert *)
  BEGIN
	RETURN inK^.Identifikator.existiert;
  END ExistiertKID;                                                 
	

  PROCEDURE Dispose (VAR inoutK : TKondition) ;
  (* -- Entfernt das ADO inoutK aus dem System *)
  BEGIN
    (* -- Vorbed.: keine *)
    WITH inoutK^ DO
       IF Identifikator.existiert THEN
          DisposeHID(Identifikator.KID)
       END;
       Datum.DisposeDatum(GueltigVon);
       Datum.DisposeDatum(GueltigBis);
       IF EntitaetsTyp=Weihnacht THEN
	 Datum.DisposeStag(Stichtag);
	 Datum.DisposeMonat(Bemessungsmonat);
       END (* IF *);
    END; (* WITH *)
    DEALLOCATE (inoutK, TSIZE (TKonditionDS));
    inoutK := NIL
    (* -- Nachbed.: ADO inoutK existiert nicht mehr *)
  END Dispose;

  PROCEDURE Duplicate (inK : TKondition) : TKondition;
  (* -- Erzeugt ein neues ADO Kondition und gleicht dieses an den 
     -- Zustand bzw. an die Attribute von inK an.  *)
  VAR aktDuplikat : TKondition;
  BEGIN
    (* -- Vorbed.: keine *)
    CASE inK^.EntitaetsTyp OF
       Vertrag: 	aktDuplikat := CreateVertragsKondition( );
      |Tarif:		aktDuplikat := CreateTarifKondition( );
      |Weihnacht:	aktDuplikat := CreateWeihnachtsKondition( );
    END (* CASE *);
    Equalize(aktDuplikat, inK);
    RETURN aktDuplikat
    (* -- Nachbed.: IsEqual (aktDuplikat, inK) AND NOT (aktDuplikat = inK) *)
  END Duplicate;

  PROCEDURE IsEqual (inK1, inK2 : TKondition) : BOOLEAN;
  (* -- Ueberprueft die beiden Objekte auf Zustands-/Wertegleichheit, 
     -- d.h. die Gleichheit aller korrespondierenden Attribute.  *)
  VAR gleicheAttribute,
      gleicheID ,
      gleicherEntitaetsTyp: BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)
    IF inK1^.EntitaetsTyp =inK2^.EntitaetsTyp THEN
      gleicherEntitaetsTyp:=TRUE;
    ELSE
      gleicherEntitaetsTyp:=FALSE;
    END; (* IF *)  
    IF inK1^.Identifikator.existiert AND inK1^.Identifikator.existiert THEN
      gleicheID := SameKondition (inK1, inK2)
    ELSIF inK1^.Identifikator.existiert OR inK1^.Identifikator.existiert THEN
      gleicheID := FALSE
    ELSE
      gleicheID := TRUE
    END; (* IF *)
    IF gleicheID AND gleicherEntitaetsTyp THEN
      gleicheAttribute := 
             Datum.IsEqualDatum (inK1^.GueltigVon, inK2^.GueltigVon)
             AND Datum.IsEqualDatum (inK1^.GueltigBis, inK2^.GueltigBis);
	     CASE inK1^.EntitaetsTyp OF 
		Vertrag:  gleicheAttribute := gleicheAttribute AND 
					    	(inK1^.BHD=inK2^.BHD) AND
						(inK1^.SOG=inK2^.SOG);
		|
		Tarif:    gleicheAttribute := gleicheAttribute AND 
						(inK1^.Stundensatz=
						inK2^.Stundensatz);
		|
		Weihnacht:  gleicheAttribute := gleicheAttribute AND
    						Datum.IsEqualMonat(
						inK1^.Bemessungsmonat,
						inK2^.Bemessungsmonat) AND
						Datum.IsEqualStag(
						inK1^.Stichtag,
						inK2^.Stichtag	);
	     END (* CASE *);
    ELSE
      gleicheAttribute := FALSE
    END; (* IF *)
    RETURN gleicheAttribute
    (* -- Nachbed.: keine *)
  END IsEqual;


  PROCEDURE SameKondition (inK1, inK2 : TKondition) : BOOLEAN;
  (* -- Prueft, ob die beiden Konditionen gleichgross sind *)
  VAR same : BOOLEAN;
  BEGIN
    (* -- Vorbed.: ExistiertID (inK1) AND ExistiertID (inK2)  *)
      same := SameSeqNrHID (inK1^.Identifikator.KID,
                                   inK2^.Identifikator.KID);
    RETURN same
    (* -- Nachbed.: same <=> inK1 "=" inK2 *)
  END SameKondition;

  PROCEDURE SmallerKondition (inSmallerK, 
                                 inGreaterK : TKondition) : BOOLEAN;
  (* -- Prueft, ob die Kondition SmallerK kleiner ist als die 
     -- Konditions GreaterK. *)
  VAR smaller : BOOLEAN;
  BEGIN
    (* -- Vorbed.: ExistiertID (inSmallerK) AND ExistiertID (inGreaterK) *)
    smaller := SmallerSeqNrHID (inSmallerK^.Identifikator.KID,
                                  inGreaterK^.Identifikator.KID);
    RETURN smaller
    (* -- Nachbed.: smaller <=> inSmallerK "<" inGreaterK *)
  END SmallerKondition;


  PROCEDURE Equalize (VAR inoutZielK, inQuellK : TKondition);
  (* -- Gleicht den Zustand /die Attribute von inoutZielK an inQuellK
     -- an, wobei die Identitaet der ADOs nicht veraendert wird. Die 
	SubTypen mssen NICHT gleich sein! D.h. der Subtyp der Zielkondition
	wird der Quellkondition angepasst *)
  BEGIN
  (* -- Vorbed.: keine *)
    WITH inoutZielK^ DO
         WITH Identifikator DO
           IF existiert AND inQuellK^.Identifikator.existiert THEN
             EqualizeHID (KID, inQuellK^.Identifikator.KID)
           ELSIF existiert THEN
             (* -- inQuellK^.Identifikator.existiert = FALSE *)
             existiert := FALSE;
             DisposeHID(KID);
           ELSIF inQuellK^.Identifikator.existiert THEN
             (* -- inoutZielK^.Identifikator.existiert = FALSE *)
             KID := DuplicateHID (inQuellK^.Identifikator.KID);
             existiert := TRUE
           ELSE 
             (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
           END (* IF *)
         END; (* WITH *)
         Datum.EqualizeDatum(GueltigVon,inQuellK^.GueltigVon);
         Datum.EqualizeDatum(GueltigBis,inQuellK^.GueltigBis); 
         EntitaetsTyp:=inQuellK^.EntitaetsTyp;
	 CASE EntitaetsTyp OF
           Vertrag:  	BHD:=inQuellK^.BHD;
  	             	SOG:=inQuellK^.SOG;
           |
           Tarif:    	Stundensatz:=inQuellK^.Stundensatz;
           |
           Weihnacht:   IF Bemessungsmonat = NIL THEN 
				Bemessungsmonat:=Datum.CreateMonat();
			END (* IF *);
			Datum.EqualizeMonat(Bemessungsmonat,
                                  inQuellK^.Bemessungsmonat);
			IF Stichtag = NIL THEN
				Stichtag:=Datum.CreateStag();
			END (* IF *);
                       	Datum.EqualizeStag( Stichtag,
                                  inQuellK^.Stichtag  );
         END (* CASE *);
    END; (* WITH *);
   (* -- Nachbed.: IsEqual (inQuellK, inoutZielK) *)
  END Equalize;

  PROCEDURE LiesEntitaetsTyp (inK : TKondition) : TKondTyp;
  (* -- Vorbed.: keine
     -- Liefert die KontoNr des Auftragskontos von inK
     -- Nachbed.: RueckgabeADO KontoNr existiert *)    
  BEGIN
	RETURN inK^.EntitaetsTyp;
  END LiesEntitaetsTyp;

  PROCEDURE SetzeKID (VAR inoutK : TKondition; inKID : TKID);
  (* -- Kennzeichnet inoutK mit inKID *)
  BEGIN
    (* -- Vorbed.: NOT ExistiertID (inoutK)  *)
    WITH inoutK^.Identifikator DO
      KID := DuplicateHID (inKID);
      existiert := TRUE
    END (* WITH *)
    (* -- Nachbed.: ExistiertID (inoutK) *)
  END SetzeKID;

  PROCEDURE LiesKID (inK : TKondition) : TKID;
  (* -- Liefert die KID von inK *)
  VAR aktID : TKID;
  BEGIN
    (* -- Vorbed.: ExistiertID (inK) *)
    aktID := DuplicateHID (inK^.Identifikator.KID);
    RETURN aktID
    (* -- Nachbed.: RueckgabeADO KID existiert *)
  END LiesKID;

  PROCEDURE LiesGueltigVon (inK : TKondition) : TDatum;
  (* -- Vorbed.: keine
     -- Liefert den Gueltigkeitsbeginn einer Kondition
     -- Nachbed.: -- *)
  VAR D: TDatum;
  BEGIN
    D := Datum.CreateDatum();
    Datum.EqualizeDatum(D,inK^.GueltigVon);
    RETURN D;
  END LiesGueltigVon;

  PROCEDURE SetzeGueltigVon (VAR inoutK : TKondition; inGV: TDatum);
  (* -- Vorbed.: keine
     -- Setzt den Gueltigkeitsbeginn einer Kondition auf inGV
     -- Nachbed.: -- *)
  BEGIN
    Datum.EqualizeDatum(inoutK^.GueltigVon,inGV);
  END SetzeGueltigVon; 

  PROCEDURE LiesGueltigBis (inK : TKondition) : TDatum;
  (* -- Vorbed.: keine
     -- Liefert das Gueltigkeitsende einer Kondition
     -- Nachbed.: RueckgabeADO Datum existiert *)
  VAR D: TDatum;
  BEGIN
    D := Datum.CreateDatum();
    Datum.EqualizeDatum(D,inK^.GueltigBis);
    RETURN D;
  END LiesGueltigBis; 

  PROCEDURE SetzeGueltigBis (VAR inoutK : TKondition; inGB: TDatum);
  (* -- Vorbed.: keine
     -- Setzt das Gueltigkeitsende einer Kondition auf inGB
     -- Nachbed.: -- *)
  BEGIN
    Datum.EqualizeDatum(inoutK^.GueltigBis,inGB);
  END SetzeGueltigBis;  
                      
  PROCEDURE SetzeBHD (VAR inoutK : TKondition; inBHD : CARDINAL);
  (* -- Vorbed.: keine
     -- Setzt die Beschaeftigungshoechstdauer von inoutK fest
     -- Nachbed.: inoutK.BHD = inBHD *)
  BEGIN
    inoutK^.BHD:=inBHD;
  END SetzeBHD;

  PROCEDURE LiesBHD (inK : TKondition) : CARDINAL;
  (* -- Vorbed.: keine
     -- Liefert die Beschaeftigungshoechstdauer von inK
     -- Nachbed.: keine *)
  BEGIN
    RETURN inK^.BHD;
  END LiesBHD;

  PROCEDURE SetzeStundenobergrenze (VAR inoutK : TKondition; inSOG : CARDINAL);
  (* -- Vorbed.: keine
     -- Setze die Stundenobergrenze auf inSOG
     -- Nachbed.: inoutK.SOG= inSOG *)
  BEGIN
    inoutK^.SOG:=inSOG;
  END SetzeStundenobergrenze;

  PROCEDURE LiesStundenobergrenze (inK : TKondition) : CARDINAL;
  (* -- Vorbed.: keine
     -- Setze die Stundenobergrenze auf inSOG
     -- Nachbed.: keine *)
  BEGIN
    RETURN inK^.SOG;
  END LiesStundenobergrenze;

  PROCEDURE SetzeStundensatz (VAR inoutK : TKondition;inSS : CARDINAL);
  (* -- Vorbed.: keine
     -- Liefert den Stundensatz von inK
     -- Nachbed.: inoutK.SS:=inSS *)
  BEGIN
    inoutK^.Stundensatz:=inSS;
  END SetzeStundensatz;

  PROCEDURE LiesStundensatz (inK : TKondition) : CARDINAL;
  (* -- Vorbed.: keine
     -- Liefert den Stundensatz von inK
     -- Nachbed.: -- *)
  BEGIN
    RETURN inK^.Stundensatz;
  END LiesStundensatz;

  PROCEDURE SetzeBemessungsmonat (VAR inoutK : TKondition; inBM : TMonat);
  (* -- Vorbed.: keine
     -- Setze den Bemessungsmonat auf inBM
     -- Nachbed.: inoutK.BM= inBM *)
  BEGIN
    Datum.EqualizeMonat(inoutK^.Bemessungsmonat, inBM);
  END SetzeBemessungsmonat;

  PROCEDURE LiesBemessungsmonat (inK : TKondition) : TMonat;
  (* -- Vorbed.: keine
     -- Liefert den Bemessungsmonat von inK
     -- Nachbed.: ADO vom Typ TMonat existiert *)
  VAR M: TMonat;
  BEGIN
    M:=Datum.CreateMonat();
    Datum.EqualizeMonat(M,inK^.Bemessungsmonat);
    RETURN M;
  END LiesBemessungsmonat;
                                      

  PROCEDURE SetzeStichtag (VAR inoutK : TKondition; inST : TStag);
  (* -- Vorbed.: keine
     -- Setze den Stichtag fuer's Weihnachtsgeld auf inST
     -- Nachbed.: inoutK.Stichtag= inST *)
  BEGIN
    Datum.EqualizeStag(inoutK^.Stichtag,inST);
  END SetzeStichtag;

  PROCEDURE LiesStichtag (inK : TKondition) : TStag;
  (* -- Vorbed.: keine
     -- Liefert den Stichtag von inK
     -- Nachbed.: ADO vom Typ TStag existiert *)
  VAR S:TStag;
  BEGIN
    S:=Datum.CreateStag();
    Datum.EqualizeStag(S,inK^.Stichtag);
    RETURN S;
  END LiesStichtag;

  PROCEDURE FruehereGueltigkeit(infruehereK, inspaetereK: TKondition):BOOLEAN;
  (* -- Vorbed.: infruehereK,inspaetereK existieren
     -- Falls infruehereK.GueltigVon < inspaetereK.GueltigVon wird TRUE
        zurckgegeben
     -- Nachbed.: -- *)
  BEGIN
    RETURN Datum.SmallerDatum(infruehereK^.GueltigVon,
                              inspaetereK^.GueltigVon);
  END FruehereGueltigkeit; 

  PROCEDURE GleicheGueltigkeit(infruehereK, inspaetereK: TKondition):BOOLEAN;
  (* -- Vorbed.: infruehereK,inspaetereK existieren
     -- Falls infruehereK.GueltigVon = inspaetereK.GueltigVon wird TRUE
        zurckgegeben
     -- Nachbed.: -- *) 
  BEGIN
    RETURN Datum.IsEqualDatum(infruehereK^.GueltigVon,
                                inspaetereK^.GueltigVon);    
  END GleicheGueltigkeit;

  PROCEDURE KIDalsString(inKID:TKID; KIDLaenge: CARDINAL):TString;
  BEGIN
	RETURN HID2Str(inKID,KIDLaenge);
  END KIDalsString;

  PROCEDURE KIDvonString(inK:TString):TKID;
  BEGIN
	RETURN Str2HID(inK);
  END KIDvonString;
END Kondition.
