(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: KstID.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:05 $
# $Revision: 1.1.1.1 $
# Bemerkungen: 

Implementation: Schoenhammer Herbert

*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KstID;                    (* Enthalten in Kostenstellen *)

  FROM Storage IMPORT ALLOCATE, DEALLOCATE;
  FROM SYSTEM  IMPORT TSIZE;
  FROM GENERIC IMPORT TObjektreferenz;
  IMPORT String;
  FROM WholeStr IMPORT ConvResults, CardToStr, StrToCard;

  TYPE TKstIDDS = RECORD
                   ID : CARDINAL
                 END;
       TKstID   = POINTER TO TKstIDDS;

  TYPE TString = String.TString;


  PROCEDURE CreateKstID () : TKstID;
  VAR aktID : TKstID;
  BEGIN (* -- Create *)
    (* -- Vorbed.: keine *)
     ALLOCATE (aktID, TSIZE (TKstIDDS)) ;
     aktID^.ID := 0;
     RETURN aktID
    (* -- Nachbed.: RueckgabeADO aktID existiert *)
  END CreateKstID;


  PROCEDURE DisposeKstID (VAR inoutID : TKstID);
  (* -- Entfernt das ADO inoutID aus dem System *)
   BEGIN
    (* -- Vorbed.: keine *)
     DEALLOCATE (inoutID, TSIZE (TKstIDDS));
     inoutID := NIL; 
    (* -- Nachbed.: ADO inoutID existiert nicht mehr *)
   END DisposeKstID;

  PROCEDURE OBJEKTREFERENZ (inID : TKstID) : TObjektreferenz;
  (* -- Typtransfer von TKstID nach TObjektreferenz *)
  VAR IDRef : TObjektreferenz;
  BEGIN
    (* -- Vorbed.: keine *)
    IDRef := inID;
    RETURN IDRef
     (* -- Nachbed.: inID = DAUERAUFTRAG (IDRef)  *)
  END OBJEKTREFERENZ;

  
  PROCEDURE ID (inObjektreferenz : TObjektreferenz) : TKstID;
  (* -- Typtransfer von TObjektreferenz nach TKstID *)
  VAR aktID : TKstID;
  BEGIN
    (* -- Vorbed.: inObjektreferenz referenziert ein ID-Objekt *)
    aktID := inObjektreferenz;
    RETURN aktID
    (* -- Nachbed.: Referenz von aktDA = inObjektreferenz *)
  END ID;

  
  PROCEDURE DuplicateKstID (inQuellID : TKstID) : TKstID;
  (* -- Erzeugt ein neues ID-Objekt und gleicht dieses an den Zustand bzw. 
     -- die Attribute von inQuellID an. *)
  VAR Duplikat : TKstID;
  BEGIN
    (* -- Vorbed.: keine *)
    Duplikat := CreateKstID ();
    EqualizeKstID (Duplikat, inQuellID);
    RETURN Duplikat
    (* -- Nachbed.: IsEqual (Duplikat, inQuellID) 
       --           AND NOT (Duplikat = inQuellID) *)
  END DuplicateKstID;

  
  PROCEDURE IsEqualKstID (inID1, inID2 : TKstID) : BOOLEAN;
  (* -- Vergleicht inID1 und inID2 auf Zustandsgleichheit *)
  VAR gleich : BOOLEAN;
  BEGIN
    (* -- Vorbed.: keine *)
    gleich := (inID1^.ID = inID2^.ID);
    RETURN gleich
    (* -- Nachbed.: keine *)
  END IsEqualKstID;


  PROCEDURE EqualizeKstID (inoutZielID, inQuellID : TKstID);
  (* -- Gleicht den Zustand /die Attribute der ZielID an die QuellID an *)
  BEGIN
    (* -- Vorbed.: keine *)
    inoutZielID^.ID := inQuellID^.ID
    (* -- Nachbed.: IsEqual (inQuellID, inoutZielID) *)
  END EqualizeKstID;


  PROCEDURE SmallerKstID (inID1, inID2 : TKstID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, on inID1 kleiner als inID2 ist.
     -- Bezeichne smaller den booleschen Rueckgabeparameter, dann gilt:
     -- Nachbed.: smaller <=> (inID1 < inID2) *)
  VAR smaller : BOOLEAN;
  BEGIN
    smaller := (inID1^.ID < inID2^.ID);
    RETURN smaller
  END SmallerKstID;


  PROCEDURE SameKstID (inID1, inID2 : TKstID) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der Wert von inID1 gleich dem Wert von inID2 ist.
     -- Bezeichne same den booleschen Rueckgabeparameter, dann gilt:
     -- Nachbed.: same <=> (inID1 = inID2) *)
  VAR same : BOOLEAN;
  BEGIN
    same := (inID1^.ID = inID2^.ID);
    RETURN same
  END SameKstID;


  PROCEDURE KstIDAlsString (inKstID : TKstID) : TString; 
  (* -- Vorbed.: keine
     -- Wandelt die uebergebene inKstID in einen lesbareb String um
     -- Nachbed.: keine *)
  VAR SchluesselArray : ARRAY [0..255] OF CHAR;
      SchluesselStr : TString;
  BEGIN
    CardToStr (inKstID^.ID, SchluesselArray);
    SchluesselStr := String.Create();
    String.Schreibe (SchluesselStr, SchluesselArray);
    RETURN SchluesselStr
  END KstIDAlsString;


  PROCEDURE StringInKstID (inString : TString) : TKstID;
    (* -- Vorbed.: keine
     -- Wandelt die uebergebene inKstID in einen lesbareb String um
     -- Nachbed.: keine *)
  VAR aktKstID : TKstID;
      CharArray : ARRAY [1..255] OF CHAR;
      result : ConvResults;
      wert : CARDINAL;
  BEGIN
    aktKstID := CreateKstID();
    String.Lies (inString, CharArray);
    StrToCard(CharArray,wert,result);
    IF (result=strAllRight)
    THEN
      aktKstID^.ID := wert;
    ELSE
      aktKstID^.ID := 0;
    END;
    RETURN aktKstID
  END StringInKstID;


  PROCEDURE PruefeKstID (inString : TString; VAR outMessage : TString) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der inString in eine gueltige ID umgewandelt werden kann
     -- Nachbed.: keine *)
  VAR CharArray : ARRAY [1..255] OF CHAR;
      result : ConvResults;
      wert : CARDINAL;
      OK : BOOLEAN;
  BEGIN
    outMessage := String.Create ();
    String.Lies (inString, CharArray);
    StrToCard(CharArray,wert,result);
    CASE result OF
         strAllRight:    OK := TRUE;
                         String.Schreibe (outMessage, "");
        |strOutOfRange:  OK := FALSE;
                         String.Schreibe (outMessage, "Eingabe ausser Wertebereich");
        |strWrongFormat: OK := FALSE;
                         String.Schreibe (outMessage, "Eingabe in falschem Format");
        |strEmpty      : OK := FALSE;
                         String.Schreibe (outMessage, " Leere Eingabe");
    END; (* CASE *)
    RETURN OK
  END PruefeKstID;

 PROCEDURE IstGesetztKID (inKstID : TKstID ) : BOOLEAN;
 (* -- Vorbed.: keine
    -- Prft, od die KID von inKst im ADO inKst schon gesetzt wurde.
    -- Dies laesst keine Aussage darber zu, ob die ID bereits im Ordner
    -- enthalten ist. Dies wird mit ExistKst geprueft.
    -- Nachbed.: (TRUE <=> inKst.ID <> default-Wert)
                 (FALSE <=> inKst.ID = default-Wert) *)
 BEGIN
   RETURN (inKstID^.ID <> 0)
 END IstGesetztKID;


END KstID.
