(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: KstenOrdner.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 14:56:56 $
# $Revision: 1.2 $
Bemerkungen:

IMPLEMENTATION MODULE KstenOrdner   ADT-Modul KostenstelleOrdnerSchablone

Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de
*)
(******************************************************************************)
 
IMPLEMENTATION MODULE KstenOrdner;   (* ENTHALTEN IN Kostenstellen TS_Ksten *)

  IMPORT KstOSchablone;

  FROM KstenIMPORTE IMPORT THandle;
  
  IMPORT String;
  FROM Stringliste IMPORT TStringliste;
  FROM Datum IMPORT TDatum;

  TYPE TKstenOrdner     = KstOSchablone.TKstenOrdner;
       TString = String.TString;




(* Lokale Datenstruktur als Modulgedaechtnis                          *)
(*====================================================================*)

   VAR KstenOrdner : TKstenOrdner;



(* ------------------------------------------------------------------
   -- StandardOperationen fuer Architektur ADO KostenstellenOrdner
   ------------------------------------------------------------------ *)



  PROCEDURE CreateKstenOrdner;
  (*    -- Generiert ein neues Architektur ADO KostenstellenOrdner *)
  BEGIN
    KstenOrdner := KstOSchablone.CreateKstenOrdner();
  END CreateKstenOrdner;


  (* -- GEMEINSAME VORBEDINGUNG FUER ALLE UEBRIGEN OPERATIONEN:  
     --     ADOs inListe bzw. inoutListe existieren *)

  (* PROCEDURE DisposeKstenOrdner (); *)
  (* -- Entfernt das ADO inoutListe aus dem System *)
  (* BEGIN
     KstOSchablone.DisposeKstenOrdner(KstenOrdner);
     END DisposeKstenOrdner; *)



	
 PROCEDURE InsertKst ( inK: TKst; VAR outMessage : TString) : BOOLEAN;
 (*
    vb: sei K die neu einzufgende Kostenstelle. Dann muss gelten: ADO Konditionsordner
	existiert and not isfull and KID ist zugewiesen

        Bei inVariante = TRUE wird die Kostenstelle inK gemaess der
        Prozessspezifikationen D3.2.2 in den KostenstellenOrdner eingefuegt; d.h.
        im Attribut EingerichtetAm wird das aktuelle HiKS-Tagesdatum gesetzt.

        Bei inVariante = FALSE wird die Kostenstelle unveraendert in den Ordner
        eingefuegt.

    nb: K ist in Kostenstellenordner eingefgt *)

 BEGIN
   RETURN KstOSchablone.InsertKst (inK, outMessage, KstenOrdner, TRUE)
 END InsertKst;


 PROCEDURE FirstKst (VAR outKst : TKst) : BOOLEAN;
 (* vb: Keine
	Liefere die erste Kostenstelle im Ordner zurck
    nb: -- *)
 BEGIN
   RETURN KstOSchablone.FirstKst (outKst, KstenOrdner)
 END FirstKst;

 
 PROCEDURE NextKst (inK:TKst; inoutK:TKst):BOOLEAN;
 (* vb: inK existiert in KostenstellenOrdner
	Suche inK im KostenstellenOrdner und falls ein Nachfolger outK existiert gib
	diesen zurck, false sonst.
    nb: -- *)
 BEGIN
   RETURN KstOSchablone.NextKst (inK, inoutK, KstenOrdner)
 END NextKst;


 PROCEDURE FirstLfdKst (VAR outKst : TKst) : BOOLEAN;
 (* Reini :
    gibt die erste Kst zurck, die weder liquidiert noch gesperrt ist *)
 BEGIN
   RETURN KstOSchablone.FirstLfdKst (outKst, KstenOrdner)
 END FirstLfdKst;


 PROCEDURE NextLfdKst (inoutKst : TKst) : BOOLEAN;
 (* Reini :
    gibt die naechste Kst zurck, die weder liquidiert noch gesperrt ist *)
 BEGIN
   RETURN KstOSchablone.NextLfdKst (inoutKst, KstenOrdner)
 END NextLfdKst;


 PROCEDURE FirstAktKst (VAR outKst : TKst) : BOOLEAN;
 (* Reini :
    gibt die erste Kst zurck, die nicht liquidiert ist *)
 BEGIN
   RETURN KstOSchablone.FirstAktKst (outKst, KstenOrdner)
 END FirstAktKst;


 PROCEDURE NextAktKst (inoutKst : TKst) : BOOLEAN;
 (* Reini :
    gibt die naechste Kst zurck, die nicht liquidiert ist *)
 BEGIN
   RETURN KstOSchablone.NextAktKst (inoutKst, KstenOrdner)
 END NextAktKst;


 PROCEDURE SearchKstZuKID ( inKID: TKstID; VAR outKst : TKst):BOOLEAN;
 (* vb: inKID ist gueltig
        Gib Kostenstellendatensatz der zu KID gehrt zurck
    nb: -- *)
 BEGIN
   RETURN KstOSchablone.SearchKstZuKID (inKID, outKst, KstenOrdner)
 END SearchKstZuKID;


 PROCEDURE BerechneEtatKst (inKstID: TKstID): INTEGER;
 (* vb: inKst existiert im Ordner
        Berechne Etat der Kostenstelle aus Kontingente * quivalente
    nb: Liefert das Produkt der Attribute  Kontingente * quivalente  *)
 BEGIN
   RETURN KstOSchablone.BerechneEtatKst (inKstID, KstenOrdner)
 END BerechneEtatKst;


 PROCEDURE AendernKst (inKst: TKst; VAR outMessage : TString): BOOLEAN;
 (* Gabi
    nderung vornehmen durch berschreiben der Attribute
    nicht gendert werden drfen die Attribute EingerichtetAm, LiquidiertAm,
    Name und Jahr sowie die ID*)
 BEGIN
    RETURN KstOSchablone.AendernKst (inKst, outMessage, KstenOrdner)
 END AendernKst;


 PROCEDURE AendernUndLiquidKst (inKst: TKst; VAR outMessage : TString): BOOLEAN;
 (* Gabi
    nderung vornehmen durch berschreiben der Attribute
    nicht gendert werden drfen die Attribute EingerichtetAm, LiquidiertAm,
    Name und Jahr sowie die ID*)
 BEGIN
    RETURN KstOSchablone.AendernUndLiquidKst (inKst, outMessage, KstenOrdner)
 END AendernUndLiquidKst;




 (* StringOperationen                                    *)
 (*======================================================*)


 PROCEDURE GibKstenAlsStringlisteInKsten(): TStringliste;
  (* -- Vorbed.: Kostenstellenordner existiert
     -- Erzeugt ein nues Rueckgabe-ADO strliste vom Typ TStringliste.
     -- In jedem String werden ausgewhlte Attribute einer Kostenstellen
     -- lesbar ausgegeben. Bercksichtigt werden nur nicht liquidierte
     -- Kostenstellen. Ausgabe mit Formatierung:
	  KstenID    ( 5 Zeichen); 1 Leerzeichen; 
        Name       (20 Zeichen); 1 Leerzeichen; 
        Jahr       ( 4 Zeichen); 1 Leerzeichen; 
        Sperrdatum wenn vorhanden, sonst Leerzeichen (10Zeichen);
	  insgesamt also 42 Zeichen
     -- Nachbed.: strliste existiert *)

 BEGIN
   RETURN KstOSchablone.GibKstenAlsStringlisteInKsten(KstenOrdner)
 END GibKstenAlsStringlisteInKsten;


PROCEDURE GibAllesAlsStringlisteInKsten(): TStringliste;
  (* -- Vorbed.: Kostenstellenordner existiert
     -- Erzeugt ein nues Rueckgabe-ADO strliste vom Typ TStringliste.
     -- In jedem String werden ausgewhlte Attribute einer Kostenstellen
     -- lesbar ausgegeben. Bercksichtigt werden alle
     -- Kostenstellen. Ausgabe mit Formatierung:
	  KstenID        ( 5 Zeichen); 1 Leerzeichen; 
        Name           (20 Zeichen); 1 Leerzeichen; 
        Jahr           ( 4 Zeichen); 1 Leerzeichen; 
        EingerichtetAm (10 Zeichen); 1 Leerzeichen;
        Sperrdatum wenn vorhanden, sonst Leerzeichen (10Zeichen); 1 Leerzeichen;
        Liquiddatum wenn vorhanden, sonst Leerzeichen (10Zeichen); 1 Leerzeichen
        Kontingent     (10 Zeichen); 1 Leerzeichen;
        Aequivalent    (10 Zeichen)
        insgesamt also ca. 86 Zeichen
     -- Nachbed.: strliste existiert *)

 BEGIN
   RETURN KstOSchablone.GibAllesAlsStringlisteInKsten(KstenOrdner)
 END GibAllesAlsStringlisteInKsten;



 (* Prozeduren zum Laden und Speichern des Teilsystems *)
 (*====================================================*)


 PROCEDURE LoadinKsten(inH:THandle; VAR outMessage : TString) : BOOLEAN;
 (* vb: Datei existiert und ist nicht leer und zum Lesen geoeffnet
        Lade den Inhalt des KostenStellenOrdners mit dem Inhalt der Datei auf die inH
	  verweist.
    nb: KostenstellenOrdner ist geladen *)
 BEGIN
   RETURN KstOSchablone.LoadinKsten (inH, outMessage, KstenOrdner)
 END LoadinKsten;


 PROCEDURE SaveinKsten(inH:THandle; VAR outMessage : TString) : BOOLEAN;	
 (* vb: Datei existiert und ist zum Schreiben geoeffnet
        Speichert den Inhalt des KostenstellenOrdner in der Datei auf die inH
	  verweist.
    nb: KostenstellenOrdner-Inhalt ist auf SekundaerSpeicher geschrieben	*)
 BEGIN
   RETURN KstOSchablone.SaveinKsten (inH, outMessage, KstenOrdner)
 END SaveinKsten;


 (* Architektur ADO KostenstellenOrdner (KstenOrdner) erstellen *)
 (*=============================================================*)

BEGIN

    CreateKstenOrdner;

END KstenOrdner.
