MODULE KstenTesten;

  IMPORT String;
  IMPORT Stringliste;

  IMPORT Ksten;    (* Kostenstellen *)

  IMPORT VDat;     (* Virtuelles Dateisystem *)

  FROM SysDat IMPORT LiesSystemDatumInSysDat, SetzeSystemDatumInSysDat;
  FROM Datum IMPORT TDatum, TJahr, LiesJahrInDatum, DatumVonString, DatumAlsString,
                    DisposeDatum, CreateDatum, SetzeDatumEternity, JahrAlsString,
                    DisposeJahr;
  FROM InOut IMPORT ReadCard, WriteCard, WriteString, WriteLn;
  FROM InOut IMPORT ReadLn, ReadString;
  FROM WholeStr IMPORT ConvResults, CardToStr, StrToCard;

  TYPE
       TKst             = Ksten.TKst;
       TKstID           = Ksten.TKstID;
       TString          = String.TString;
       TStringliste     = Stringliste.TStringliste;

 VAR Befehl : CARDINAL;
     MaxProcNr : CARDINAL;
     TestProcs       : ARRAY[0..40] OF PROC;


(**********************************************************************************)
(* Initialisierungen                                                              *)
(**********************************************************************************)


  PROCEDURE CreateKstAndInsert (inID : TString; inName : TString);
  VAR kst, kst2 : TKst;
      kstID : TKstID;
      outMessage, tempDatumString : TString;
      tempDatum : TDatum;
      tempJahr : TJahr;
      funkt : BOOLEAN;
      arrayofchar : ARRAY [1..255] OF CHAR;
  BEGIN

    kst := Ksten.CreateKst();
    kstID := Ksten.StringInKstID (inID);
    Ksten.SetzeKIDinKst (kstID, kst);
    Ksten.SetzeNameinKst (kst, inName);

    tempDatumString := String.Create();
    String.Schreibe (tempDatumString, "01.03.2001");

    outMessage := String.Create();
    IF DatumVonString (tempDatumString, tempDatum, outMessage)
    THEN
       tempJahr := LiesJahrInDatum (tempDatum);
       Ksten.SetzeJahrinKst (kst, tempJahr);
    ELSE
       WriteString ("Fehler bei Datumskonvertierung String -> Datum");
       WriteLn;
    END;
    String.Dispose (outMessage);


    IF Ksten.InsertKst (kst, outMessage)
    THEN WriteString ("-> Kostenstelle ");
         String.Lies (inID, arrayofchar) ;
         WriteString (arrayofchar);
         WriteString (" erfolgreich in den Ordner aufgenommen"); WriteLn;
    ELSE
         WriteString ("-> FEHLER: Kostenstelle ");
         String.Lies (inID, arrayofchar) ;
         WriteString (arrayofchar);
         WriteString (" nicht aufnehmbar"); WriteLn;
         String.Dispose (outMessage);
    END;

    Ksten.DisposeKst (kst);
    Ksten.DisposeKstID (kstID);

  END CreateKstAndInsert;

  PROCEDURE KstenInit;
  VAR aktStringID, aktStringName : TString;
  BEGIN

    WriteLn; WriteLn;
    WriteString ("Kostenstellen initialisieren"); WriteLn;
    WriteString ("============================"); WriteLn;


    aktStringID := String.Create();
    aktStringName := String.Create ();

    (* String.Schreibe ( aktStringID, "125"); *)
    String.Schreibe (aktStringID, "125");
    String.Schreibe (aktStringName, "Kostenstelle 125");
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Schreibe (aktStringID, "126");
    String.Schreibe (aktStringName, "Kostenstelle 126");
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Schreibe (aktStringID, "127");
    String.Schreibe (aktStringName, "Kostenstelle 127");
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Schreibe (aktStringID, "128");
    String.Schreibe (aktStringName, "Kostenstelle 128 Langname_Laenger_als_20_Zeichen");
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Schreibe (aktStringID, "129");
    String.Schreibe (aktStringName, "Kostenstelle 129");
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Schreibe (aktStringID, "130");
    String.Schreibe (aktStringName, "Kost 130");
    CreateKstAndInsert (aktStringID, aktStringName);

    (* Fehler provozieren *)
    CreateKstAndInsert (aktStringID, aktStringName);

    String.Dispose (aktStringID);
    String.Dispose (aktStringName);

  END KstenInit;


(**********************************************************************************)
(* Kostenstellen                                                                  *)
(**********************************************************************************)


 PROCEDURE KstenListeAusgeben;
 VAR KstenListe : TStringliste;
     ueberschrift : TString;
 BEGIN

    WriteString("******** Operation: Ksten.GibKstenAlsStringlisteInKsten **********");
    WriteLn;

    KstenListe := Ksten.GibKstenAlsStringlisteInKsten();
    ueberschrift := String.Create();
    String.Schreibe (ueberschrift, "KstID Name                 Jahr Sperrdatum");
    StringAusgeben (ueberschrift);
    String.Dispose (ueberschrift);

    KompletteListeAusgeben (KstenListe);
    Stringliste.Dispose (KstenListe);
 END KstenListeAusgeben;


 PROCEDURE KstenListeAllesAusgeben;
 VAR KstenListe : TStringliste;
     ueberschrift : TString;
 BEGIN

    WriteString("******** Operation: Ksten.GibAllesAlsStringlisteInKsten **********");
    WriteLn;

    KstenListe := Ksten.GibAllesAlsStringlisteInKsten();
    ueberschrift := String.Create();
    String.Schreibe (ueberschrift,
    "KstID Name                 Jahr Einr-Datum SperrDatum LiquiDatum Kontingent quivalent");
    StringAusgeben (ueberschrift);
    String.Dispose (ueberschrift);

    KompletteListeAusgeben (KstenListe);
    Stringliste.Dispose (KstenListe);
 END KstenListeAllesAusgeben;


 PROCEDURE KstSperren;
 VAR sperrDatumString, sperrKst, outMessage : TString;
     sperrDatum : TDatum;
     kstID : TKstID;
     kst : TKst;
     arrayofchar : ARRAY [1..255] OF CHAR;
 BEGIN

    WriteString("* Operationen: Ksten.SetzeGesperrtAminKst, SearchKstZuKID, AendernKst *");
    WriteLn;

     WriteString ("KstID der zu sperrenden Kst eingeben:");
     sperrKst := String.Create ();
     StringEingeben (sperrKst);
     kstID := Ksten.StringInKstID (sperrKst);

     WriteString ("SperrDatum der zu sperrenden Kst eingeben:");
     sperrDatumString := String.Create ();
     StringEingeben (sperrDatumString);

     (* !! Zum test die beiden ADO's schon einmal anlgene; ist aber falsch *)
     (* sperrDatum:=CreateDatum(); *)
     outMessage := String.Create();

     IF NOT DatumVonString (sperrDatumString, sperrDatum, outMessage)
     THEN
       WriteString ("-> Fehler: Ungueltiges Datum eingegeben"); WriteLn;
     ELSE
       IF Ksten.SearchKstZuKID (kstID, kst)
       THEN
           Ksten.SetzeGesperrtAminKst (kst, sperrDatum);
           IF Ksten.AendernKst (kst, outMessage)
           THEN
             WriteString ("-> Kostenstelle erfolgreich im Ordner gendert"); WriteLn;
                ELSE
             WriteString ("-> Fehler: Kostenstelle NICHT im Ordner gendert"); WriteLn;
             String.Dispose (outMessage);
           END;
           Ksten.DisposeKst (kst);
       ELSE
           WriteString ("-> Fehler: Kostenstelle nicht gefunden"); WriteLn;
       END;

       DisposeDatum (sperrDatum);
     END; (* IF NOT DatumVonString *)

     String.Dispose (sperrKst);
     String.Dispose (sperrDatumString);
     Ksten.DisposeKstID (kstID);

 END KstSperren;


 PROCEDURE KstLiquid;
 VAR sperrDatumString, sperrKst, outMessage : TString;
     sperrDatum : TDatum;
     kstID : TKstID;
     kst : TKst;
     arrayofchar : ARRAY [1..255] OF CHAR;
 BEGIN

    WriteString("* Operationen: Ksten.SetzeLiquidAminKst, SearchKstZuKID, AendernUndLiquidKst *");
    WriteLn;

     WriteString ("KstID der zu liquidierenden Kst eingeben:");
     sperrKst := String.Create ();
     StringEingeben (sperrKst);
     kstID := Ksten.StringInKstID (sperrKst);

     WriteString ("Liquidier_Datum der Kst eingeben:");
     sperrDatumString := String.Create ();
     StringEingeben (sperrDatumString);

     (* !! Zum test die beiden ADO's schon einmal anlgene; ist aber falsch *)
     (* sperrDatum:=CreateDatum(); *)
     outMessage := String.Create();

     IF NOT DatumVonString (sperrDatumString, sperrDatum, outMessage)
     THEN
       WriteString ("-> Fehler: Ungueltiges Datum eingegeben"); WriteLn;
     ELSE
       IF Ksten.SearchKstZuKID (kstID, kst)
       THEN
           Ksten.SetzeLiquidAminKst (kst, sperrDatum);
           IF Ksten.AendernUndLiquidKst (kst, outMessage)
           THEN
             WriteString ("-> Kostenstelle erfolgreich im Ordner gendert"); WriteLn;
                ELSE
             WriteString ("-> Fehler: Kostenstelle NICHT im Ordner gendert"); WriteLn;
             String.Dispose (outMessage);
           END;
           Ksten.DisposeKst (kst);
       ELSE
           WriteString ("-> Fehler: Kostenstelle nicht gefunden"); WriteLn;
       END;

       DisposeDatum (sperrDatum);
     END; (* IF NOT DatumVonString *)

     String.Dispose (sperrKst);
     String.Dispose (sperrDatumString);
     Ksten.DisposeKstID (kstID);

 END KstLiquid;


 PROCEDURE KstSetzeKonting;
 VAR suchKst, kontingentString, outMessage  : TString;
     kstID : TKstID;
     kst : TKst;
     wert : CARDINAL;
     arrayofchar : ARRAY [1..255] OF CHAR;
     result : ConvResults;
 BEGIN

    WriteString("* Operationen: Ksten.SetzeKontinginKst, SearchKstZuKID, AendernKst *");
    WriteLn;

     WriteString ("KstID der Kst eingeben:");
     suchKst := String.Create ();
     StringEingeben (suchKst);
     kstID := Ksten.StringInKstID (suchKst);

     WriteString ("Kontingent der Kst eingeben:");
     kontingentString := String.Create ();
     StringEingeben (kontingentString);
     String.Lies (kontingentString, arrayofchar) ;
     StrToCard(arrayofchar,wert,result);
     IF (result=strAllRight)
     THEN

       IF Ksten.SearchKstZuKID (kstID, kst)
       THEN
           Ksten.SetzeKontinginKst (kst, wert);
           IF Ksten.AendernKst (kst, outMessage)
           THEN
             WriteString ("-> Kostenstelle erfolgreich im Ordner gendert"); WriteLn;
           ELSE
             WriteString ("-> Fehler: Kostenstelle NICHT im Ordner gendert"); WriteLn;
             String.Dispose (outMessage);
           END;
           Ksten.DisposeKst (kst);
       ELSE
           WriteString ("-> Fehler: Kostenstelle nicht gefunden"); WriteLn;
       END;
     ELSE
           WriteString ("-> Fehler: Konvertierungsfehler String"); WriteLn;
     END; (* IF result *)

     String.Dispose (suchKst);
     String.Dispose (kontingentString);
     Ksten.DisposeKstID (kstID);

 END KstSetzeKonting;


 PROCEDURE KstSetzeAequiv;
 VAR suchKst, kontingentString, outMessage  : TString;
     kstID : TKstID;
     kst : TKst;
     wert : CARDINAL;
     arrayofchar : ARRAY [1..255] OF CHAR;
     result : ConvResults;
 BEGIN

    WriteString("* Operationen: Ksten.SetzeAequivinKst, SearchKstZuKID, AendernKst *");
    WriteLn;

     WriteString ("KstID der Kst eingeben:");
     suchKst := String.Create ();
     StringEingeben (suchKst);
     kstID := Ksten.StringInKstID (suchKst);

     WriteString ("Aequivalent der Kst eingeben:");
     kontingentString := String.Create ();
     StringEingeben (kontingentString);
     String.Lies (kontingentString, arrayofchar) ;
     StrToCard(arrayofchar,wert,result);
     IF (result=strAllRight)
     THEN

       IF Ksten.SearchKstZuKID (kstID, kst)
       THEN
           Ksten.SetzeAequivinKst (kst, wert);
           IF Ksten.AendernKst (kst, outMessage)
           THEN
             WriteString ("-> Kostenstelle erfolgreich im Ordner gendert"); WriteLn;
           ELSE
             WriteString ("-> Fehler: Kostenstelle NICHT im Ordner gendert"); WriteLn;
             String.Dispose (outMessage);
           END;
           Ksten.DisposeKst (kst);
       ELSE
           WriteString ("-> Fehler: Kostenstelle nicht gefunden"); WriteLn;
       END;
     ELSE
           WriteString ("-> Fehler: Konvertierungsfehler String"); WriteLn;
     END; (* IF result *)

     String.Dispose (suchKst);
     String.Dispose (kontingentString);
     Ksten.DisposeKstID (kstID);

 END KstSetzeAequiv;


 PROCEDURE KstBerechneEtat;
 VAR suchKst: TString;
     kstID : TKstID;
     kst : TKst;
     arrayofchar : ARRAY [1..255] OF CHAR;
     etat : CARDINAL;
 BEGIN

    WriteString("************ Operationen: Ksten.BerechneEtatKst *****************");
    WriteLn;

     WriteString ("KstID der auszulesenden Kst eingeben:");
     suchKst := String.Create ();
     StringEingeben (suchKst);
     kstID := Ksten.StringInKstID (suchKst);

     IF Ksten.SearchKstZuKID (kstID, kst)
     THEN
        etat := Ksten.BerechneEtatKst (kstID);
        CardToStr (etat, arrayofchar);
        WriteString ("-> Kst hat Etat : ");
        WriteString (arrayofchar);
        WriteLn;
        Ksten.DisposeKst (kst);
     ELSE
        WriteString ("-> Fehler: Kostenstelle nicht gefunden"); WriteLn;
     END;

     Ksten.DisposeKstID (kstID);
     String.Dispose (suchKst);
 END KstBerechneEtat;


 PROCEDURE KstPruefeKstID;
 VAR wert : CARDINAL;
     arrayofchar : ARRAY [1..255] OF CHAR;
     result : ConvResults;
     erfolgreich : BOOLEAN;
     outMessage, idString : TString;
 BEGIN

    WriteString("************** Operation: Ksten.PruefeKstID *********************");
    WriteLn;

     WriteString ("KstID der Kst eingeben:");
     idString := String.Create ();
     StringEingeben (idString);
     IF Ksten.PruefeKstID (idString, outMessage)
     THEN
       WriteString ("-> Format des Strings ist OK");
       WriteLn;
     ELSE
       WriteString ("-> Konvertierungsfehler: ");
       String.Lies (outMessage, arrayofchar);
       WriteString (arrayofchar);
       WriteLn;
     END; (* IF *)
     String.Dispose (idString);
     String.Dispose (outMessage);
 END KstPruefeKstID;


 PROCEDURE KstZuKID;
 VAR name, suchKID, jahr_str : TString;
     kstID : TKstID;
     kst : TKst;
     jahr : TJahr;
     arrayofchar : ARRAY [1..255] OF CHAR;
 BEGIN

    WriteString("*********** Operation: Ksten.SearchKstZuKID *********************");
    WriteLn;

     WriteString ("KstID der zu suchenden Kst eingeben:");
     suchKID := String.Create ();
     StringEingeben (suchKID);
     kstID := Ksten.StringInKstID (suchKID);
     IF Ksten.SearchKstZuKID (kstID, kst)
     THEN
       name := Ksten.LiesNameinKst (kst);
       String.Lies (name, arrayofchar);
       WriteString ("-> Kostenstelle gefunden. Name ist: ");
       WriteString (arrayofchar);
       WriteLn;

       jahr := Ksten.LiesJahrinKst (kst);
       jahr_str := JahrAlsString (jahr);
       String.Lies (jahr_str, arrayofchar);
       WriteString ("->                        Jahr ist: ");
       WriteString (arrayofchar);
       WriteLn;

       Ksten.DisposeKst (kst);
       DisposeJahr (jahr);
       String.Dispose (jahr_str);
    ELSE
      WriteString ("-> Fehler: Gesuchte Kostenstelle nicht gefunden");
      WriteLn;
    END;
 END KstZuKID;


 PROCEDURE LfdKstListen;
 VAR exist : BOOLEAN;
     stringID : TString;
     arrayofchar : ARRAY [1..255] OF CHAR;
     kst : TKst;
     kstID : TKstID;
 BEGIN

    WriteString("**** Operationen: Ksten.FirstLfdKst, NextLfdKst *****************");
    WriteLn;

     IF Ksten.FirstLfdKst (kst)
     THEN
       exist := TRUE;

       WriteString ("-> Lfd. KostenstellenID :");
       WHILE exist DO
          kstID := Ksten.LiesKIDinKst (kst);
          stringID := Ksten.KstIDAlsString (kstID);
          String.Lies (stringID, arrayofchar);
          WriteString (arrayofchar);
          WriteString (", ");
          exist := Ksten.NextLfdKst (kst);
      END; (* WHILE *)
      WriteLn;
      Ksten.DisposeKstID (kstID);
      String.Dispose(stringID);
      Ksten.DisposeKst (kst);
    ELSE
      WriteString ("-> Keine lfd. Kostenstelle gefunden");
      WriteLn;
   END;
 END LfdKstListen;


 PROCEDURE AktKstListen;
 VAR exist : BOOLEAN;
     stringID : TString;
     arrayofchar : ARRAY [1..255] OF CHAR;
     kst : TKst;
     kstID : TKstID;
 BEGIN

    WriteString("**** Operationen: Ksten.FirstAktKst, NextAktKst *****************");
    WriteLn;

     IF Ksten.FirstAktKst (kst)
     THEN
       exist := TRUE;

       WriteString ("-> Lfd. KostenstellenID :");
       WHILE exist DO
          kstID := Ksten.LiesKIDinKst (kst);
          stringID := Ksten.KstIDAlsString (kstID);
          String.Lies (stringID, arrayofchar);
          WriteString (arrayofchar);
          WriteString (", ");
          exist := Ksten.NextAktKst (kst);
      END; (* WHILE *)
      WriteLn;
      Ksten.DisposeKstID (kstID);
      String.Dispose(stringID);
      Ksten.DisposeKst (kst);
    ELSE
      WriteString ("-> Keine lfd. Kostenstelle gefunden");
      WriteLn;
   END;
 END AktKstListen;


 PROCEDURE AlleKstListen;
 VAR exist : BOOLEAN;
     stringID : TString;
     arrayofchar : ARRAY [1..255] OF CHAR;
     kst, outkst : TKst;
     kstID : TKstID;
 BEGIN

    WriteString("******* Operationen: Ksten.FirstKst, NextKst ********************");
    WriteLn;

     IF Ksten.FirstKst (kst)
     THEN
       exist := TRUE;

       WriteString ("-> KostenstellenID :");
       WHILE exist DO
          kstID := Ksten.LiesKIDinKst (kst);
          stringID := Ksten.KstIDAlsString (kstID);
          String.Lies (stringID, arrayofchar);
          WriteString (arrayofchar);
          WriteString (", ");
          outkst := Ksten.CreateKst();
          exist := Ksten.NextKst (kst, outkst);
            IF exist THEN
               Ksten.EqualizeKst (kst, outkst);
            END;
      END; (* WHILE *)
      WriteLn;
      Ksten.DisposeKstID (kstID);
      String.Dispose(stringID);
      Ksten.DisposeKst (kst);
      Ksten.DisposeKst (outkst);
    ELSE
      WriteString ("-> Keine Kostenstelle gefunden");
      WriteLn;
   END;
 END AlleKstListen;


  PROCEDURE KstenOrdnerSave();
  VAR  	H 	: VDat.THandleInVDat;
       	M 	: VDat.TModusInVDat;
	message : String.TString;
	Filename: String.TString;
  BEGIN
    WriteString("***************** Operation: Ksten.Save *************************");
    WriteLn;
    WriteString("Der gesamte Ordnerinhalt wird gespeichert:");
    WriteLn;
    M:=VDat.SetWriteInVDat();
    Filename:=String.Create();
    String.Schreibe(Filename,"kostenstellen.dat");
    IF VDat.OpenInVDat(H,Filename,M) THEN
      IF NOT Ksten.SaveinKsten(H,message) THEN
	StringAusgeben(message);
	String.Dispose(message);
      ELSE
	WriteString("KostenstellenOrdner ist gespeichert!");	
	WriteLn;
      END (* IF *);
      IF NOT VDat.CloseInVDat(H) THEN
	WriteString("Die Datei konnte nicht geschlossen werden!");
	WriteLn();
      END (* IF *);
    ELSE
      WriteString("Die Datei konnte nicht geoeffnet werden!");
      WriteLn;
    END (* IF *);
    WriteLn();
  END KstenOrdnerSave;



  PROCEDURE KstenOrdnerLoad();
  VAR   H       : VDat.THandleInVDat;
        M       : VDat.TModusInVDat;
        message : String.TString;
        Filename: String.TString;
  BEGIN
    WriteString("***************** Operation: Ksten.Load *************************");
    WriteLn;
    WriteString("Der Ordnerinhalt wird geladen:");
    WriteLn;
    M:=VDat.SetReadInVDat();
    Filename:=String.Create();
    String.Schreibe(Filename,"kostenstellen.dat");
    IF VDat.OpenInVDat(H,Filename,M) THEN
      IF NOT Ksten.LoadinKsten(H,message) THEN
        StringAusgeben(message);
        String.Dispose(message);
      ELSE
        WriteString("KostenstellenOrdner ist geladen!");
        WriteLn;
      END (* IF *);
      IF NOT VDat.CloseInVDat(H) THEN
        WriteString("Die Datei konnte nicht geschlossen werden!");
        WriteLn();
      END (* IF *);
    ELSE
      WriteString("Die Datei konnte nicht geoeffnet werden!");
      WriteLn;
    END (* IF *);
    WriteLn();
  END KstenOrdnerLoad;

(**********************************************************************************)
(* System                                                                         *)
(**********************************************************************************)

 PROCEDURE SystemSetzeDatum;
 VAR  systemdatum, outmessage1 : TString;
      datum : TDatum;
 BEGIN
   WriteString ("System-Datum eingeben:");
   systemdatum := String.Create ();
   StringEingeben (systemdatum);

   IF DatumVonString (systemdatum, datum, outmessage1) THEN
      SetzeSystemDatumInSysDat (datum);
      WriteString ("-> Systemdatum erfolgreich gesetzt");
      WriteLn;
      DisposeDatum (datum);
   ELSE
      WriteString ("-> Konvertierungsfehler Datum");
      WriteLn;
      String.Dispose (outmessage1);
   END; (* IF DatumVonString *)

   String.Dispose (systemdatum);
END SystemSetzeDatum;



(**********************************************************************************)
(* Oberflaeche                                                                    *)
(**********************************************************************************)

 PROCEDURE IhreWahl() : CARDINAL;
 VAR outEingabe : CARDINAL;
 BEGIN
   WriteString("Befehl Nr.: ");
   ReadCard(outEingabe);
   WriteLn;
   RETURN outEingabe
 END IhreWahl;


 PROCEDURE BefehlAuswerten(inBefehl : CARDINAL);
 BEGIN
    IF inBefehl < MaxProcNr THEN
      TestProcs [inBefehl];
    ELSE
      WriteString("Dies war eine falsche Eingabe.");
      WriteLn;
    END; (* IF *)
 END BefehlAuswerten;


 PROCEDURE Programmende;
 BEGIN
   WriteString("Programmende!");
   WriteLn
 END Programmende;

  PROCEDURE InitTestProcs;
  VAR ProcNr : CARDINAL;

     PROCEDURE ProcEintragen(inOperation : PROC);
     BEGIN
       TestProcs [ProcNr] := inOperation;
       INC (ProcNr)
     END ProcEintragen;

  BEGIN
    ProcNr := 0;
    ProcEintragen (Programmende);
    ProcEintragen (KstenListeAusgeben);
    ProcEintragen (KstenListeAllesAusgeben);
    ProcEintragen (AlleKstListen);
    ProcEintragen (LfdKstListen);
    ProcEintragen (AktKstListen);
    ProcEintragen (KstenInit);
    ProcEintragen (KstSperren);
    ProcEintragen (KstLiquid);
    ProcEintragen (KstSetzeKonting);
    ProcEintragen (KstSetzeAequiv);
    ProcEintragen (KstBerechneEtat);
    ProcEintragen (KstPruefeKstID);
    ProcEintragen (KstZuKID);
    ProcEintragen (KstenOrdnerSave);
    ProcEintragen (KstenOrdnerLoad);

    ProcEintragen (SystemSetzeDatum);
    MaxProcNr := ProcNr;
  END InitTestProcs;


 PROCEDURE Auswahlmenue () : CARDINAL;
 VAR ItemNr : CARDINAL;

	PROCEDURE MenueItem (inItemText : ARRAY OF CHAR);
	BEGIN
	   WriteCard (ItemNr, 3);
	   WriteString (" --> ");
	   WriteString (inItemText);
	   WriteLn;
           INC (ItemNr);
	END MenueItem;

 BEGIN
   WriteLn;
   WriteString ("======= Testmenue : Lassen Sie es sich schmecken !===============");
   WriteLn;

   ItemNr := 0;
   MenueItem ("Programm beenden");
   MenueItem ("Kostenstellen: StringListe wie spezifiziert ausgeben");
   MenueItem ("Kostenstellen: StringListe alle Ksten-Daten ausgeben");
   MenueItem ("Kostenstellen: ID's aller Kostenstellen auflisten");
   MenueItem ("Kostenstellen: ID's der lfd. Kst (NOT liquidiert AND NOT gesperrt");
   MenueItem ("Kostenstellen: ID's der akt. Kst (NOT liquidiert)");
   MenueItem ("Kostenstellen: Ordner mit Dummy Kostenstellen fllen");
   MenueItem ("Kostenstellen: Kst sperren");
   MenueItem ("Kostenstellen: Kst liquidieren");
   MenueItem ("Kostenstellen: Setze Kontingente");
   MenueItem ("Kostenstellen: Setze Aequivalent");
   MenueItem ("Kostenstellen: Berechne Etat");
   MenueItem ("Kostenstellen: String -> KID pruefen");
   MenueItem ("Kostenstellen: Kst zu KID suchen");
   MenueItem ("Kostenstellen: Ordner auf Datei speichern");
   MenueItem ("Kostenstellen: Ordner aus Datei restaurieren");

   MenueItem ("System       : SystemDatum setzen");
   RETURN IhreWahl()
 END Auswahlmenue;


 PROCEDURE StringAusgeben (inStr : String.TString);
   VAR Test : ARRAY[1..90] OF CHAR;
 BEGIN
   WriteString("[");
   String.Lies (inStr, Test);
   WriteString(Test);
   WriteString("]");
   WriteLn;
 END StringAusgeben;


 PROCEDURE StringEingeben (inoutStr : String.TString);
 VAR Wert : ARRAY [0..90] OF CHAR;
 BEGIN
   ReadString(Wert);
   String.Schreibe (inoutStr, Wert);
   WriteLn;
 END StringEingeben;


 PROCEDURE KompletteListeAusgeben (Testliste : TStringliste);
    VAR AktuellesElement : String.TString;
 BEGIN
    Stringliste.GoStart (Testliste);
    WHILE NOT Stringliste.OffEnd(Testliste) DO
       AktuellesElement := Stringliste.Current(Testliste);
       StringAusgeben (AktuellesElement);
       String.Dispose (AktuellesElement);
       Stringliste.StepNext(Testliste)
    END; (* WHILE *)
    WriteLn;
    WriteString("Aktuelle Position: OffEnd!");
    WriteLn;
 END KompletteListeAusgeben;

BEGIN

  (*                    Programm                                          *)
  (*======================================================================*)

  InitTestProcs;

  REPEAT
    Befehl := Auswahlmenue();
    BefehlAuswerten (Befehl);
  UNTIL Befehl = 0;

  WriteLn; WriteLn;
END KstenTesten.
