(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: VDruck.mod,v $
# $Author: Gabi
# $Date: 2001/02/25
# $Revision: 1.1.1.1 $
# Bemerkungen:
*)
(******************************************************************************)


IMPLEMENTATION MODULE VDruck;

FROM VDruckIMPORTE 	IMPORT	THandleInVDat, TModusInVDat,           (* Typen*)
                     		VOpenFile, VCloseFile, VSetWriteFile,  (* Prozeduren*)
				VWriteLineFile, VSetAppendFile;
FROM String 		IMPORT 	TString, Schreibe, Create, Dispose;
FROM Stringliste	IMPORT	TStringliste, Current, GoStart, StepNext, OffEnd;

(******************************************************************************)

PROCEDURE Print(inText: TString; VAR outMessage: TString):BOOLEAN;
(*
Schreibt den Inhalt des Strings inText auf die Datei erg.txt im aktuellen Verzeichnis.
Existiert die Datei erg.txt noch nicht, wird sie erzeugt, existiert sie bereits, werden
die neuen Strings angehngt.
Im Fehlerfall wird outMessage gefllt und FALSE zurckgegeben, ansonsten TRUE.
*)
VAR	Handle	  : THandleInVDat;
	PrintFile : TString;
	rc, weiter  : BOOLEAN;				(* rc = Returncode der Funktion*)
BEGIN
	rc := FALSE;					(* Initialisierung*)
	weiter := TRUE;					(* zur Prfung, ob beim Dateiffnen ein Fehler aufgetreten ist*)

	PrintFile := Create();
	Schreibe(PrintFile, "erg.txt");      		(* Name der Druckdatei im aktuellen Verzeichnigs als "erg.txt" festlegen*)

	IF NOT VOpenFile(Handle, PrintFile, VSetAppendFile()) THEN	(* Datei erg.txt mit Modus Append ffnen,
									   wenn OpenInVDat = FALSE muss sie erst mit Modus SetWrite erzeugt werden
								 	   wenn Open = TRUE, kann direkt in sie geschrieben werden*)
		IF NOT VOpenFile(Handle, PrintFile, VSetWriteFile()) THEN	(*Fehler beim Erzeugen der Datei*)
			outMessage := Create();
			Schreibe(outMessage, "Print: Fehler beim Erzeugen der Druckdatei.");
			rc := FALSE;
			weiter := FALSE;
		END; (*IF*)
	END; (*IF NOT VOpenFile(Handle, PrintFile, VSetAppendFile())*)
	
        IF weiter THEN							(* kein Fehler aufgetreten, in die Datei kann geschrieben werden*)
		IF NOT VWriteLineFile(Handle, inText, outMessage) THEN  (* Fehler beim Schreiben in Datei aufgetreten*)
			rc := FALSE;                                    (* outMessage wird in WriteLine gesetzt*)
		ELSE IF NOT VCloseFile(Handle) THEN              	(* Fehler beim Dateischlieen*)	
				outMessage := Create();
				Schreibe(outMessage, "Print: Fehler beim Schlieen der Druckdatei.");
				rc := FALSE;
			ELSE
				rc := TRUE;
		     END; (*IF NOT VCloseFile*)		
		END; (*IF NOT VWriteLineFile*)
	END; (*IF NOT rc*)

	Dispose(PrintFile);                             (* lokale Variable freigeben*)
        RETURN rc;
END Print;
(*******************************************************************************************************)

PROCEDURE initDrucker(VAR outMessage: TString): BOOLEAN;
(*
Erzeugt eine leere Datei erg.txt, egal ob die Datei bereits exisitert
oder nicht. Hat die Datei schon exisitert, wird ihr bisheriger Inhalt gelscht.
Im Fehlerfall wird outMessage gesetzt und FALSE zurckgegeben, ansonsten TRUE.
*)
VAR	Handle	  : THandleInVDat;
	PrintFile : TString;
	rc	  : BOOLEAN;				(* rc = Returncode der Funktion*)
BEGIN
	rc := FALSE;					(* Initialisierung*)
	
	PrintFile := Create();
	Schreibe(PrintFile, "erg.txt");      		(* Name der Druckdatei im aktuellen Verzeichnigs als "erg.txt" festlegen*)

	IF NOT VOpenFile(Handle, PrintFile, VSetWriteFile()) THEN	(* Datei erg.txt mit Modus Schreiben geffnet, dh. neu erzeugt*)
		outMessage := Create();
		Schreibe(outMessage, "initDrucker: Fehler beim Erzeugen der Druckdatei.");
		rc := FALSE;
        ELSE
		rc := TRUE;
	END; (*IF NOT VOpenFile(Handle, PrintFile, VSetWriteFile())*)
	
        IF NOT VCloseFile(Handle) THEN              	(* Fehler beim Dateischlieen*)	
		outMessage := Create();
		Schreibe(outMessage, "initDrucker: Fehler beim Schlieen der Druckdatei.");
		rc := FALSE;
	END; (*IF NOT VCloseFile*)		
	
	Dispose(PrintFile);                    		(* lokale Variable freigeben*)
        RETURN rc;
END initDrucker;
(*******************************************************************************************************)

PROCEDURE PrintStringliste(inStringliste: TStringliste; VAR outMessage: TString):BOOLEAN;
(*
Schreibt den Inhalt einer Stringliste auf die Datei erg.txt im aktuellen Verzeichnis.
Existiert die Datei erg.txt noch nicht, wird sie erzeugt, existiert sie bereits, werden
die neuen Strings angehngt.
Im Fehlerfall wird outMessage gefllt und FALSE zurckgegeben, ansonsten TRUE.
*)
VAR	Handle	  : THandleInVDat;
	PrintFile : TString;
	inText	  : TString;
	rc, weiter  : BOOLEAN;				(* rc = Returncode der Funktion*)
BEGIN
	rc := FALSE;					(* Initialisierung*)
	weiter := TRUE;					(* zur Prfung, ob beim Dateiffnen ein Fehler aufgetreten ist*)

	PrintFile := Create();
	Schreibe(PrintFile, "erg.txt");      		(* Name der Druckdatei im aktuellen Verzeichnigs als "erg.txt" festlegen*)

	IF NOT VOpenFile(Handle, PrintFile, VSetAppendFile()) THEN	(* Datei erg.txt mit Modus Append ffnen,
									   wenn OpenInVDat = FALSE muss sie erst mit Modus SetWrite erzeugt werden
								 	   wenn Open = TRUE, kann direkt in sie geschrieben werden*)
		IF NOT VOpenFile(Handle, PrintFile, VSetWriteFile()) THEN	(*Fehler beim Erzeugen der Datei*)
			outMessage := Create();
			Schreibe(outMessage, "PrintStringliste: Fehler beim Erzeugen der Druckdatei.");
			rc := FALSE;
			weiter := FALSE;
		END; (*IF*)
	END; (*IF NOT VOpenFile(Handle, PrintFile, VSetAppendFile())*)
	
        IF weiter THEN							(* kein Fehler aufgetreten, in die Datei kann geschrieben werden*)
		GoStart(inStringliste);                                 (* an Begin der Stringliste gehen*)
		WHILE NOT OffEnd(inStringliste) AND weiter DO			(* solange noch nicht am Ende der Stringliste*)
			inText := Current(inStringliste);
			IF NOT VWriteLineFile(Handle, inText, outMessage) THEN  (* Fehler beim Schreiben in Datei aufgetreten*)
				rc := FALSE;                                    (* outMessage wird in WriteLine gesetzt*)
				weiter := FALSE;		
			END; (*IF NOT VWriteLineFile*)
			StepNext(inStringliste);                    	(* nchster Satz in Stringliste*)
		END; (* WHILE*)
		IF NOT VCloseFile(Handle) THEN              	(* Fehler beim Dateischlieen*)	
			outMessage := Create();
			Schreibe(outMessage, "PrintStringliste: Fehler beim Schlieen der Druckdatei.");
			rc := FALSE;
			weiter := FALSE;
		ELSE
			rc := TRUE;
		END; (*IF NOT VCloseFile*)	
	END; (*IF NOT rc*)

	Dispose(PrintFile);                             (* lokale Variable freigeben*)
        RETURN rc;
END PrintStringliste;

END VDruck.


