(******************************************************************************)
(*                         -> Sopra WS2000/2001 <-                            *)
(*                   Hilfskraftinformations-System 'HiKS'                     *)
(*
# $RCSfile: Vtr.mod,v $
# $Author: Sopra $
# $Date: 2001/03/10 09:48:09 $
# $Revision: 1.1.1.1 $

IMPLEMENTATION MODULE Vtr     ADT Vertrag

Implementation : Herbert Schoenhammer, Herbert.Schoenhammer@Fernuni-Hagen.de
*)
(******************************************************************************)

IMPLEMENTATION MODULE Vtr;   (* ENTHALTEN IN Vertraege TS_Vtre*)


 FROM Storage IMPORT ALLOCATE, DEALLOCATE;
 FROM SYSTEM IMPORT TSIZE;
 FROM GENERIC IMPORT TObjektreferenz;
 FROM WholeStr IMPORT ConvResults, StrToCard;

 FROM VtreIMPORTE IMPORT LiesSystemDatumInSysDat,

                         DuplicatePersID,
                         DisposePersID,
                         IsEqualPersID,
                         EqualizePersID,

                         CreateKstID,
                         DuplicateKstID,
                         DisposeKstID,
                         IsEqualKstID,
                         EqualizeKstID,

                         DisposeHID,
                         EqualizeHID,
                         DuplicateHID,
                         SameSeqNrHID,
                         SmallerSeqNrHID,
                         PruefeHID,
                         HID2Str,
                         Str2HID;


 FROM Datum IMPORT TDatum, CreateDatum, SetzeDatumEternity, IstDatumEternity,
                   DisposeDatum, DuplicateDatum, EqualizeDatum, IsEqualDatum,
                   SmallerDatum, LiesJahrInDatum, TJahr, SmallerJahr, 
                   IsEqualJahr, DisposeJahr, DatumInZeitraum;

 IMPORT String;


 TYPE TVtrDS = RECORD
                  Identifikator : RECORD
                                     ID : TVtrID;
                                     existiert : BOOLEAN;
                                  END;
                  existPersID : BOOLEAN;
                  PersonenID : TPersID;
                  KostenstellenID : TKstID;
                  Stundenzahl : CARDINAL;
                  Taetigkeit : TString;
                  Begruendung : TString;
                  GueltigVon : TDatum;
                  GueltigBis : TDatum;
                  ErstelltAm : TDatum;
                  StorniertAm : TDatum;
               END;

 TYPE TVtr = POINTER TO TVtrDS;

(* Standard-Operationen auf dem Entittstyp Vertrag                        *)
(*=========================================================================*)


PROCEDURE CreateVtr () : TVtr;
VAR aktVtr : TVtr;
BEGIN
    ALLOCATE (aktVtr, TSIZE (TVtrDS));
    WITH aktVtr^ DO
      Identifikator.existiert := FALSE;
      existPersID := FALSE;
      PersonenID := NIL;
      KostenstellenID := CreateKstID ();
      Stundenzahl := 0;
      Taetigkeit := String.Create();
      Begruendung := String.Create ();
      GueltigVon := CreateDatum ();
      GueltigBis := CreateDatum ();
      ErstelltAm := CreateDatum ();
      StorniertAm := CreateDatum ();
    END; (* WITH *)
    RETURN aktVtr
END CreateVtr;


PROCEDURE DisposeVtr (VAR inoutVtr : TVtr);
  (* -- Vorbed.: keine
     -- Entfernt das ADO inoutVtr aus dem System
     -- Nachbed.: ADO inoutVtr existiert nicht mehr *)
BEGIN
    WITH inoutVtr^ DO
 
      IF Identifikator.existiert
       THEN
          DisposeHID (Identifikator.ID)
       END; (* IF *)

       IF existPersID
        THEN
           DisposePersID (PersonenID);
        END; (*IF*)

       DisposeKstID (KostenstellenID);
       String.Dispose (Taetigkeit);
       String.Dispose (Begruendung);
       DisposeDatum (GueltigVon);
       DisposeDatum (GueltigBis);
       DisposeDatum (ErstelltAm);
       DisposeDatum (StorniertAm);       
    END; (* WITH *)
    DEALLOCATE (inoutVtr, TSIZE (TVtrDS));
    inoutVtr := NIL
END DisposeVtr;


PROCEDURE DuplicateVtr (inQuellVtr : TVtr) : TVtr;
  (* -- Vorbed.: keine
     -- Erzeugt ein neues Vertrags-ADO und gleicht dieses an den
     -- Zustand bzw. die Attribute von inVtrObjekt an. Bezeichnet
     -- Duplikat das Rueckgabeobjekt der Operation, so gilt:
     -- Nachbed.: IsEqual(Duplikat, inQuell) AND NOT (Duplikat=inQuellVtr) *)
VAR aktDuplikat : TVtr;
BEGIN
      aktDuplikat := CreateVtr();
      EqualizeVtr (aktDuplikat, inQuellVtr);
      RETURN aktDuplikat;
END DuplicateVtr;


PROCEDURE IsEqualVtr (inVtr1, inVtr2 : TVtr) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Ueberprueft die beiden Objekte auf Zustands-/Wertegleichheit,
     -- d.h. die Gleichheit aller korrespondierenden Attribute.
     -- ID wird mitgeprft.
     -- Nachbed.: keine; *)
VAR gleicheVtrID, gleich : BOOLEAN;
BEGIN

    (* VertragsID *)
    IF inVtr1^.Identifikator.existiert AND inVtr2^.Identifikator.existiert THEN
      gleicheVtrID := SameVertrag (inVtr1, inVtr2)
    ELSIF inVtr1^.Identifikator.existiert OR inVtr2^.Identifikator.existiert THEN
      gleicheVtrID := FALSE
    ELSE
      gleicheVtrID := TRUE
    END; (* IF *)

    IF gleicheVtrID AND IsEqualVtrDaten (inVtr1, inVtr2)
    THEN
      gleich := TRUE;
    ELSE
      gleich := FALSE;
    END;

    RETURN gleich;
 END IsEqualVtr;


PROCEDURE IsEqualVtrDaten (inVtr1, inVtr2 : TVtr) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Ueberprueft die beiden Objekte auf Zustands-/Wertegleichheit,
     -- d.h. die Gleichheit aller korrespondierenden Attribute.
     -- ID wird mitgeprft.
     -- Nachbed.: keine; *)
VAR gleichePersID, gleicheAttribute : BOOLEAN;
BEGIN

    IF inVtr1^.existPersID AND inVtr2^.existPersID THEN
      gleichePersID := IsEqualPersID (inVtr1^.PersonenID, inVtr2^.PersonenID)
    ELSIF inVtr1^.existPersID OR inVtr2^.existPersID THEN
      gleichePersID := FALSE
    ELSE
      gleichePersID := TRUE
    END; (* IF *)

    IF gleichePersID THEN
      gleicheAttribute :=
          IsEqualKstID (inVtr1^.KostenstellenID, inVtr2^.KostenstellenID)
      AND (inVtr1^.Stundenzahl =  inVtr2^.Stundenzahl)
      AND String.IsEqual (inVtr1^.Taetigkeit, inVtr2^.Taetigkeit)
      AND String.IsEqual (inVtr1^.Begruendung, inVtr2^.Begruendung)
      AND IsEqualDatum (inVtr1^.GueltigVon, inVtr2^.GueltigVon)
      AND IsEqualDatum (inVtr1^.GueltigBis, inVtr2^.GueltigBis)
      AND IsEqualDatum (inVtr1^.ErstelltAm, inVtr2^.ErstelltAm)
      AND IsEqualDatum (inVtr1^.StorniertAm, inVtr2^.StorniertAm);
     ELSE
       gleicheAttribute := FALSE
     END; (* IF gleicheID *)

     RETURN gleicheAttribute;
 END IsEqualVtrDaten;


PROCEDURE EqualizeVtr (inoutZielVtr, inQuellVtr : TVtr);
  (* -- Vorbed.: keine
     -- Gleicht den Zustand /die Attribute von inoutZielDA an inQuellDA
     -- an, wobei die Identitaet der ADOs nicht veraendert wird.
     -- Nachbed.: IsEqual (inQuellDA, inoutZielDA) *)
BEGIN

  WITH inoutZielVtr^ DO

   (* VertragsIdentifikatoren angleichen *)
    WITH Identifikator DO
      IF existiert AND inQuellVtr^.Identifikator.existiert THEN
        EqualizeHID (ID, inQuellVtr^.Identifikator.ID)
      ELSIF existiert THEN
        (* -- inQuelleVtr^.Identifikator.existiert = FALSE *)
        existiert := FALSE;
        DisposeHID (ID)
      ELSIF inQuellVtr^.Identifikator.existiert THEN
        (* -- inoutZielVtr^.Identifikator.existiert = FALSE *)
        ID := DuplicateHID (inQuellVtr^.Identifikator.ID);
        existiert := TRUE
      ELSE
        (* -- keine der beiden IDs existiert, es ist nichts zu tun *)
      END (* IF *)
    END; (* WITH Identifikator DO*)

    (* PersonenIdentifikatoren angleichen *)
    IF existPersID AND inQuellVtr^.existPersID THEN
       EqualizePersID (PersonenID, inQuellVtr^.PersonenID)
    ELSIF existPersID THEN
        (* -- inQuelleVtr^.existPersID = FALSE *)
        existPersID := FALSE;
        DisposePersID (PersonenID)
    ELSIF inQuellVtr^.existPersID THEN
        (* -- inoutZielVtr^existPersID = FALSE *)
        PersonenID := DuplicatePersID (inQuellVtr^.PersonenID);
        existPersID := TRUE
    ELSE
        (* -- keine der beiden PersIDs existiert, es ist nichts zu tun *)
    END; (* IF *)

       (* Attribute angleichen *)

       EqualizeKstID (KostenstellenID, inQuellVtr^.KostenstellenID);
       Stundenzahl := inQuellVtr^.Stundenzahl;
       String.Equalize (Taetigkeit, inQuellVtr^.Taetigkeit);
       String.Equalize (Begruendung, inQuellVtr^.Begruendung);
       EqualizeDatum (GueltigVon, inQuellVtr^.GueltigVon);
       EqualizeDatum (GueltigBis, inQuellVtr^.GueltigBis);
       EqualizeDatum (ErstelltAm, inQuellVtr^.ErstelltAm);
       EqualizeDatum (StorniertAm, inQuellVtr^.StorniertAm);
    END; (* WITH inoutZielVtr^ DO*)

END EqualizeVtr;


PROCEDURE SameVertrag (inVtr1, inVtr2 : TVtr) : BOOLEAN;
  (* -- Vorbed.: ExistiertID (inVtr1) AND ExistiertID (inVtr2)
     -- Prueft, ob die beiden inVtr1.ID und inVtr2.ID gleichgross sind.
     -- Bezeichne same den boole`schen Rueckgabeparameter, dann gilt:
     -- Nachbed.: inVtr1 "=" inVtr2 *)
VAR same : BOOLEAN;
BEGIN
    same := SameSeqNrHID (inVtr1^.Identifikator.ID,
                             inVtr2^.Identifikator.ID);
    RETURN same
END SameVertrag;


PROCEDURE SmallerVertrag (inSmallerVtr,
                                 inGreaterVtr : TVtr) : BOOLEAN;
  (* -- Vorbed.: ExistiertID (inSmallerDA) AND ExistiertID (inGreaterDA)
     -- Prueft, ob der Vertrag SmallerVtr1.ID kleiner ist als der
     -- Vertrag GreaterVtr.ID Bezeichne smaller den boole`schen
     -- Rueckgabeparameter, dann gilt:
     -- Nachbed.: smaller <=> SmallerDA "<" GreaterDA; *)
VAR smaller : BOOLEAN;
BEGIN
    smaller := SmallerSeqNrHID (inSmallerVtr^.Identifikator.ID,
                                   inGreaterVtr^.Identifikator.ID);
    RETURN smaller;
END SmallerVertrag;
 

(* Typ-Wandlungen                                                       *)
(*======================================================================*)


PROCEDURE VERTRAG (inRef : TObjektreferenz) : TVtr;
  (* Vb: inRef referenziert ein Hilfskraft-Objekt
         Typtransfer von TObjektreferenz nach THK
     Nb: inRef = OBJEKTREFERENZ(Rueckgabe-HK) *)

VAR aktVtr : TVtr;
BEGIN
    aktVtr := inRef;
    RETURN aktVtr;
END VERTRAG;


PROCEDURE OBJEKTREFERENZ (inVtr : TVtr) : TObjektreferenz;
   (* Vb: keine
          Typtransfer von TVtr nach TObjektreferenz
      Nb: inVtr = Vtr(Rueckgabe-Objektreferenz) *)

VAR VTRRef : TObjektreferenz;
BEGIN
    VTRRef := inVtr;
    RETURN VTRRef;
END OBJEKTREFERENZ;


(* Operationen auf der ID des Entittstyps Vertrag                             *)
(*=============================================================================*)


PROCEDURE ExistiertVtrID (inVtr:TVtr):BOOLEAN;
  (* Vb: keine
         Prueft, ob der Vertrag bereits mit einer VtrID versehen wurde
     Nb: Rueckgabewert = TRUE <=> VtrID existiert *)
  BEGIN
    RETURN inVtr^.Identifikator.existiert;
  END ExistiertVtrID;


PROCEDURE SetzeVtrID (inoutVtr:TVtr; inVtrID : TVtrID);
  (* -- Vb: NOT ExistiertVtrID()
        Versetzt den Vertrag inVtr mit dem Identifikator VtrID
     -- Nb: ExistiertVtrID() *)
  BEGIN
    WITH inoutVtr^.Identifikator DO
      ID := DuplicateHID (inVtrID);
      existiert := TRUE;
    END (* WITH *)
  END SetzeVtrID;


PROCEDURE LiesVtrIDinVtr (inVtr: TVtr) : TVtrID;
  (* -- Vorbed.: ExistiertID (inVtr)
     -- Liefert die VtrID von inVtr
     -- Nachbed.: RueckgabeADO VtrID existiert *)
  BEGIN
    RETURN DuplicateHID (inVtr^.Identifikator.ID)
  END LiesVtrIDinVtr;


PROCEDURE PruefeVtrID (inString : TString; VAR outMessage : TString) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der inString in eine gueltige ID umgewandelt werden kann
     -- Nachbed.: keine *)
  BEGIN
   RETURN PruefeHID (inString, outMessage)
  END PruefeVtrID;


PROCEDURE VtrIDAlsString (inVtrID : TVtrID) :TString;
  (* -- Vorbed.: keine
     -- Liefert die VertragsID des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TString existiert *)
  BEGIN
     RETURN HID2Str (inVtrID, 0);
  END VtrIDAlsString;


PROCEDURE StringInVtrID (inString : TString) :TVtrID;
  (* -- Vorbed.: PruefeVtrID (inString) = TRUE
     -- Konvertiert den bergebenen String in ein neu geschaffenes
     -- Rckgabe ADO vom Typ TVtrID. Vor Aufruf mu inString
     -- auf Konvertierbarkeit mittels PruefeVtrID getestet sein.
     -- Nachbed.: keine *)
  BEGIN
     RETURN Str2HID (inString);
  END StringInVtrID;


(* Operationen auf der PersonenID des Entittstyps Vertrag                     *)
(*=============================================================================*)


PROCEDURE ExistiertPersIDinVtr (inVtr:TVtr):BOOLEAN;
  (* Vb: keine
         Prueft, ob der Vertrag bereits mit einer PersonenID versehen wurde
     Nb: Rueckgabewert = TRUE <=> PersID existiert *)
  BEGIN
    RETURN inVtr^.existPersID;
  END ExistiertPersIDinVtr;


PROCEDURE SetzePersIDinVtr (inoutVtr : TVtr; inPersID : TPersID);
  (* -- Vorbed.: Person (PersID) existiert
     -- Setzt die PersonenID von inoutVtr fest
     -- Nachbed.: inoutVtr.PersonenID = inPersID *)
  VAR vtrPersID : TPersID;
  BEGIN

   IF inoutVtr^.existPersID THEN
      (* inoutVtr besitzt bereits eine PersID => ueberschreiben *)
      EqualizePersID (inoutVtr^.PersonenID, inPersID)
   ELSE
      (* inoutVtr besitzt noch keine PersID => neue erzeugen *)
      vtrPersID := DuplicatePersID (inPersID);
      inoutVtr^.PersonenID := vtrPersID;
      inoutVtr^.existPersID := TRUE
   END; (* IF *)

  END SetzePersIDinVtr;


PROCEDURE LiesPersIDinVtr (inVtr : TVtr) : TPersID;
  (* -- Vorbed.: keine
     -- Liefert die PersonenID des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TPersID existiert *)
  VAR aktPersID : TPersID;
  BEGIN
    aktPersID := DuplicatePersID (inVtr^.PersonenID);
    RETURN aktPersID
  END LiesPersIDinVtr;


(* Operationen auf der Kostenstellen ID des Entittstyps Vertrag             *)
(*=============================================================================*)


PROCEDURE SetzeKstIDinVtr (inoutVtr : TVtr; inKstID : TKstID);
  (* -- Vorbed.: Kostenstelle (KstID) existiert
     -- Setzt die KostenstellenID von inoutVtr fest
     -- Nachbed.: inoutVtr.KostenstellenID = inKstID *)
  BEGIN
    EqualizeKstID (inoutVtr^.KostenstellenID, inKstID);
  END SetzeKstIDinVtr;


PROCEDURE LiesKstIDinVtr (inVtr : TVtr ) : TKstID;
   (* -- Vorbed.: keine
      -- Liefert die KostenstellenID des Vetrages inVtr
      -- Nachbed.: RueckgabeADO TKstID existiert *)
  VAR aktKstID : TKstID;
  BEGIN
    aktKstID := DuplicateKstID (inVtr^.KostenstellenID);
    RETURN aktKstID
  END LiesKstIDinVtr;


(* Attribute des Entittstyps Vertrag                                          *)
(*=============================================================================*)


PROCEDURE SetzeGueltigVoninVtr (inoutVtr : TVtr; inDatum : TDatum);
  (* -- Vorbed.: keine
     -- Setzt das Anfangsdatum inGueltigVon von inoutVtr fest
     -- Nachbed.: inoutVtr.GueltigVon = inDatum *)
  BEGIN
    EqualizeDatum (inoutVtr^.GueltigVon, inDatum);
  END SetzeGueltigVoninVtr;


PROCEDURE LiesGueltigVoninVtr (inVtr: TVtr) : TDatum;
   (* -- Vorbed.: keine
      -- Liefert das Anfangsdatum des Vetrages inVtr
      -- Nachbed.: RueckgabeADO TDatum existiert *)
  VAR aktDatum : TDatum;
  BEGIN
   aktDatum := DuplicateDatum(inVtr^.GueltigVon);
   RETURN aktDatum
  END LiesGueltigVoninVtr;


PROCEDURE SetzeGueltigBisinVtr (inoutVtr: TVtr; inDatum : TDatum);
  (* -- Vorbed.: keine
     -- Setzt das Endedatum inGueltigBis von inoutVtr fest
     -- Nachbed.: inoutVtr.GueltigBis = inDatum *)
  BEGIN
    EqualizeDatum (inoutVtr^.GueltigBis, inDatum);
  END SetzeGueltigBisinVtr;


PROCEDURE LiesGueltigBisinVtr (inVtr : TVtr) : TDatum;
  (* -- Vorbed.: keine
     -- Liefert das Endedatum des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TDatum existiert *)
  VAR aktDatum : TDatum;
  BEGIN
   aktDatum := DuplicateDatum(inVtr^.GueltigBis);
   RETURN aktDatum
  END LiesGueltigBisinVtr;


PROCEDURE SetzeErstellDatinVtr (inoutVtr : TVtr;
                                  inErstellDatum : TDatum) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Setzt das Erstellungsdatum von inoutVtr fest
     -- Nachbed.: inoutVtr.ErstelltAm = inErstellDatum *)
  BEGIN
    EqualizeDatum (inoutVtr^.ErstelltAm, inErstellDatum);
    RETURN TRUE
  END SetzeErstellDatinVtr;


PROCEDURE LiesErstellDatinVtr (inVtr : TVtr) : TDatum;
  (* -- Vorbed.: keine
     -- Liefert das Erstellungsdatum des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TDatum existiert *)
  VAR aktDatum : TDatum;
  BEGIN
   aktDatum := DuplicateDatum(inVtr^.ErstelltAm);
   RETURN aktDatum
  END LiesErstellDatinVtr;


PROCEDURE SetzeStornoAminVtr (inoutVtr : TVtr;
                                inStorniertAm : TDatum) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Setzt das Stornierungsdatum von inoutVtr fest
     -- Nachbed.: inoutVtr.StorniertAm = inStorniertAm *)
  BEGIN
     EqualizeDatum (inoutVtr^.StorniertAm, inStorniertAm);
     RETURN TRUE
  END SetzeStornoAminVtr;


PROCEDURE LiesStornoAminVtr (inVtr : TVtr) : TDatum;
  (* -- Vorbed.: keine
     -- Liefert das Stornierungsdatum des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TDatum existiert *)
  VAR aktDatum : TDatum;
  BEGIN
   aktDatum := DuplicateDatum(inVtr^.StorniertAm);
   RETURN aktDatum
  END LiesStornoAminVtr;


PROCEDURE IstStornoInVtr (inVtr : TVtr) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob das Stornodatum von inVtr gesetzt ist.
     -- Sei storniert der boolesche Rueckgabewert, dann gilt:
     -- Nachbed.: (storniert <=> inVtr.StornoAm "<>" Eternity)
                  (NOT storniert <=> inVtr.StornoAm "=" Eternity *)
  BEGIN
     RETURN NOT IstDatumEternity (inVtr^.StorniertAm)
  END IstStornoInVtr;


PROCEDURE SetzeStdzahlinVtr (inoutVtr: TVtr;
                               inStdzahl : CARDINAL) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Setzt die Stundenzahl von inoutVtr fest
     -- Nachbed.: inoutVtr.Stundenzahl = inStdzahl *)
  VAR egal : BOOLEAN;
  BEGIN
    inoutVtr^.Stundenzahl := inStdzahl;
    egal := TRUE;
    RETURN egal
  END SetzeStdzahlinVtr;


PROCEDURE LiesStdzahlinVtr (inVtr: TVtr) : CARDINAL;
  (* -- Vorbed.: keine
     -- Liefert die Stundenzahl des Vetrages inVtr
     -- Nachbed.: keine *)
  BEGIN
    RETURN inVtr^.Stundenzahl
  END LiesStdzahlinVtr;


PROCEDURE SetzeTaetigktinVtr (inoutVtr: TVtr;
                                inTaetigkt : TString) : BOOLEAN;
  (* -- Vorbed.: keine
     -- Setzt die Ttigkeit von inoutVtr fest
     -- Nachbed.: inoutVtr.Taetigkeit = inTaetigkeit *)
  BEGIN
    String.Equalize (inoutVtr^.Taetigkeit, inTaetigkt);
    RETURN TRUE
  END SetzeTaetigktinVtr;


PROCEDURE LiesTaetigktinVtr (inVtr : TVtr) : TString;
  (* -- Vorbed.: keine
     -- Liefert die Taetigkeit des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TString existiert *)
  VAR aktText : TString;
  BEGIN
    aktText := String.Duplicate (inVtr^.Taetigkeit);
    RETURN aktText
  END LiesTaetigktinVtr;


PROCEDURE SetzeBegruendinVtr (inoutVtr: TVtr;
                                inBegruend : TString) : BOOLEAN ;
  (* -- Vorbed.: keine
     -- Setzt die Begruendung von inoutVtr fest
     -- Nachbed.: inoutVtr.Begruendung = inBegruend *)
  BEGIN
    String.Equalize (inoutVtr^.Begruendung, inBegruend);
    RETURN TRUE
  END SetzeBegruendinVtr;


PROCEDURE LiesBegruendinVtr (inVtr: TVtr) : TString;
  (* -- Vorbed.: keine
     -- Liefert die Begruendung des Vetrages inVtr
     -- Nachbed.: RueckgabeADO TString existiert *)
  VAR aktText : TString;
  BEGIN
    aktText := String.Duplicate (inVtr^.Begruendung);
    RETURN aktText
  END LiesBegruendinVtr;


(* Datumsoperationen                                                      *)
(**************************************************************************)


PROCEDURE IstAbgelaufenVtr(inVtr: TVtr): BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der Vertrag bereits abgelaufen ist.
     -- Bezeichne abgelaufen den booleschen Rueckgabewert, dann gilt:
     -- Nachbed.: (abgelaufen <=> inVtr.GueltigBis < Heute) *)
  VAR heute : TDatum;
      istAbgelaufen : BOOLEAN;
  BEGIN
    heute := LiesSystemDatumInSysDat();
    istAbgelaufen := SmallerDatum (inVtr^.GueltigBis, heute);
    DisposeDatum (heute);
    RETURN istAbgelaufen;
  END IstAbgelaufenVtr;


PROCEDURE BeginntNotImLfdJahr(inVtr: TVtr): BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob das Jahr des Vertragsbeginns ein zukuenftiges Jahr ist.
     -- Bezeichne notimlfdjahr den booleschen Rueckgabewert, dann gilt:
     -- (notimlfdjahr <=> GueltigVon.Jahr <> Heute.Jahr) *)
  VAR heute : TDatum;
      heuteTJahr, gueltigVonTJahr : TJahr;
      notimlfdjahr : BOOLEAN;
  BEGIN
    heute := LiesSystemDatumInSysDat();
    heuteTJahr := LiesJahrInDatum (heute);
    gueltigVonTJahr := LiesJahrInDatum (inVtr^.GueltigVon);
    notimlfdjahr := NOT IsEqualJahr (heuteTJahr, gueltigVonTJahr);
    DisposeDatum (heute);
    DisposeJahr (heuteTJahr);
    RETURN notimlfdjahr
  END BeginntNotImLfdJahr;


PROCEDURE IstNochNichtLfdVtr(inVtr: TVtr): BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der Vertragsbeginn in der Zukunft liegt.
     -- Bezeichne nochnichtlfd den booleschen Rueckgabewert, dann gilt:
     -- Nachbed.: (nochnichtlfd <=> Gueltig.Bis > HEUTE)	*)
  VAR heute : TDatum;
      nochnichtlfd : BOOLEAN;
  BEGIN
    heute := LiesSystemDatumInSysDat();
    nochnichtlfd := SmallerDatum (heute, inVtr^.GueltigVon);
    DisposeDatum (heute);
    RETURN nochnichtlfd    
  END IstNochNichtLfdVtr;


PROCEDURE IstLfdVtr(inVtr: TVtr; inEndeNeu:TDatum): BOOLEAN;
  (* -- Vorbed.: keine
     -- Prueft, ob der Vertrag laufend ist, d.h.
     -- inVtr.Gueltig.VON <= inEndeNeue <= inVtr.GueltigBis dann TRUE, sonst FALSE
     -- Bezeichne istlfd den booleschen Rueckgabewert, dann gilt:
     -- Nachbed.: (istlfd <=> inVtr.Gueltig.VON < inEndeNeue < inVtr.GueltigBis) *)
  VAR istlfd : BOOLEAN;
  BEGIN
    istlfd :=  DatumInZeitraum (inEndeNeu, inVtr^.GueltigVon, inVtr^.GueltigBis);
    RETURN istlfd
  END IstLfdVtr;


END Vtr.
